<?php
/**
 * Facebook functions
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * Bookmark structure (stored in "bookmark" resource data)
 *  array(4) {
    ["id"]=> string: unique id of item
    ["name"]=> string: UTF8 label
    ["type"]=> string: b=bookmark, f=folder, q=end of folder
    ["url"]=>  string: URL (bookmarks only)
 *
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 2.2.0
 */
define('FACEBOOK_WEEZO_APP_ID','126424020722760');

define('FACEBOOK_COMMENTS_URL','weezo.net/facebook/'); // Facebook vertical comments panel URL (http/https is later added)
define('FACEBOOK_CONNECT_REDIRECT_URI','https://weezo.net/facebook/connect/');

define('FACEBOOK_CONNECT_URI','https://graph.facebook.com/oauth/authorize');
define('FACEBOOK_CONNECT_SCOPE','offline_access,read_stream,publish_stream');
define('FACEBOOK_LOGIN_SCOPE','');

define('FACEBOOK_GRAPH_URI','https://graph.facebook.com/');
define('FACEBOOK_OLDRESTAPI_URI','https://api.facebook.com/method/');

define('FACEBOOK_COMMENTS_UPDATE_INTERVAL',3); // Min auto update interval for displayed comments in facebookCommentsBox, in seconds

/**
 * @desc Perform request(s) to facebook server
 *
 * @param array $reqs: array of id=>url of requests or single url
 */
function facebookRequests($reqs){
	if(!is_array($reqs)) {
		$singleRequest=1;
		$reqs=array('r1'=>$reqs);
	}
	if(defined('FACEBOOK_REQ_PROXY')){
		foreach ($reqs as $k=>$url) $reqs[$k]=FACEBOOK_REQ_PROXY.'?url='.base64_encode($url);
	}
	$res=cfCURLMultiRequests($reqs);

	if(isset($singleRequest)) return $res['r1']; else return $res;
}

/**
 * @desc Return facebook account data or false if no account is defined
 *
 * @return array:
 * 			access_token: token used for Facebook API access
 * 			id: account id
 * 			name: account name
 * 			icon: account base 64 encoded icon
 */
function facebookAccountGet($resetStatic=false){
	static $facebookAccount;
	if($resetStatic) $facebookAccount=null;
	//return array('access_token'=>'126424020722760|d45a98ee3d816cc2673b5bb5-1814038210|cFdS9tI8Y7_FAhZ3xo_j_KPAzts.')
	if($facebookAccount===false || $facebookAccount) return $facebookAccount;

	if(!($facebookAccount=cfGGetVar('facebookAccount'))) {
		$facebookAccount=false;
	}
	else{
		$facebookAccount=@unserialize(cfStoredPasswordDecrypt($facebookAccount));
		if(!isset($facebookAccount['access_token'])||!isset($facebookAccount['id'])) return false;
	}
	return $facebookAccount;
}

/**
 * @desc Retreive user information from Facebook and save it
 *
 * @param string $access_token
 * @return boolean result
 */
function facebookAccountCreate($access_token){
	// Retreive profile from Facebook through an API call
	$userProfile=facebookAPIRequest('me',false,false,false,$access_token);
	if(!$userProfile) return false;

	$facebookAccount=array(
		'id'=>$userProfile['id'], // Facebook user id
		'name'=>$userProfile['name'], // Facebook user full name
		'send_notification'=>false, // Send notifications when an user makes a Facebook comment
		'access_token'=>$access_token // Access token used for APIs
	);
	cfGUpdateVar('facebookAccount',cfStoredPasswordCrypt(serialize($facebookAccount)),true,true);

	// Retreive and cache facebook picture
	$picture=facebookAPIRequest('me','picture',false,false,$access_token,false);
	if($picture) file_put_contents(facebookProfilePictureFile(),$picture);

	return true;
}

/**
 * @desc Update a part of a facebook account, and save
 *
 * @param string $key
 * @param mixed $value
 * @return array new facebook account or false if failure
 */
function facebookAccountUpdate($key,$value){
	if(!($facebookAccount=facebookAccountGet())) return false;
	$facebookAccount[$key]=$value;
	cfGUpdateVar('facebookAccount',cfStoredPasswordCrypt(serialize($facebookAccount)),true,true);
	return $facebookAccount;
}

/**
 * @desc Remove Facebook account and revoke acces_token
 *
 */
function facebookAccountRevoke(){
	// Revoke access token
	facebookOLDRESTAPIRequest('auth.revokeAuthorization');

	// Clear local data
	cfGUpdateVar('facebookAccount','',true,true);
	@unlink(facebookProfilePictureFile());
}


/**
 * @desc Return Facebook Graph API request's URL
 *
 * @param string $objectId: id of object to request
 * @param string $connection_type: facebook connection type (friends, feed, like...)
 * @param mixed $extra_GET: extra parameters passed to request (array or string)
 * @param string $access_token: access token to use. If not set, use access token from Facebook profile
 * @return string
 */
function facebookAPIRequestURL($objectId,$connection_type=false,$extra_GET=false,$access_token=false){
	if(!$access_token){
		if(!($access_token=cfArrayItem(facebookAccountGet(),'access_token'))) return false;
	}
	$url=FACEBOOK_GRAPH_URI.$objectId.'/'.(($connection_type)?$connection_type.'/':'').'?access_token='.$access_token;
	if($extra_GET){
		if(is_array($extra_GET)) {
			foreach ($extra_GET as $k=>$v) $url.='&'.$k.'='.$v;
		}
		else $url.=((substr($extra_GET,0,1)=='&')?'':'&').$extra_GET;
	}
	return $url;
}

/**
 * @desc Perform a Facebook Graph API Request (see http://developers.facebook.com/docs/api)
 *
 * @param string $objectId: id of object to request
 * @param string $connection_type: facebook connection type (friends, feed, like...)
 * @param mixed $extra_GET: extra parameters passed to request (array or string)
 * @param string $access_token: access token to use. If not set, use access token from Facebook profile
 * @param boolean $JSONDecode: true to json_decode result, false to get raw result
 * @return array (false if failure)
 */
function facebookAPIRequest($objectId,$connection_type=false,$extra_GET=false,$extra_POST=array(),$access_token=false,$JSONDecode=true){
	//$res=cfCURLMultiRequests(array('r1'=>facebookAPIRequestURL($objectId,$connection_type,$extra_GET,$access_token)));

	$url=facebookAPIRequestURL($objectId,$connection_type,$extra_GET,$access_token);

	// POST arguments: directly use CURL as facebookRequests function doesn't support POST data (so far...)
	if($extra_POST){
		$ch=curl_init($url);
		$options=array(
			CURLOPT_FAILONERROR=>1,
			CURLOPT_HEADER=>0,
			CURLOPT_RETURNTRANSFER=>1,
			CURLOPT_FOLLOWLOCATION=>0,
			CURLOPT_MAXREDIRS=>0,
			CURLOPT_FRESH_CONNECT=>1,
			CURLOPT_TIMEOUT=>7,
			CURLOPT_RETURNTRANSFER=>1,
			CURLOPT_POST=>1,
			CURLOPT_POSTFIELDS=>$extra_POST,
			CURLOPT_SSL_VERIFYHOST => 0,
			CURLOPT_SSL_VERIFYPEER => false
		);
		curl_setopt_array($ch, $options);
		$res=curl_exec($ch);
		curl_close($ch);
	}

	// No POST arguments
	else{
		$res=facebookRequests($url);
	}

	if($JSONDecode)	{
		require_once(INCLUDE_DIR.'JSON.php');
		$json=new Services_JSON(SERVICES_JSON_LOOSE_TYPE);
		return $json->decode($res);
	}
	else return $res;
}


/**
 * @desc Facebook API query using OLD REST API (see see http://developers.facebook.com/docs/reference/rest/)
 *
 * @param string $method_name
 * @param array $args: see http://developers.facebook.com/docs/reference/rest/
 * @param string $access_token: access token to use. If not set, use access token from Facebook profile
 * @return array parsed result
 */
function facebookOLDRESTAPIRequest($method_name,$args=array(),$access_token=false){
	// Retreive access token
	if(!$access_token){
		if(!($access_token=cfArrayItem(facebookAccountGet(),'access_token'))) return false;
	}
	// Build base URL
	$url=FACEBOOK_OLDRESTAPI_URI.$method_name.'?access_token='.$access_token;
	// Add args
	if(is_array($args))	foreach ($args as $k=>$v) $url.='&'.urlencode($k).'='.urlencode($v);
	else $url.=((substr($args,0,1)=='&')?'':'&').$args;
	// Request JSON formated result
	$url.='&format=JSON';

	// Send request
	//$res=cfSocketHTTPRequest($url);
	$res=facebookRequests($url);

	if(!defined('SERVICES_JSON_SLICE')) require_once(INCLUDE_DIR.'JSON.php'); // Include JSON functions (note: there is an include bug, so manualy check if already included)
	$json=new Services_JSON(SERVICES_JSON_LOOSE_TYPE);
	return $json->decode($res);
}

/**
 * @desc Facebook FQL request (see http://developers.facebook.com/docs/reference/fql/)
 *
 * @param string $query
 * @param array $args: see http://developers.facebook.com/docs/reference/rest/
 * @param string $access_token: access token to use. If not set, use access token from Facebook profile
 * @return array parsed result
 */
function facebookFQLRequest($query,$args=array(),$access_token=false){
	$args['query']=$query;
	return facebookOLDRESTAPIRequest('fql.query',$args,$access_token);
}

/**
 * @desc Return facebook profile cached picture location
 *
 * @return string
 */
function facebookProfilePictureFile(){
	return cfAppDataDir().'/facebookPicture.jpg';
}


/**
 * @desc Display a Facebook profile box, process async commands
 * @param bool $inResourceConfigScript: set to true to indicate the profile box is displayed in a resource configuration script
 *
 */
function facebookProfileBox($inResourceConfigScript=false){
	// Facebook profile box async commands
	if(cfIsAsync()) {
		// Create Facebook account with passed access_token
		if (isset($_POST['facebookAccountCreate'])) {
			$_POST['facebookAccountCreate']=str_replace('access_token=','',$_POST['facebookAccountCreate']);
			$ok=facebookAccountCreate($_POST['facebookAccountCreate']);

			if($inResourceConfigScript) return rcAddAsyncResponse(cfAsyncJSaction('wl.goURL()'));

			cfAsyncHeader();
			if($ok) echo cfAsyncJSaction('wl.goURL()');
			die(cfAsyncFooter());
		}
		// Revoke Facebook account
		if (isset($_POST['facebookAccountRevoke'])) {
			// Revoke account
			facebookAccountRevoke();

			// Reload page
			if($inResourceConfigScript) return rcAddAsyncResponse(cfAsyncJSaction('wl.goURL()'));
			cfAsyncHeader();
			echo cfAsyncJSaction('wl.goURL()');
			die(cfAsyncFooter());
		}
		// Async command is not for facebookProfileBox => proceed
		return;
	}

	// Facebook profile box picture
	if(isset($_GET['facebookProfilePicture'])){
		// Cached file doesn't exist: send no-profile picture
		if(!file_exists(facebookProfilePictureFile())) {
			WHeaders::contentType('gif');
			die(file_get_contents(cfAppDocRoot().outIcon('facebookNoProfile.gif')));
		}
		// Send cached file
		else{
			WHeaders::contentType('jpg');
			die(file_get_contents(facebookProfilePictureFile()));
		}
	}

	$profile=facebookAccountGet();

?>
<div class="frame2Header" onclick="if(W.uiCommunity) uiCommunity.profilesToggle(1)">
<?php echo outImageIcon('facebook');?>Facebook - <?php echo cfCaption('blogProfile');?></div>
<?php
	if(!$profile) echo '<div style="position:absolute;top:0;right:2px">'.outBt(cfCaption('genConfigure'),false,'javascript:if(uiCommunity.profilesHidden) uiCommunity.profilesToggle();WFacebook.connect()',false,false,false,'up','150').'</div>';
	// Foldable div
	echo '<div id="facebookProfileDetail" style="display:'.((cfGGetVar('uiCommunityHideProfiles'))?'none':'block').'">';

	// Facebook profile defined
	if($profile){
		echo '<img src="'.$_SERVER['REQUEST_URI'].(((strpos($_SERVER['REQUEST_URI'],'?')!==false)?'&':'?').'facebookProfilePicture').'" style="width:50px;height:50px;float:left;margin:5px 10px 0 0" alt="FB profile picture">';
		echo '<div style="font-weight:bold">'.$profile['name'].'</div><br>';
		echo outBt(cfCaption('genClear'),outIcon('eraser'),'javascript:WFacebook.revoke()');
	}
	// Facebook profile not defined
	else{
		echo outImage(outIcon('facebookNoProfile'),false,false,'float:left;margin:2px 10px 3px 0');
		echo cfCaption('facebookAccountConfigExpl').'<br>';

	}
?>
<br clear="all">
</div>
<script type="text/javascript">
function facebookAccountCreate(access_token){
	wl.asr.send({'facebookAccountCreate':access_token})
}
</script>
<?php
	// Insert facebook connect script
	facebookConnectScript('connect','facebookAccountCreate');
?>
<?php
}

/**
 * @desc Script for login/connecting to a Facebook account
 *
 * @param string $type: "login" for "friends users" login to site, "connect" for site owner profile definition
 * @param string $jsCallbackFunction: name of javascript function called with access_token as parameter
 */
function facebookConnectScript($type,$jsCallbackFunction){
	// Include current URL into request so user can be redirected to this very URL with a hash part containing response
	$redirectURI=urlencode(FACEBOOK_CONNECT_REDIRECT_URI).'?serverURL=';

	// If in app, don't send this parameter as it would need to include weezoKey
	if(cfIsInApp()) $redirectURI=urlencode(FACEBOOK_CONNECT_REDIRECT_URI);

	// Set connect scope (if simple user login, don't request offline_access and publish_stream)
	$scope=($type==='connect')?FACEBOOK_CONNECT_SCOPE:FACEBOOK_LOGIN_SCOPE;

	// Build connect URL
	$url=FACEBOOK_CONNECT_URI.'?client_id='.FACEBOOK_WEEZO_APP_ID.(($scope)?'&scope='.$scope:'').'&redirect_uri='.$redirectURI;
	// If mobile version, include display=touch parameter so facebook display mobile page
	if(cfIsMobile()) $url=FACEBOOK_CONNECT_URI.'?display=touch&client_id='.FACEBOOK_WEEZO_APP_ID.(($scope)?'&scope='.$scope:'').'&redirect_uri='.$redirectURI;
	else $url=FACEBOOK_CONNECT_URI.'?client_id='.FACEBOOK_WEEZO_APP_ID.(($scope)?'&scope='.$scope:'').'&redirect_uri='.$redirectURI;

	/**
	 * Mobile redirect
	 */
	if(cfIsMobile()){
?>
<script type="text/javascript">
var WFacebook = {
	'connect':function(){
		var url="<?php echo $url; ?>",t;
		url+=D.location.href.replace(/\//g,'W-S').replace(/\?/g,'W-Q').replace(/\&/g,'W-A').replace(/\:/g,'W-P').replace(/=/g,'W-E')+'W-AmobileW-E1';
		D.location.href=url;
	}
}
</script>
<?php
	}

	/**
	 * Desktop popup
	 */
	else{
?>
<script type="text/javascript">
var WFacebook = {
	'connectProceed':function(response){
		if(response.indexOf('access_token=')==-1) return;
		response=response.substr(response.indexOf('access_token=')+13);
		<?php echo $jsCallbackFunction;?>(response);
	},
	'connect':function(){
		var url="<?php echo $url; ?>",p=url.indexOf('?serverURL='),t;
		if(p!=-1) url+=D.location.href.replace(/\//g,'W-S').replace(/\?/g,'W-Q').replace(/\&/g,'W-A').replace(/\:/g,'W-P').replace(/=/g,'W-E');
		wl.crossDomainPopup(url,"Facebook",{"width":820,"height":500},WFacebook.connectProceed);
	},
	'revoke':function(){wl.asr.send('facebookAccountRevoke=1')}
}
</script>
<?php
	}
}


/**
 * @desc Return Facebook comments XID for users or for a given resource
 *
 * @param string $resourceId: resource ID. If not set, return XID for users
 * @return string
 */
function facebookCommentsXID($resourceId=false){
	if(!($reg=cfRegistrationData())) return false;

	// Facebook comments secret
	if(!$facebookCommentsSecret=cfGGetVar('facebookCommentsSecret')){
		$facebookCommentsSecret=cfGenerateID(10);
		cfGUpdateVar('facebookCommentsSecret',$facebookCommentsSecret,true);
	}

	// Comments group XID component
	if($resourceId) $comp='/res/'.$resourceId;
	else $comp='/user/0';// .cfUTF8Encode(cfUGetVar('name'));: no: same comments for every users

	// Compute facebook comments XID
	$xid=str_replace('=','-',str_replace('+','.',str_replace('/','_',base64_encode($reg['regName'].'/'.$facebookCommentsSecret.$comp))));

	return $xid;
}

/**
 * @desc Return facebook url to facebook-displayed comments
 *
 * @param string $xid
 * @param integer $displayWidth
 */
function facebookCommentsPanelURL($xid,$displayWidth=false){
	$baseURL=((@$_SERVER['HTTPS']==='on')?'https':'http').'://'.FACEBOOK_COMMENTS_URL;

	if(!$displayWidth) $displayWidth=COMMENTS_BANNER_WIDTH;

	$url=$baseURL.'inlineComments.php?xid='.$xid.'&displayWidth='.$displayWidth.'&lng='.cfGGetVar('language').'&redirect=1';
	$url.='&theme='.basename(cfAppThemeDir());

	if(($facebookAccount=facebookAccountGet()) && @$facebookAccount['send_notification']) $url.='&send_notification_uid='.$facebookAccount['id'];

	return $url;
}

/**
 * @desc Insert code for facebook comments vertical panel
 *
 */
function facebookCommentsPanel(){

	// Check if comments are displayed for this user/resource
	if(cfIsResource() && !cfRGetVar('comments')) return;
	elseif(!cfIsResource() && !cfUGetVar('comments')) return;
	if(!cfRegistrationData()) return;

	// Default comments state
	$state=(cfIsResource())?cfRGetVar('commentsPanelSate'):cfUGetVar('commentsPanelSate');

	// Comments group XID component
	$xid=facebookCommentsXID((cfIsResource())?cfRGetVar('id'):false);
	$url=facebookCommentsPanelURL($xid,COMMENTS_BANNER_WIDTH-5);
?>
<div id="commentsDiv" class="vPanel" style="right:-<?php echo COMMENTS_BANNER_WIDTH?>px">
	<div id="commentsHandle" class="handleLeft" onclick="wComments.toggle()"></div>
	<div id="commentsHandle2" class="commentsHandle2">
		<div class="commentsHandleUF" onclick="stopPropagation(event);wComments.toggle(1)"<?php echo outToolTip(cfCaptionJS('blogCommentsShow'));?>></div>
		<div class="commentsHandleF" onclick="stopPropagation(event);wComments.toggle(-1)"<?php echo outToolTip(cfCaptionJS('blogCommentsHide'));?>></div>
	</div>
	<iframe id="commentsIFrame" name="commentsIFrame" src="<?php echo $url;?>" onload="if(this.src) this.style.visible=true" frameborder="0"></iframe>
</div>
<script type="text/javascript">
var wComments=WComments();
wComments.adjustSize(<?php echo COMMENTS_BANNER_WIDTH;?>);
<?php
	// Unfold panel if state=1 or 2
	if($state) echo 'connect(W,"load",wComments.toggle);';
?>
</script>
<?php
}

/**
 * @desc Get cached comments data
 *
 * @return array $commentsData: list of all comments done on the weezo personal space + cached id<=>user name of users who made comments + timestamps (see below)
 */
function facebookCommentsCacheGet(){
	if(is_file(cfAppDataDir().'/facebookComments.txt')){
		$commentsData=unserialize(file_get_contents(cfAppDataDir().'/facebookComments.txt'));
	}
	else{
		$commentsData=array(
			'updateTime'=>0, // timestamp of last update
			'viewTime'=>0, // timestamp of last view
			'xidFolding'=>array(), // xid=>bool folded/unfolded array
			'comments'=>array(), // array of comment items: id,username,fromid,time,text,xid
			'profiles'=>array() // cache of profile id=>array('name'=>name) of facebook users
		);
		facebookCommentsCacheSave($commentsData);
	}
	return $commentsData;
}

/**
 * @desc Save comments data config
 *
 * @param array $commentsData
 */
function facebookCommentsCacheSave($commentsData){
	file_put_contents(cfAppDataDir().'/facebookComments.txt',serialize($commentsData));
}

/**
 * @desc Display a box with facebook comments made on the site / on the resources
 *
 */
function facebookCommentsBox(){
	/**
	 * Facebook profile not defined => cannot display comments (only sync display)
	 */
	if(!facebookAccountGet() && !cfIsAsync()){
?>
<div class="frame2Header"><?php echo outImageIcon('comments','id="facebookCommentsBoxIcon"').cfCaption('genComments');?></div>
<center style="position:absolute;top:50%;margin-top:-15px;width:98%"><div class="extendedWarning" style="width:50%"><br><br><?php echo  outImageIcon('info2').cfCaption('facebookAccountConfigWarning');?><br><br>&nbsp;</div></center></div>
<?php
		return;
	}

	// Only process relevant async requests
	if(cfIsAsync() && !isset($_POST['facebookCommentsBoxUpdate']) && !isset($_POST['facebookCommentsChangeFolding'])) return;


	// Retreive cached comments and profiles data
	$commentsData=facebookCommentsCacheGet();


	// Get updated XIDs list
	if(0) $resConfig=new WResConfig(); // Zend IDE class detection WA
	$xids=array(0=>array('xid'=>facebookCommentsXID(false),'resConfig'=>false));
	// Compute xid for each comments-enabled resource
	foreach (WEnv::resConfigs() as $id=>$resConfig){
		if($resConfig->getVar('comments')) $xids[$id]=array('xid'=>facebookCommentsXID($id),'resConfig'=>$resConfig);
	}

	// Store tree nodes folding states
	if(isset($_POST['facebookCommentsChangeFolding'])){
		// Check passed xid exist
		foreach ($xids as $v) if($v['xid']==$_POST['facebookCommentsChangeFolding']) {
			$commentsData['xidFolding'][$_POST['facebookCommentsChangeFolding']]=($_POST['isFolded'])?1:0;
			facebookCommentsCacheSave($commentsData);
			break;
		}
        cfAsyncHeader();
        cfAsyncFooter();
	}


	/**
	 * Update data from Facebook servers
	 */
	if(cfIsAsync() && isset($_POST['facebookCommentsBoxUpdate'])){

		// Create Facebook FQL query that retreive time-sorted relevant data for xid's
		$query='SELECT id,username,fromid,time,text,xid from comment where ';
		if($commentsData['updateTime']>0) $query.= 'time > '.$commentsData['updateTime'].' AND ';
		// Add all xid to query
		foreach ($xids as $xid)	$query.=((@$nb++>0)?' OR ':'(').'xid="'.$xid['xid'].'"';
		$query.=') ORDER BY time';
		// Run query
		$newComments=facebookFQLRequest($query);

		// Update last data retreival time
		$commentsData['updateTime']=time();

		// If query is successfull, add new comments to saved results
		if($newComments){

			// Array of unknown facebook user ids
			$unknownFromid=array();

			// Add comments to already-stored list
			foreach ($newComments as $comment) array_unshift($commentsData['comments'],$comment);

			// If this comment has no username (i.e. has not been posted by a non-facebook user), and is not in known users list,
			// add to list of user which profiles should be retreived
			if(@$newComments['error_code']){
				$updateError=@cfUTF8Encode($newComments['error_msg']);
			}
			else{
				foreach ($newComments as $comment){
					if(!@$comment['username'] && !isset($commentsData['profiles'][@$comment['fromid']])) $unknownFromid[]=@$comment['fromid'];
				}
			}
			// If any profile info should be retreived
			if(count($unknownFromid)){
				$query='select id,name from profile where ';
				$nb=0;
				foreach ($unknownFromid as $fromid) $query.=(($nb++)?' OR ':'').'id="'.$fromid.'"';
				$usersInfo=facebookFQLRequest($query);

				// Store it into the "profiles" section of comments data
				foreach ($usersInfo as $userInfo) $commentsData['profiles'][$userInfo['id']]=$userInfo['name'];
			}
			//file_put_contents(facebookCommentsFile(),serialize($commentsData));
			$asyncUpdated=true;
		}
	}

	// Comments tree top node
	$tree=array('ico'=>outIcon('comments'),'path'=>'top','label'=>'','sub'=>array());

	// Display comments grouped by resource
	foreach ($xids as $resId=>$res){
		$resConfig=$res['resConfig']; if(0) $resConfig=new WResConfig(); // Zend IDE class detection WA
		$xid=$res['xid'];

		// Get icon & label
		if($resConfig) {
			$ico=$resConfig->icon16();
			$label=cfUTF8Encode($resConfig->name());
		}
		else {
			$ico=outIcon('accounts');
			$label=cfCaption('genSite');
		}
		if(cfIsInApp()) $tag=$xid; // Ouvrir popup sur le xid
		else $tag=false;

		// Create tree node
		$tree['sub'][$resId]=array(
			'ico'=>$ico,
			'label'=>$label,
			'tag'=>$tag,
			'sub'=>array());

		if(@$commentsData['xidFolding'][$xid]!==1) $tree['sub'][$resId]['unfolded']=true;

		$nbComments=0;
		$newComments=0;

		// Display comments for this resource (or users)
		foreach ($commentsData['comments'] as $k=>$comment) if(@$comment['xid']==$res['xid']){
			$userName=@$comment['username'];
			if(!$userName) $userName=@$commentsData['profiles'][$comment['fromid']];
			if(!$userName) $userName='?';

			$ico=outIcon('comment');
			if($comment['time']>$commentsData['viewTime']){
				$ico=outIcon('commentNew');
				$newComments++;
			}

			// Subnode(s)
			$tree['sub'][$resId]['sub'][$comment['id']]=array(
				'ico'=>$ico,
				'label'=>'<b style="width:120px;overflow:hidden;text-overflow:ellipsis">'.$userName.'</b><fieldset>'.str_replace("\n",'<br>',htmlspecialchars($comment['text'])).'</fieldset>',
				'sub'=>array()
			);
			$nbComments++;
		}
		$tree['sub'][$resId]['label'].=(($newComments)?'<b> &nbsp;(':' &nbsp;(').$nbComments.(($newComments)?')</b>':')');
	}

	// Create tree object from array
	$treeView=new treeView('facebookCommentsTree');
	$treeView->phpTree=$tree;
	$treeView->selectFunction='facebookCommentsSelectRes';
	$treeView->toggleFoldFunction='facebookCommentsChangeFolding';

	// Asynchronous request: insert tree in facebookCommentsTreeOuter div (only if actually updated) and exit
	if(cfIsAsync()){
		cfAsyncHeader();

		if(@$asyncUpdated) echo cfAsyncJSaction($treeView->HTMLUpdate());
		echo cfAsyncJSaction('try{facebookCommentsBoxUpdated("'.@$updateError.'")} catch(e){}');

		// Update last time comments have been viewed
		$commentsData['viewTime']=time();

		// ...and save
		facebookCommentsCacheSave($commentsData);

		die(cfAsyncFooter());
	}

?>
<style type="text/css">
#facebookCommentsTree fieldset{vertical-align:top;cursor:default}
#facebookCommentsTree a:hover{background:none;border:0px;padding:2px;padding-left:1px}
#facebookCommentsTree a{}
#facebookCommentsTree .subTree .subTree div{border:1px solid #DDD;background:#FFF;width:100%;cursor:pointer}
#facebookCommentsTree .subTree .subTree a{cursor:default}
</style>
<script type="text/javascript">
function facebookCommentsSelectRes(n){
	if(!n.tag()) return;
	var l=n.getLabel();
	if(l.indexOf('<')>0) l=l.substr(0,l.indexOf('<'));
	else if(l.indexOf('&nbsp;')>0) l=l.substr(0,l.indexOf('&nbsp;'));
	var url='<?php echo facebookCommentsPanelURL('XID',500);?>';
	wl.UIWebPopup('/local/uiPopupRedirect.php?url='+encodeURIComponent(url.replace(/XID/,n.tag()))+'&width=500&height=600&title=<?php echo urlencode(cfCaption('genComments'));?> - '+encodeURIComponent(l));
}
function facebookCommentsBoxUpdated(errorMsg){
	var icoNode=dgi("facebookCommentsBoxIcon");
	if(errorMsg){
		icoNode.src="<?php echo outIcon('warning');?>";
		tooltipCreate(icoNode,errorMsg)
	}
	else icoNode.src="<?php echo outIcon('comments');?>";
}
function facebookCommentsBoxUpdate(){
	dgi("facebookCommentsBoxIcon").src="<?php echo outIcon('loading');?>";
	wl.asr.send({'facebookCommentsBoxUpdate':1})
}
function facebookCommentsChangeFolding(n){
	wl.asr.send({'facebookCommentsChangeFolding':n.tag(),'isFolded':(n.isFolded())?1:0})}
<?php
	// Async update data if necessary
	if(time()-$commentsData['updateTime']>FACEBOOK_COMMENTS_UPDATE_INTERVAL) echo 'connect(W,"load",facebookCommentsBoxUpdate);';
?>
</script>
<div class="frame2Header"><?php echo outImageIcon('comments','id="facebookCommentsBoxIcon"').cfCaption('genComments');?></div>
<div id="facebookCommentsTreeOuter">
<?php
	// Generate tree's HTML
	echo $treeView->HTML(false,false,false);
?>
</div>
<?php
}