<?php
/**
 * HTML output functions
 *
 * Misc function formating output HTML
 *
* parameters : POST data1 (file directory),  data2 (file name),  data3 (="view"),  data4 (action (optional)),  data5 (parameters (optional))
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

// outControl constants
define('dataText',0);
define('dataNumeric',1);
define('dataList',2);
define('dataBoolean',3);
define('dataFolderPath',4);
define('dataFilePath',5);
define('dataPassword',6);
define('dataImage',7);
define('dataArray',8);
define('dataMultiList',9);
define('dataUserRights',10);
define('alignLabelControl',0); // states that control is located right of caption/image
define('alignControlLabel',1); // states that control is located left of caption/image

/*
 ***************************************************************************************************************************
 * Error page
 ***************************************************************************************************************************
 */

/**
 * @return void
 * @param string $errorMessage: utf-8 encoded message to display
 * @param string $errorTitle: utf-8 encoded message to display
 * @param bool $exitAfter: true if script should exist after having displayed error page
 * @param array $options:
 * 			"alignBottom": align main div bottom's to page's bottom
 * @desc Display an error page and exit
*/
function outDisplayErrorPage($errorMessage, $errorTitle=false, $exitAfter=true, $options=false){
	if(!cfBGetVar('name')) cfSetBrowserCaps();

	// Mobiles
	if(cfIsMobile()){
		require_once(INCLUDE_DIR.'mobileFunctions.php');
		mfInsertHead(array('noTopControls'=>1));
	}
	// Regular browsers
	else{
		cfInsertHEAD(false);
		if(cfBGetVar('OS')=='iOS') echo '<meta name="viewport" content="width=device-width, user-scalable=yes,initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0">';
		echo '</head>';
		echo '<body class="errorPage" style="overflow:hidden;margin:0;padding:0">';
	}
	?>
<table class="frame1" height="100%" width="100%" border="0" cellspacing=0 cellpadding=0 style="margin:0;padding:0;" id="errTable">
<tr><td class="frame1Header" style="width:100%; vertical-align:middle">
	<?php
	// Icon
	if(is_array($errorTitle) && isset($errorTitle['icon'])) echo outImageIcon($errorTitle['icon']);
	else echo outImage(outIcon('alert'),false,false, "vertical-align:middle;margin-right:1em");
	// Title
	if(is_array($errorTitle) && isset($errorTitle['title'])) echo $errorTitle['title'];
	elseif ($errorTitle) echo $errorTitle;
	else echo cfCaption('genError');
	// Body
?>
</td></tr><tr><td height="100%" valign="middle" align="center" style="font-weight:bold">
<?php echo $errorMessage;?>
<?php if(@$options['alignBottom']){?>
<script type="text/javascript">alignBottom(dgi('errTable'),winMe,5)</script>
<?php }?>
</td></tr></table>
</body></html>
<?php
	cfLog('Error page displayed: '.$errorMessage,LOG_DBG);
	if($exitAfter) exit();
}

/*
 ***************************************************************************************************************************
 * General purpose HTML output scripts
 ***************************************************************************************************************************
 */

/**
 * @return HTML code of div frame
 * @param string $class
 * @param string $extraHTML : HTML code to insert in div code
 * @param string $extraStyle : HTML code to insert in style
 * @desc echo frame div declarartion, based on browser type
*/
function outDivFrame($class, $extraHTML=false, $extraStyle=false){
	if(cfGetBrowser()=='ie') $output = '<div class="'.$class.'" style="width:100%;'.$extraStyle.'"';
	else {
		$output='<div class="'.$class.'"';
		if($extraStyle) $output.=' style="'.$extraStyle.'"';
	}
	if($extraHTML) $output.=' '.$extraHTML;
	$output.=">";
	return $output;
}

/**
 * @return HTML code of div frame table
 * @param string $class
 * @param string $leftHTML : left part HTML code
 * @param string $rightHTML : right part HTML code
 * @param string $extraHTML : HTML code to insert in div code
 * @param string $extraStyle : HTML code to insert in style
 * @desc echo frame div declarartion, based on browser type
*/
function outFrameHeaderTable($class, $leftHTML, $rightHTML=false, $extraHTML=false, $extraStyle=false, $centerHTML=false){
	if($rightHTML===false) return '<div class="'.$class.'"'.(($extraHTML)?' '.$extraHTML:'').'>'.$leftHTML."</div>\n";
	$output = '<div class="'.$class.'"'.(($extraHTML)?' '.$extraHTML:'').(($extraStyle)?' style="'.$extraStyle.'"':'').'><table class="'.$class.'" style="width:100%;background:transparent;margin:0px;padding:0px;border:0px;'.(($extraStyle)?$extraStyle:'').'" cellpadding="0" cellspacing="0">';
	if(cfGetBrowser()!='ie' && $centerHTML) $output.='<colgroup><col width="33%"><col width="1%"><col width="33%"></colgroup>';
	$output.='<tr>';

	$output.='<td style="display:block;overflow:hidden;white-space:nowrap'.(($centerHTML)?';width:'.((cfGetBrowser()=='ie'||cfGetBrowser()=='webkit')?'33%':'100%'):'').'">'.$leftHTML.'</td>';
	if($centerHTML) $output.='<td style="text-align:center'.(($centerHTML)?';width:1%;white-space:nowrap':'').'">'.$centerHTML.'</td>';
	$output.='<td style="text-align:right;overflow:hidden'.(($centerHTML)?';width:33%;white-space:nowrap':'').'">'.$rightHTML.'</td>';
	$output.='</tr></table></div>';
	return $output;
}

/**
 * @return HTML code of div frame
 * @param string $class
 * @param string $extraHTML : HTML code to insert in div code
 * @param string $extraStyle : HTML code to insert in style
 * @desc echo frame div declarartion, based on browser type
*/
function outFrameFooterTable($class, $leftHTML, $rightHTML, $extraHTML=false){return outFrameHeaderTable($class, $leftHTML, $rightHTML,$extraHTML);}

/**
 * @return HTML code of table tag
 * @param string $class : css class of table
 * @param string $extraHTML : extra HTML code added to table tag
 * @param [extra args] : columns width
 * @desc return declaration of a $class table with no border, margin, padding or background, to be used for elements positionning
 * 				if more than 3 args passed, use extra args as column width
*/
function outTableTransparent($class=false, $extraStyle=false, $extraHTML=false){
	$output='<table cellpadding="0" cellspacing="0" style="width:100%;padding:0;margin:0;'.(($extraStyle)?$extraStyle:'').'"'.(($extraHTML)?' '.$extraHTML:'').">";
	if(func_num_args()>3){
		$output.='<colgroup>';
		for($i=3;$i<func_num_args();$i++)	$output.='<col width="'.func_get_arg($i).'">';
		$output.='</colgroup>';
	}
	return $output;
}

/**
 * @return string: code
 * @param string $caption : button's caption (utf-8 encoded)
 * @param string $action : optional : action (href)
 * @param string $img : optional : path to button's image
 * @param string $title : optional : button's title
 * @param string $id : optional : button's ID
 * @param string $extraHTML : optional : extra HTML code included in button definition
 * @param boolean $forceFullDisplay: display caption even in compact mode
 * @param boolean $disabled: true to disable button (button can be JS-enable via wl.button.enable function)
 * @param boolean $forceStandardTitle: true to use html title instead of custom tooltip for title display
 * @desc  Insert HTML code for a button
*/
function outButton($caption,$href='#',$img=false,$title=false,$id=false,$extraHTML=false,$forceFullDisplay=false,$disabled=false,$forceStandardTitle=false){
	//Format : <a class="button|buttonDisabled"><i></i><b><img src="icon"/><span>text</span></b><u></u></a>
	//Format : <a><i></i><b><span icon></span><span>text</span></b><u></u></a> (IE with PNG image)/

	if(substr($href,0,11)=='javascript:'){
		$isJS=true;
		$JS=substr($href,11);
	}
	else $isJS=false;

	// Theme specific button
	if(cfTGetVar('buttonFunction') && !isset($_ENV['weezo']['buttonFunction']))
		$_ENV['weezo']['buttonFunction'] = create_function('$caption,$href,$img,$title,$id,$extraHTML,$forceFullDisplay', cfTGetVar('buttonFunction'));
	if(isset($_ENV['weezo']['buttonFunction'])) return $_ENV['weezo']['buttonFunction']($caption,$href,$img,$title,$id,$extraHTML,$forceFullDisplay);

	if(cfTGetVar('compactDisplay') && !$forceFullDisplay && $img!==false && $title===false){
		if(!$title) $title=$caption;
		$caption='';
	}
	// Disabled button, no link
	if($href=='###') $output='<a href="javascript:void()" class="buttonDisabled"';
	// Disabled button, link or js
	elseif($disabled){
		$output='<a href="javascript:void(0)" class="buttonDisabled" ';
		$output.='disabledaction="'.$href.'"';
	}
	// Enabled button
	else {
		$output='<a class="button" ';
		if($isJS) $output.=' href="javascript:void(0)" onclick="'.$JS.'"';
		else $output.='href="'.$href.'"';
	}

	if($id) $output.=' id="'.$id.'"';
	if($extraHTML) $output.=' '.$extraHTML;

	// Title
	if($title) {
		if($forceStandardTitle) $output.=' title="'.$title.'"';
		else $output.=outToolTip($title,false,false,true); // image view bug WA
	}


	$output.='><i>&nbsp;</i><b>';
	// If IE6 and png, use AlphaImageLoader filter
	if(false && !cfBGetVar('png') && cfFileExtension($img)=='png') {
		$output.='<span style="filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='.$img.');';
		if(strlen($caption)>0) $output.='margin-right:0.4em;';
		$output.='" class="ieBtPng"></span><span></span>';
	}
	else {
		$output.='<img alt="" src="';
		if(strlen($caption)>0) $mr='margin-right:0.4em;'; else $mr='';
		if($img) $output.=$img.'" style="'.$mr.'"'; else $output.='/gfx/v.gif"';
		$output.='><span></span>';
	}

	if(cfIsInApp() && cfGetBrowser()=='ie' && stripos(($_SERVER['HTTP_USER_AGENT']), "msie 7.0")!==false && $caption=='')
		$output.='<span> </span></b><u>&nbsp;</u></a>';
	else
		$output.='<span>'.str_replace(' ','&nbsp;',$caption).'</span></b><u>&nbsp;</u></a>';

	return $output;
}

/**
 * @return string: code
 * @param string $caption : button's caption (utf-8 encoded)
 * @param string $href: button link (false for no)
 * @param string $action : optional : action (href)
 * @param string $img : optional : path to button's image
 * @param string $title : optional : button's title
 * @param string $id : optional : button's ID
 * @param string $extraHTML : optional : extra HTML code included in button definition
 * @param boolean $forceFullDisplay: display caption even in compact mode
 * @desc  Insert HTML code for a disabled button
*/
function outButtonDisabled($caption,$href='###',$img=false,$title=false,$id=false,$extraHTML=false,$forceFullDisplay=false){
	return outButton($caption,((!$href)?'###':$href),$img,$title,$id,$extraHTML,$forceFullDisplay,true);
}

/**
 * @return void
 * @param string $caption : button's caption (utf-8 encoded)
 * @param string $action : optional : action (href)
 * @param string $img : optional : path to button's image
 * @param string $title : optional : button's title
 * @param string $id : optional : button's ID
 * @param string $extraHTML : optional : extra HTML code included in button definition
 * @param array $options: 'forceHRef' to force href link
 * @desc  Insert HTML code for a "buttonSmall" class button
*/
function outButtonSmall($caption,$action='#',$img=false,$title=false,$id=false,$extraHTML=false,$options=false){
	// Theme specific button
	if(cfTGetVar('buttonSmallFunction') && !isset($_ENV['weezo']['buttonSmallFunction']))
		$_ENV['weezo']['buttonSmallFunction'] = create_function('$caption,$action,$img,$title,$id,$extraHTML', cfTGetVar('buttonSmallFunction'));
	if(isset($_ENV['weezo']['buttonSmallFunction'])) return $_ENV['weezo']['buttonSmallFunction']($caption,$action,$img,$title,$id,$extraHTML);

	if(cfGetBrowser()=='gecko' || isset($options['forceHRef'])){
		$output='<a href="'.$action.'" class="buttonSmall"';
	}
	else{
		if(substr($action,11)=='javascript:') $action=substr($action,11);
		$output='<a href="javascript:void(0)" class="buttonSmall" '.(($action!='#')?'onmousedown="'.$action.'"':'');
	}
	if($id) $output.=' id="'.$id.'"';
	if($extraHTML) $output.=' '.$extraHTML;
	if($title) $output.=' title="'.$title.'"';
	if(strlen($caption)>0) $mr='margin-right:0.4em;'; else $mr='';
	$output.='>'.outImage((($img)?$img:'/gfx/v.gif'),'',(($title)?' title="'.$title.'"':''),'vertical-align:middle;'.$mr);
	return $output.$caption."</a>";
}

/**
 * @return void
 * @param string $caption : button's caption (utf-8 encoded)
 * @param string $img : optional : path to button's image
 * @param string $title : optional : button's title
 * @param string $id : optional : button's ID
 * @param string $extraHTML : optional : extra HTML code included in button definition
 * @desc  Insert HTML code for a "buttonSmall" class button
*/
function outButtonSmallDisabled($caption,$img=false,$title=false,$id=false,$extraHTML=false){
	$output='<a href="javascript:void(0)" class="buttonSmallDisabled"';
	if($id) $output.=' id="'.$id.'"';
	if($extraHTML) $output.=' '.$extraHTML;
	if($title) $output.=' title="'.$title.'"><img src="'; else $output.='><img src="';
	if(strlen($caption)>0) $mr='margin-right:0.5em;'; else $mr='';
	if($img) $output.=$img.'" style="vertical-align:middle;'.$mr.'" alt="">'; else $output.='/gfx/v.gif" style="vertical-align:middle;" alt="">';
	return $output.$caption."</a>\n";
}

/**
 * @desc Return code for a 80x80 button
 *
 * @param string $content: button HTML content
 * @param string $onclick: onclick action
 * @param string $extraHTML (opt): extra HTML in <>
 * @param string $tooltipText (opt): tootlip text (HTML)
 * @param array $options: 'disabled' to disable
 * @return unknown
 */
function outButton45($content,$onclick,$extraHTML='',$tooltipText='',$options=array()){
	$options=cfArrayValuesToKeys($options);
	$out='<div class="button45'.((@$options['disabled'])?' disabled':'').'"';
	if(!@$options['disabled']) $out.=' onclick="'.$onclick.'"  onmousedown="$(this).e.className().remove(\'button45Hover\');$(this).e.className().add(\'button45Active\')" onmouseover="$(this).e.className().add(\'button45Hover\')'.(($tooltipText)?";tooltip(this,'".(addslashes(str_replace('"',"''",$tooltipText)))."')":'').'" onmouseout="$(this).e.className().remove(\'button45Hover\');$(this).e.className().remove(\'button45Active\')"';
	$out.=' '.$extraHTML.'><table><tr><td>'.$content.'</td></td></table></div>';
	return $out;
}


/**
 * @desc Return HTML code for a text-copy button
 *
 * @param string $srcId: id of source element
 * @param bool $disabled: true to disable button
 * @return string
 */
function outButtonClipboardCopy($srcId,$disabled=false){
	if(cfGetBrowser()=='ie') return outButton(false,"javascript:wl.clipboardCopy('".$srcId."')",outIcon('paste'),cfCaption('genCopy'),'btID','name="copyButton"',false,$disabled); else return '';
}

/**
 * Show a tooltip on mouseover
 * output mouseover and mouseout JS that must be inserted into tag
 *
 * @param string $text (optional) : main tooltip text (utf-8 encoded)
 * @param string $title (optional) : tooltip title (utf-8 encoded)
 * @param string $iconSrc (optional) : tooltip icon src
 */
function outToolTip($text=false, $title=false, $iconSrc=false,$immediateDisplay=false){
	if($iconSrc) $title='<img src=\''.$iconSrc.'\' style=\'vertical-align:middle;margin-right:0.5em\'>'.$title;
	if($title) $title='\'<b>'.addslashes(str_replace('"','&quot;',$title)).'</b><br>\''; else $title="''";
	if(!$text) $text='false'; else if(!cfCmpLeft($text,'javascript:')) $text="'".addslashes($text)."'"; else $text=substr($text,11);
	$noDelay=($immediateDisplay)?',1':'';
	return ' onmouseover="if(W.dd) tooltip(this,'.$title.'+'.$text.$noDelay.'); else this.alt=\''.str_replace(array("'",'"'),'',$text).'\'" ';
}

/**
 * @desc Set document-level keydown handler
 *
 * @param array $keys (keycode=>javascript eval)
 * @return string
 */
function outKeyHandler($jsNode,$keys,$scriptTags=false){
	$out='';
	foreach ($keys as $kc=>$js) $out.=(($out=='')?'':',').'"'.$kc.'":\''.$js.'\'';
	return (($scriptTags)?'<script type="text/javascript">':'').'wl.setKDHandler('.$jsNode.',{'.$out.'})'.(($scriptTags)?'</script>':'');
}

/**
 * @return string : path to icon file
 * @param string $iconName : name of icon, with or without file extension
 * @desc return path to icon named $iconName and located in resource's directory or (if not found in resource's directory) in /gfx/.
 	Extension (.gif or .png) is determined according to webbrowser's transparency support
*/
function outIcon($iconName,$default='/gfx/broken.gif'){

	// Search for icon in cache
	if(!isset($_ENV['weezoIcons'])) $_ENV['weezoIcons']=cfMGetVar('weezoIcons');
	if(isset($_ENV['weezoIcons'][cfActualTheme()][$iconName])) return cfCDNPrefix().$_ENV['weezoIcons'][cfActualTheme()][$iconName];

	// Look for icon in theme directory
	$iconDir=cfAppThemeDir().'/';
	$themeDir=cfAppThemeDir(true).'/';

	if(cfFileExtension($iconName) && file_exists($iconDir.$iconName)) return cfCDNPrefix().$themeDir.$iconName;
	$iconNameNE=cfFileWithoutExtension($iconName);
	if(file_exists($iconDir.$iconNameNE.'.png')) return  cfCDNPrefix().$themeDir.$iconNameNE.'.png';
	if(file_exists($iconDir.$iconNameNE.'.gif')) return  cfCDNPrefix().$themeDir.$iconNameNE.'.gif';

	// resources called in application environnement
	if(isset($_SERVER['APPLICATION_PHP_SELF'])){
		$iconDir=cfAppDocRoot().dirname($_SERVER['APPLICATION_PHP_SELF']).'/';
		if(cfFileExtension($iconName) && file_exists($iconDir.$iconName)) return dirname($_SERVER['APPLICATION_PHP_SELF']).'/'.$iconName;
		$iconName=cfFileWithoutExtension($iconName);
		if(file_exists($iconDir.$iconName.'.png')) return dirname($_SERVER['APPLICATION_PHP_SELF']).'/'.$iconName.'.png';
		if(file_exists($iconDir.$iconName.'.gif')) return dirname($_SERVER['APPLICATION_PHP_SELF']).'/'.$iconName.'.gif';
	}
	// Look for icon in resource directory
	if(isset($_SERVER['PHP_SELF'])) {
		$iconDir=cfAppDocRoot().dirname($_SERVER['PHP_SELF']).'/';
		if(cfFileExtension($iconName) && file_exists($iconDir.$iconName)) return $iconName;
		$iconName=cfFileWithoutExtension($iconName);
		if(file_exists($iconDir.$iconName.'.png')) return  dirname($_SERVER['PHP_SELF']).'/'.$iconName.'.png';
		if(file_exists($iconDir.$iconName.'.gif')) return  dirname($_SERVER['PHP_SELF']).'/'.$iconName.'.gif';
	}
	// If not found, look in /gfx directory
	$iconDir=cfAppDocRoot().'/gfx/';
	if(cfFileExtension($iconName) && file_exists($iconDir.$iconName)) return cfCDNPrefix().'/gfx/'.$iconName;
	$iconName=cfFileWithoutExtension($iconName);
	if(file_exists($iconDir.$iconName.'.png')) return  cfCDNPrefix().'/gfx/'.$iconName.'.png';
	if(file_exists($iconDir.$iconName.'.gif')) return  cfCDNPrefix().'/gfx/'.$iconName.'.gif';

	return $default;
}

/**
 * @desc Return GIF version of icon if browser doesn't support PNG
 *
 * @param string $iconName
 * @return string
 */
function outIconNPNG($iconName,$default='/gfx/broken.gif'){
	$ico=outIcon($iconName,$default);
	if(!cfBGetVar('png') && cfFileExtension($ico)=='png' && file_exists(cfAppDocRoot().cfFileWithoutExtension($ico).'.gif')) return cfFileWithoutExtension($ico).'.gif';
	return $ico;
}

/**
 * @desc Insert a png loading icon, that will rotate with javascript (and not animated gif)
 *
 * @param string $id: img id
 * @param boolean $big: true for 32x32 icon, false for 16x16
 * @param string $extraHTML
 * @return string
 */
function outLoadingIcon($id=false,$big=false,$extraHTML=false){
	if(!$id) $id='l'.rand();
	return '<img id="'.$id.'" src="'.outIcon(($big)?'loadingS32':'loadingS16').'" '.$extraHTML.'><script type="text/javascript">wl.loading.set("'.$id.'",'.((int)$big).')</script>';
}

/**
 * @desc Return application's Icon
 *
 * @param integer $size: icon's size
 */
function outIconApp($size=16,$mobile=false){
	if($mobile){
		if($size==16) return '/gfx/appIcon16mobile.gif';
		if($size==32) return '/gfx/appIcon32mobile.gif';
	}
	switch ($size){
		case 16: return '/gfx/appIcon16.png';break;
		case 32: return '/gfx/appIcon32.png';break;
		case 48: return '/gfx/appIcon.png';break;
	}
	return false;
}

/**
 * @return string path from document root
 * @param string $type : type of resource
 * @param string $subType : sub type of resource
 * @desc return path (from document root) to resource's icon
*/
function outResourceIcon($type, $subType){
	// Check if describer file exists and defines an icon
	if(file_exists(cfAppDocRoot().'/res/'.$type.'/'.$subType.'/describer.ini')){
		$describer=cfParse_ini_file(cfAppDocRoot().'/res/'.$type.'/'.$subType.'/describer.ini');
		if(isset($describer['iconFile']) && file_exists( cfAppDocRoot().'/res/'.$type.'/'.$subType.'/'.$describer['iconFile']))
			return '/res/'.$type.'/'.$subType.'/'.$describer['iconFile'];
	}
	// Else check for resourceIcon.gif/.jpg in resource directory
	if(file_exists(cfAppDocRoot().'/res/'.$type.'/'.$subType.'/resourceIcon.gif')) return '/res/'.$type.'/'.$subType.'/resourceIcon.gif';
	elseif(file_exists(cfAppDocRoot().'/res/'.$type.'/'.$subType.'/resourceIcon.jpg')) return '/res/'.$type.'/'.$subType.'/resourceIcon.jpg';
	// Else check for resourceIcon.gif/.jpg in resource upper directory
	elseif(file_exists(cfAppDocRoot().'/res/'.$type.'/resourceIcon.jpg')) return '/res/'.$type.'/resourceIcon.jpg';
	elseif(file_exists(cfAppDocRoot().'/res/'.$type.'/resourceIcon.gif')) return '/res/'.$type.'/resourceIcon.gif';
	// Else return default icon
	elseif(file_exists(cfAppDocRoot().'/res/resourceIcon.gif')) return '/res/resourceIcon.gif';
	else return '';
}

/**
 * @desc return HTML code for image, with specified height & width (based on actual image size) and optional link
 * 		if img is png and browser IE, return span/DXImageTransform
 * @param string $imagePath: either path & name of image or relative path & name of image
 * @param string href: link
 * @param string $extraHTML: extra HTML code inserted into <img> tag
 * @param string $extraHTML: extra CSS style code inserted into <img> tag
 * @param number $imageSizeRatio: enlarge/shrink image ratio. Set to 1 for real-size
 * @param boolean $setSize: force image size by setting width and height attributes
 * @param boolean $fadeInOnLoad: set visibility=hidden and onload=fade in
 * @return string: HTLM code
*/
function outImage($imagePath, $href=false, $extraHTML=false, $extraStyle=false, $imageSizeRatio=1, $setSize=true, $fadeInOnLoad=false){
	/*
	static $sprites=false;
	if(!$sprites) $sprites=cfMGetVar('cssSprites'); // Load CSS sprites index
	$tmps=cfMGetVar('tmpSprite');
	$tmps[$imagePath]=1;
	cfMSetVar('tmpSprite',$tmps);
	unset($tmps);
	*/


	if ($href=='#') $href=false;
	// if href, put the <a> tag
	if($href) $output='<a href="'.$href.'"'.(($extraStyle)?' style="'.$extraStyle.'"':'').(($extraHTML)?' '.$extraHTML:'').'>'; else $output='';

	// Get image relative and absolute paths
	if(substr($imagePath,1,1)==':') {
		$src=cfExtImage($imagePath);
		$completeFileName=$imagePath;
	}
	else{
		$src=$imagePath;
		if(substr($imagePath,0,1)=='/') $completeFileName=cfAppDocRoot().$imagePath; else $completeFileName=substr($_SERVER['SCRIPT_FILENAME'],0,strrpos($_SERVER['SCRIPT_FILENAME'],'/')).'/'.$imagePath;
	}
	// If browser doesn't support PNG
	if(!cfBGetVar('png') && cfFileExtension($src)=='png') {
		// Get image size
		list($width, $height, $type, $attr) = @getimagesize($completeFileName); $width*=$imageSizeRatio; $height*=$imageSizeRatio;

		// if a width or height is passed in extra style, stretch image, else display full-size
		if(strpos($extraStyle,'height') || strpos($extraStyle,'width')) $sizingMethod='scale'; else  $sizingMethod='image';
		$output.='<span style="filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='.$src.', sizingMethod='.$sizingMethod.');';
		$output.='width:'.$width.'px; height:'.$height.'px;';
		$output.=(($href)?' cursor:pointer;':'');
		if(!$href) $output.=(($extraStyle)?' '.$extraStyle:'');
		$output.='"';
		if(!$href) $output.=(($extraHTML)?' '.$extraHTML:'');
		$output.="></span>";
	}
	// Browser supports PNG
	else {
		// No png fade-in in IE
		if(cfGetBrowser()=='ie') $fadeInOnLoad=0;
		// CSS Sprites
		if(isset($sprites[$src])){
			$output.='<img src="/gfx/v.gif" alt=""';
			$s=$sprites[$src];
			$extraStyle.=';background:url(/gfx/'.$s['f'].'.png) -'.$s['x'].'px 0';
			$extraStyle.=';width:'.$s['w'].'px;height:'.$s['h'].'px';
			$setSize=false;
		}
		// Regular img output
		else $output.='<img src="'.$src.'" alt=""';

		// Force image size
		if($setSize){
			list($width, $height, $type, $attr) = @getimagesize($completeFileName); $width*=$imageSizeRatio; $height*=$imageSizeRatio;
			$extraStyle.=';width:'.$width.'px;height:'.$height.'px';
		}
		// Fade-in and style
		if($fadeInOnLoad) $output.=' onload="fade(this)"';
		if($extraStyle && !$href) $output.=' style="'.$extraStyle.(($fadeInOnLoad)?';visibility:hidden':'').'"';
		elseif ($fadeInOnLoad) $output.=' style="visibility:hidden"';
		// extraHTML
		if(!$href && $extraHTML) $output.=' '.$extraHTML.'>'; else $output.='>';
	}
	if($href) $output.='</a>';
	return $output;
}

/**
 * @desc return outImage, with theme-defined size (use theme var "themeImageWidth")
 *
 * @param string $imagePath : either path & name of image or relative path & name of image
 * @param string href : link
 * @param string $extraHTML : extra HTML code inserted into <img> tag
 * @param string $extraHTML : extra CSS style code inserted into <img> tag
 */
function outThemeImage($imagePath,$href=false,$extraHTML=false, $extraStyle=false){
	return outImage($imagePath, $href, $extraHTML, $extraStyle, ((cfTGetVar('iconRatio'))?cfTGetVar('iconRatio'):1),true);
}

/**
 * @desc Return code for a simple left-side icon
 *
 * @param string $icoName: ico filename without ext, from /gfx path
 * @param string $extraHTML
 * @param string $extraStyle
 * @return string
 */
function outImageIcon($icoName,$extraHTML='',$extraStyle=''){
	return '<img src="'.outIcon($icoName).'" alt="" style="vertical-align:middle;margin-right:0.5em;'.$extraStyle.'" '.$extraHTML.'>';
}

/**
 * @return void
 * @desc draw a drop shadow around an HTML object, including it into a table containing parts of shadow.
 *	   this function must be called after object has been sent to output.
 * @param int $width   HTML width of table (optional)
*/
function outShadowBefore($width=false){
	static $b;	if(!$b) $b=cfGetBrowser();

	if(cfTGetVar('noShadow')) return;
	echo '<table cellpadding=0 cellspacing=0 ';
	if($width) echo 'width="'.$width.'"'; else echo 'width="1px"';
	if($b=='opera') echo '><tr><td colspan="2" rowspan="2">';// Opera
	elseif($b=='gecko'||$b=='webkit')  echo '><tr><td colspan="2">';// Firefox/Netscape/Chrome/Safari
	else echo '><tr><td>';// IE
}

/**
 * @return void
 * @desc draw a drop shadow around an HTML object, including it into a table containing parts of shadow. (mainly used to draw shadow around photo frames)
 *	   this function must be called after object has been sent to output.
*/
function outShadowAfter(){
	static $b;	if(!$b) $b=cfGetBrowser();

	if(cfTGetVar('noShadow')) return;

	if($b=='opera'){ // Opera
?></td><td class="sOrt"><img src="/gfx/s/rt.png" alt=""></td></tr><tr><td class="sOr"><img src="/gfx/s/_h.png" alt=""></td></tr><tr class="sOtrb"><td><img src="/gfx/s/lb.png" alt=""></td><td class="sOb"><img src="/gfx/s/_v.png" alt=""></td><td><img src="/gfx/s/rb.png" alt=""></td></tr></table><?php } // Firefox / Netscape
	else if (cfGetBrowser()=='gecko'||$b=='webkit') {?></td><td class="sGr"></td></tr><tr class="sGtrb"><td width="10"><img src="/gfx/s/lb.png" alt=""></td><td class="sGb"><img src="/gfx/s/_h.png" alt=""></td><td><img src="/gfx/s/rb.png" alt=""></td></tr></table><?php }
	else{//IE and others...
		?></td><td class="sIEr">&nbsp;</td></tr><tr class="sIEtrb"><td class="sIEb"></td><td class="sIErb"></td></tr></table><?php }
}

/**
 * @return void
 * @desc draw a drop shadow around an HTML object, including it into a table containing parts of shadow. Used to draw shadow around frame1/2/3
 *	   this function must be called after object has been sent to output.
*/
function outFrameShadowAfter(){
	static $b;	if(!$b) $b=cfGetBrowser();

	if(cfTGetVar('noShadow')) return;

	if($b=='opera'){ // Opera
		?></td>
<td class="fsOrt"><img src="/gfx/s/rt.png" alt=""></td></tr><tr>
<td class="fsOr"><img src="/gfx/s/_h.png" alt=""></td></tr><tr class="fsOtrb">
<td><img src="/gfx/s/lb.png" alt=""></td>
<td class="fsOb"><img src="/gfx/s/_v.png" alt=""></td>
<td><img src="/gfx/s/rb.png" alt=""></td>
</tr></table>
	<?php } // Firefox / Netscape
	else if ($b=='gecko'||$b=='webkit') {?></td>
<td class="fsGr"></td>
</tr><tr class="fsGtrb"><td width="10"><img src="/gfx/s/lb.png" alt=""></td>
<td class="fsGb"><img src="/gfx/s/_h.png" alt=""></td>
<td><img src="/gfx/s/rb.png" alt=""></td></tr></table>
	<?php }
	else{//IE and others...
		?>
</td><td class="fsIEr">&nbsp;</td></tr><tr class="fsIEtrb">
<td class="fsIEb"></td>
<td class="fsIErb"></td></tr></table>
	<?php }
}

/**
 * @return void
 * @param string $URL : destination URL of POST method
 * @param string $value1 : default value of field 1 (usually current directory)
 * @param string $value2 : default value of field 2 (usually selected file, filled by javascript)
 * @param string $value3 : default value of field 3 (usually required action, filled by javascript)
 * @param string $value4 : default value of field 4 (usually empty, extra passed data, filled by javascript)
 * @param string $value5 : default value of field 5 (usually empty, extra passed data, filled by javascript)
 * @param boolean $forceAsyncSubmit : true if form must be submited using HTTPXmlRequest
 * @param boolean $noSt : true if noSt input must be added in order to disable request log into DB
 * @desc Insert hidden form used by most "explorer" resources for communicating users actions to scripts
*/
function outInsertStandardComForm($URL=false, $value1=false, $value2=false, $value3=false, $value4=false, $value5=false, $forceAsyncSubmit=false, $noSt=false){
	static $includedOnce=0;

	if($includedOnce) return; $includedOnce=1;

	if(!$URL) $URL=$_SERVER['PHP_SELF'];

	$dbg=cfMGetVar('comFormDebug');
?>
<script language="javascript" type="text/javascript">
function fAS(d2,d3,d4,d5,forceAsync){
var cF=D.comForm; for(var i=2;i<6;i++) eval('if(d'+i+'!==undefined && d'+i+'!==false) cF.data'+i+'.value=d'+i)
<?php
	// Fix for non-resource scripts
	if(!isset($_SESSION['activeResourceId'])) $_SESSION['activeResourceId']='';
	// If resource uses Async submit, form should be asyns submited if no target is specified for form
	if($forceAsyncSubmit || cfRGetVar('useHTTPXmlRequest')) {
?>
if((cF.target=="" || cF.target=="_self" || forceAsync) && forceAsync!="sync") return asyncSubmit(true);	else cF.submit();
<?php
	} else{
?>
if(forceAsync && forceAsync!='sync') return asyncSubmit(true);
else {if(W.winUseWindows && (cF.target=="" || cF.target=="_self")) winMe.setReloading();cF.submit();}
<?php
	}
?>
return true;
}
</script>
<form id="comForm" name="comForm" action="<?php echo $URL;?>" method="POST" enctype="multipart/form-data" style="display:none<?php if($dbg) echo 'e';?>" target="_self">
<input name="data1"<?php if($value1!==false) echo ' value="'.$value1.'"';?>>
<input name="data2"<?php if($value2!==false) echo ' value="'.$value2.'"';?>>
<input name="data3"<?php if($value3!==false) echo ' value="'.$value3.'"';?>>
<input name="data4"<?php if($value4!==false) echo ' value="'.$value4.'"';?>>
<input name="data5"<?php if($value5!==false) echo ' value="'.$value5.'"';?>>
<input name="resId" value="<?php echo $_SESSION['activeResourceId'];?>">
<?php if($noSt) echo '<input name="noSt" value="true">';
	if($dbg) {
?>
<input type="button" value="Sync" onclick="fAS(undefined,undefined,undefined,undefined,'sync')">
<input type="button" value="Async" onclick="fAS(undefined,undefined,undefined,undefined,true)">
<?php
	}
	echo "</form>";
}

/**
 * @desc Toggle collapse button
 *		When clicked on, this button collapse/uncollapse the element identified by $collapsedId id.
 *  		This button must be used along with wl.nodeCollapseToggle JS function (PHP function : w)
 *
 * @param string $collapsedId : id of element to collapse
 * @param string $phpLinkedVarName (optional): name of php boolean resource variable indicating wether element should be shown (true) or collapsed (false)
 * 			if present, resource variable named $phpLinkedVarName will be dynamically updated through async request
 * @param string $defaultShow (optional): true if default visibility is shown (unused if $phpLinkedVarName)
 * @param string $jsCallbackFunction (optional): javascript function called back on fold/unfold steps
 *
 *
  * @return string : HTML codexxxxx
 */
function outButtonToggleCollapse($collapsedId, $phpLinkedVarName=false, $defaultShow=true, $jsCallbackFunction='false'){
	//if($jsCallbackFunction!='false') $jsCallbackFunction="'".$jsCallbackFunction."'";
	if(!$phpLinkedVarName) $show=$defaultShow; else $show=cfRGetVar($phpLinkedVarName);
	$output=outButtonSmall(false,'javascript:wl.nodeCollapseToggle(\''.$collapsedId.'\','.(($phpLinkedVarName)?'\''.$phpLinkedVarName.'\'':'false').', '.$jsCallbackFunction.')',outIcon('minus'),cfCaption('genHide'),$collapsedId.'BtMinus','style="display:'.(($show)?'inline':'none').'"');
	return $output.outButtonSmall(false,'javascript:wl.nodeCollapseToggle(\''.$collapsedId.'\','.(($phpLinkedVarName)?'\''.$phpLinkedVarName.'\'':'false').', '.$jsCallbackFunction.')',outIcon('plus'),cfCaption('genShow'),$collapsedId.'BtPlus','style="display:'.((!$show)?'inline':'none').'"');
}

/**
 * @desc Toggle collapse button
 *		When clicked on, this button collapse/uncollapse the element identified by $collapsedId id.
 *  		This button must be used along with wl.nodeCollapseToggle JS function (PHP function : w)
 *
 * @param string $collapsedId : id of element to collapse
 * @param string $phpLinkedVarName (optional): name of php boolean resource variable indicating wether element should be shown (true) or collapsed (false)
 * 			if present, resource variable named $phpLinkedVarName will be dynamically updated through async request
 * @param string $defaultShow (optional): true if default visibility is shown (unused if $phpLinkedVarName)
 * @param string $jsCallbackFunction (optional): javascript function called back on fold/unfold steps
 *
 *
  * @return string : HTML codexxxxx
 */
function outButtonToggleCollapse2($collapsedId, $phpLinkedVarName=false, $defaultShow=true, $jsCallbackFunction='false'){
	//if($jsCallbackFunction!='false') $jsCallbackFunction="'".$jsCallbackFunction."'";
	if(!$phpLinkedVarName) $show=$defaultShow; else $show=cfRGetVar($phpLinkedVarName);
	$output='<img class="buttonToggleCollapse" onclick="wl.nodeCollapseToggle(\''.$collapsedId.'\','.(($phpLinkedVarName)?'\''.$phpLinkedVarName.'\'':'false').', '.$jsCallbackFunction.')" src="'.outIcon('up').'" '.outToolTip(cfCaption('genHide')).' id="'.$collapsedId.'BtMinus" style="display:'.(($show)?'inline':'none').'">';
	$output.='<img class="buttonToggleCollapse" onclick="wl.nodeCollapseToggle(\''.$collapsedId.'\','.(($phpLinkedVarName)?'\''.$phpLinkedVarName.'\'':'false').', '.$jsCallbackFunction.')" src="'.outIcon('down').'" '.outToolTip(cfCaption('genShow')).' id="'.$collapsedId.'BtPlus" style="display:'.((!$show)?'inline':'none').'">';
	return $output;
}



/**
 * @desc Toggle collapse button
 *		When clicked on, this button collapse/uncollapse the element identified by $collapsedId id.
 *  		This button must be used along with wl.nodeCollapseToggle JS function (PHP function : w)
 *
 * @param string $collapsedId : id of element to collapse
 * @param string $phpLinkedVarName (optional): name of php boolean resource variable indicating wether element should be shown (true) or collapsed (false)
 * 			if present, resource variable named $phpLinkedVarName will be dynamically updated through async request
 * @param string $defaultShow (optional): true if default visibility is shown (unused if $phpLinkedVarName)
 * @param string $jsCallbackFunction (optional): javascript function called back on fold/unfold steps
 *
 *
  * @return string : HTML codexxxxx
 */
function outButtonToggleFrameCollapse($collapsedIds, $phpLinkedVarName=false, $defaultShow=true, $jsCallbackFunction='false'){
	//if($jsCallbackFunction!='false') $jsCallbackFunction="'".$jsCallbackFunction."'";
	if(!$phpLinkedVarName) $show=$defaultShow; else $show=cfRGetVar($phpLinkedVarName);
	if(!is_array($collapsedIds)) $collapsedIds=array($collapsedIds);
	$js='';
	foreach ($collapsedIds as $collapsedId){
		if(!$js) {
			$js='javascript:$(\''.$collapsedId.'\').e.frameToggle('.(($phpLinkedVarName)?'\''.$phpLinkedVarName.'\'':'false').', '.$jsCallbackFunction.')';
			$mainId=$collapsedId;
		}
		else $js.=';$(\''.$collapsedId.'\').e.frameToggle()';
	}

	$output=outButtonSmall(false,$js,outIcon('sup'),cfCaption('genHide'),$mainId.'BtMinus','style="display:'.(($show)?'inline':'none').'"');
	return $output.outButtonSmall(false,$js,outIcon('sdown'),cfCaption('genShow'),$mainId.'BtPlus','style="display:'.((!$show)?'inline':'none').'"');
}

/**
 * @desc : insert div tag for a collapsable frame
 *
 * @param string $collapsedId : id of frame
 * @param string $phpLinkedVarName (optional) : linked php var. If not set, always display frame at page load
 * @param string $extraHTML (optional): extra HTML
 * @param string $isCollapsed (optional): if $phpLinkedVarName is provided, set to "hidden" to apply display:none instead of height:1. if $phpLinkedVarName not provided, boolean state of frame
 */
function outCollapsableFrame($collapsedId, $phpLinkedVarName=false,$extraHTML=false, $isCollapsed='auto'){
	if($isCollapsed==='hidden' && $phpLinkedVarName) return '<div id="'.$collapsedId.'"'.((cfRGetVar($phpLinkedVarName))?'':' style="display:none"').' '.(($extraHTML)?$extraHTML:'').'>';
	return '<div id="'.$collapsedId.'"'.((($phpLinkedVarName && cfRGetVar($phpLinkedVarName)) || !$isCollapsed)?'':' style="position:relative;height:1px;overflow:hidden;margin:0;padding:0"').' '.(($extraHTML)?$extraHTML:'').'>';
}

/**
 * @desc HTML code for a vertical td for folding / unfolding another TD
 *
 * @param string $tdId: id of td to collapse/unfold, name of PHP linked var
 * @param string $pos: 'right' (if collapsable td is in the right-part of screen) or 'left' else
 * @param array $options (innerHTML, extraHTML, postToggleJS, class)
 * @return string
 */
function outCollapsableTD($tdId,$pos='right',$options=array()){
	$preToggleJS=isset($options['preToggleJS'])?$options['preToggleJS']:'';
	$postToggleJS=isset($options['postToggleJS'])?$options['postToggleJS']:'';
	$extraHTML=isset($options['extraHTML'])?$options['extraHTML']:'';

	// Handle
	$out='<td class="colTD colTD'.ucfirst($pos).'" onclick="if(this.clickFrozen) return; '.$preToggleJS.';toggleTD(\''.$tdId.'\');'.$postToggleJS.'" id="'.$tdId.'_colTD" '.$extraHTML.' title="'.((cfRGetVar($tdId))?cfCaption('genHide'):cfCaption('genShow')).'">';
	if(isset($options['innerHTML'])) $out.=$options['innerHTML'].'<br>';
	$out.='<script type="text/javascript">var hideCaption="'.cfCaption('genHide').'", showCaption="'.cfCaption('genShow').'"</script>';
	$out.='<div class="'.((!cfRIssetVar($tdId) || cfRGetVar($tdId))?'colTDUF':'colTDF').'"></div>';

	cfDragAddItem($tdId.'_colTD','+RECEIVER_ONLY+HIGHLIGHT_FUNCtoggleTDHL');
	return $out.'</td>';
}

/**
 * @desc : insert help button : async call /help.php script for on-screen help display
 *
 * @param string $helpCaption: caption reference
 * @param string $helpTitle: title reference (in most cases, title was included into help caption's text)
 */
function outHelpButton($helpCaption,$helpTitle=false){
	return outButtonSmall(false, 'javascript:wl.showHelp(\''.addslashes($helpCaption).'\',\''.addslashes($helpTitle).'\')',outIcon('help'),cfCaption('genHelp'),false,'style="vertical-align:middle"');
}

/**
 * @desc return string truncated to $maxlength chars, and finished with "..."
 *
 * @param string $caption
 * @param integer $maxlength
 * @return string
 */
function outCaptionTruncated($caption, $maxlength){
	if(strlen($caption)>$maxlength) return substr($caption,0,$maxlength-3).'...'; else return $caption;
}

/**
 * @desc Return tags needed to transform a node's children nodes with eH attribute into eHover-able items
 *
 * @return string attributes
 */
function outEHoverFrameAttr($extramousemove='',$extramouseout=''){return ' onmousemove="eHover(event);'.$extramousemove.'" onmouseout="eHover();'.$extramouseout.'"';}

/**
 * @desc Return flash player code
 *
 * @param string $src: swf source
 * @param string $id: id
 * @param integer $width
 * @param integer $height
 * @param array $options: array of player options (param & embed properties)
 * @param array $srcGET: get options for swf source
 * @param string $extraHTML: extra HTML code included to object and embed tags
 * @return string: HTML code
 */
function outFlashPlayer($src,$id,$width=100,$height=100,$options=array(),$srcGET=array(),$extraHTML=''){
	// SWF source GET parameters
	if(!strpos($src,'?')) $src.='?'; else $src.='&amp;';
	$src.='v='.cfGGetVar('appVersion');
	if(is_array($srcGET) && count($srcGET)){
		foreach ($srcGET as $key=>$value) $src.='&amp;'.$key.'='.$value;
	}
	//$src=cfHostName().$src;

	// Player default options
	if(!is_array($options)) $options=array();
	$options+=array(
	'movie'=>$src,
	'allowScriptAccess'=>'sameDomain',
	'allowFullScreen'=>'true',
	'fullScreenRect'=>'true',
	'quality'=>'high',
	'bgcolor'=>'#000',
	'wmode'=>'transparent',
	'swLiveConnect'=>'true'
	);

	$output='<object classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000"'."\n".'	id="'.$id.'"'."\n".'	codebase="http'.((isset($_SERVER['HTTPS']))?'s':'').'://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=7,0,0,0"'."\n".'	width="'.$width.'"'."\n".'	height="'.$height.'"'."\n".'	align="middle" '.$extraHTML.'>'."\n";

	// IE Settings
	foreach ($options as $key=>$value) {
		if($value===true) $value='true';
		elseif($value===false) $value='false';
		$output.='	<param name="'.$key.'" value="'.$value.'" />'."\n";
	}

	// Firefox / Opera
	$options+=array(
	'width'=>$width,
	'height'=>$height,
	'type'=>'application/x-shockwave-flash',
	'quality'=>'high',
	'pluginspage'=>'http'.((isset($_SERVER['HTTPS']))?'s':'').'://www.macromedia.com/go/getflashplayer'
	);
	unset($options['movie']);
	unset($options['swLiveConnect']);
	$output.='	<embed name="'.$id.'" id="'.$id.'Embed" src="'.$src.'" '.$extraHTML."\n";
	foreach ($options as $key=>$value) {
		if($value===true) $value='true';
		elseif($value===false) $value='false';
		$output.='		'.$key.'="'.$value."\" \n";
	}
	return $output."	/>\n</object>\n";
}

/**
 * @desc Display a radio button with a label. Clicking on label will result in radio button click
 *
 * @param string $radioName
 * @param string $radioValue
 * @param bool $checked
 * @param string $caption
 * @param array $options: 'extraHTML' for radio tag, 'onclick' for onclick action
 * @return unknown
 */
function outRadioAndCaption($radioName, $radioValue, $checked=false, $caption=false, $options=array()){
	$out='<div style="cursor:pointer"><input type="radio" name="'.$radioName.'" value="'.$radioValue.'" '.(($checked)?'checked="checked"':'').' style="float:left;vertical-align:middle;margin-right:0.5em"';
	if(@$options['extraHTML']) $out.=' '.$options['extraHTML'];
	if(@$options['onclick']) $out.=' onclick="'.$options['onclick'].'"';
	$out.='><div onclick="this.previousSibling.click()">'.$caption.'</div></div>';
	return $out;
}

/**
 * @desc Return code for a drop down menu
 *
 * @param string $mainCaption
 * @param array $items ('iconSrc'=>,'text'=>,'action'=>)
 * @param string $options: ('extraHTML'=>)
 * @return string
 */
function outDropDownMenu($mainCaption,$items,$options=array()){
	$id='dd'.rand();
	$nb=0;

	$out='<span id="'.$id.'"><span class="link" '.@$options['extraHTML'].'>'.$mainCaption.outImage(outIcon('down'),false,false,'vertical-align:middle"').'</span>';
	$array='[';
	if(!is_array($items)) throw new Exception('Arg 2 must be an array');

	foreach ($items as $key=>$value) {
		$nb++;
		// Separator
		if($value==='sep') {
			$array.=(($nb==1)?'':',').'ContextMenuItem("sep")';
			continue;
		}

		if(!isset($value['text']) || !isset($value['action'])) {
			throw new Exception('Missing text or action property');
			return;
		}

		$array.=(($nb==1)?'':',').'ContextMenuItem("'.@$value['iconSrc'].'","'.$value['text'].'",'.json_encode($value['action']).((@$value['default'])?',1':',0').')';
	}
	$array.=']';

	$out.='<script type="text/javascript">';
	$out.="ContextMenuAttach($('".$id."').firstChild,".$array.",{'align':'right','leftClick':1,'button':1})";
	$out.='</script>';

	return $out;
}

/*
 ***************************************************************************************************************************
 * Explorer specific HTML controls output scripts
 ***************************************************************************************************************************
 */

/**
 * @desc Send files button. Button is not present if file send not allowed
 *
  * @return string : HTML code
 */
function outExplorerButtonSend(){if(cfRGetVar('sendAllowed')) return outButton(cfCaption('genSend'),'javascript:send(\'displayForm\')',outIcon('send'));}

/**
 * @desc Download file button. Button is not present if file download not allowed
 *
 * @param integer $fileIndex : index of file to download
 * @param string $title (optional) : title of button
 * @return string : HTML code
 */
function outExplorerButtonDownload($fileIndex, $title=false){
	if(cfRGetVar('downloadShortcut')) return outButton(false,'javascript:dl(\''.$fileIndex.'\')',outIcon('dl'),($title)?$title:cfCaption('genDownload'));
}

/**
 * @desc Display a path where sub-paths are hyperlink to sub-paths
 *
 * @param string $path : path to display
 * @param boolean $includeJS : include or not neededJS
 * @return string : HTML code
 */
function outClickablePath($path, $includeJS=true){
	global $clickablePathIndex;

	$bPath=cfRGetVar('path');

	$output='';
	// Include javascript
	if(!isset($clickablePathIndex)) {
		$clickablePathIndex=1;
		if($includeJS){
			$output.= '<script language="javascript" type="text/javascript">'."\n";
			$output.="function cPath(index,lvl,fbp){\n";
			if($bPath=='computerRoot') $output.='if(index=="computerRoot") {document.comForm.data1.value="*resourceBasePath*"; path=".";} else {';
			$output.='if(fbp) {var path="*resourceBasePath*/"; var i=1;} else {var path="";var i=0;}';
			$output.='for(;i<lvl+1;i++){path+=dgi(\'cp\'+index+\'_\'+i).innerHTML+\'/\';}'."\n";
			if($bPath=='computerRoot') $output.="}\n";
			$output.="wl.mask.show(1);\n";
			$output.="if(path=='*resourceBasePath*/') path='*resourceBasePath*';"; // Linux home workaround
			$output.='fAS(path,"goDirect","","");}</script>'."\n";
		}
	}

	if(!cfRGetVar('subFoldersIncluded')) return false;
 	if(cfRGetVar('efCurrentDirectory')=='computerRoot') {return $output.'<a href="javascript:cPath(\'computerRoot\')">'.outImage(outIcon('computerRoot'),false,'title="'.cfCaption('explorerComputerRoot').'"','vertical-align:middle').'</a>';}
	$fromResourceBasePath=false;

	if(cfSharedMode()=='list'){
		$path=str_replace('*resourceBasePath*','.',$path);
		$fromResourceBasePath=true;
	}

	// Internal counter, incremented on each outClickablePath function call, used to have multiple clickable paths in a page
	elseif(cfCmpLeft($path,'*resourceBasePath*')){
		$path=str_replace('*resourceBasePath*',substr(strrchr('/'.$bPath,'/'),1),$path);
		$fromResourceBasePath=true;
	}
	$path=explode('/',$path);
	foreach ($path as $k=>$v) if($v=="") unset($path[$k]);
	if($bPath=='/') array_unshift($path,'/');

	if(count($path)==0 || (!$fromResourceBasePath && (strlen($path[0])!=2 || substr($path[0],1,1)!=':'))) return false;
	// Computer root label
	if($bPath=='computerRoot') $output.='<a href="javascript:cPath(\'computerRoot\')" class="link">'.outImage(outIcon('computerRoot'),false,'title="'.cfCaption('explorerComputerRoot').'"','vertical-align:middle').'</a> /';
	foreach ($path as $key=>$value){
		if($value=='/')
			$output.='<a href="javascript:cPath('.$clickablePathIndex.','.$key.','.(($fromResourceBasePath)?'true':'false').')" id="cp'.$clickablePathIndex.'_'.$key.'" class="link">'.outImage(outIcon('home'),false,false,'vertical-align:middle').'</a>'.((count($path)>1)?'/':'');
		else
			$output.='<a href="javascript:cPath('.$clickablePathIndex.','.$key.','.(($fromResourceBasePath)?'true':'false').')" id="cp'.$clickablePathIndex.'_'.$key.'" class="link">'.(($value=='.')?outImage(outIcon('home'),false,false,'vertical-align:middle'):cfUTF8Encode($value)).'</a>/';
	}
	return $output;
}

/**
 * @desc Insert form (file input + upload button) and script for file upload
 *
 * @param integer $inputWidth : optional : width of file input
 * @param string $ackJSFunction : optional : name of javascript function, located in parent window, called when upload is completed
 * @param bool $submitButton : optional : submit button state : 'enabled', 'disabled', 'none'
 * @param bool singleUpload : optional : set to true if as single upload window may be opened,
 * 										 set to false if multiple simultaneous downloads are allowed
 * @param string $extraInnerHTML : extra HTML code to be embeded in form
 * @param array $options: 'allowedDestDir': set to true to override upload access control,
 * 							'noDragDrop': set to true to forbid drag & drop upload (used in selfBlog)
 * 							'maxFiles': set to limit number of uploaded files at a time
 * 							'extensions': array of allowed extensions ('jpg'=>1, 'gif'=>1...)
 * 							'onlyClassical': don't show html5/flash uploads
 * 							'autoSubmit': submit form as soon as a file has been selected (works only with classical)
 */
function outUploadForm($inputWidth=20,$ackJSFunction=false,$submitButton='enabled', $singleUpload=false, $extraInnerHTML=false,$options=false){
	$dirPermissions=cfFileRights(cfRGetVar('efCurrentDirectory'));

	if(!isset($options['extensions'])) $options['extensions']=cfRGetVar('extensionFilter'); // Extension filter

	/**
	 * Classical file upload and flash upload button
	 */

		// Pre-upload form
	?>
<div id="htmlUploadStuff" style="display:none">
<?php if(!@$options['compact']) echo '<br><br>'; ?>
	<form name="preUploadForm" method="post" enctype="multipart/form-data" action="upload.php" style="display:none">
	<input type="hidden" name="preUlDir" id="preUlDir" value="<?php echo cfUTF8Encode(cfResourceRelativePath(cfRGetVar('efCurrentDirectory')));?>">
	<input type="hidden" name="preUlFile" id="preUlFile" value="">
	<input type="hidden" name="overwrite" value="0">
	<input type="hidden" name="resId" value="<?php echo $_SESSION['activeResourceId'];?>">
	</form>
	<?php // Upload form  ?>
	<form name="uploadForm" method="post" enctype="multipart/form-data" action="upload.php">
	<input type="hidden" name="UPLOAD_IDENTIFIER">
	<input type="hidden" name="ulDir" id="ulDir" value="<?php echo cfUTF8Encode(cfResourceRelativePath(cfRGetVar('efCurrentDirectory')));?>">
	<input type="hidden" name="postUlFile" id="postUlFile" value="">
	<input type="hidden" name="resId" value="<?php echo $_SESSION['activeResourceId'];?>">
	<input type="hidden" name="ulTransferId" id="ulTransferId" value="">
	<?php
	// ack JS function
	if($ackJSFunction) echo '<input type="hidden" name="ulAckJSFunction" id="ulAckJSFunction" value="'.$ackJSFunction.'">';
	// File input
	?>
	<input class="textInput<?php if(cfGetBrowser()=='gecko') echo ' fileInput';// Gecko workaround/file unpload bg is always white?>" type="file" id="ubId" name="ulInput" size="<?php echo $inputWidth;?>" <?php if(@$options['autoSubmit']) echo ' onchange="initUpload()"';?>>
	<?php
	if($submitButton!='none'){
		// Submit button
		if($submitButton=='enabled')
			echo outButton(cfCaption('genSend'),'javascript:initUpload();',outIcon('ok16'),false,'uploadFormSubmitButton');
		else
			echo outButtonDisabled(cfCaption('genSend'),'javascript:initUpload();',outIcon('ok16'),false,'uploadFormSubmitButton');
		// Multiple uploads button
		if(!$singleUpload && cfBGetVar('flash')) {
			if($submitButton=='enabled') echo '<br>&nbsp;'.outButton(cfCaption('multiUpload'),'javascript:flashUploadOpen()',outIcon('uploadMult'),false,'flashUploadOpenBt');
			else echo '<br>'.outButtonDisabled(cfCaption('multiUpload'),'javascript:flashUploadOpen()',outIcon('uploadMult'),false,'flashUploadOpenBt');
		}
	}
	// Extra HTML
	echo $extraInnerHTML;
	?>
</form>
<script type="text/javascript">
var windowId,uploadAllowedDir=<?php echo (int)(@$options['allowedDestDir'] || @$dirPermissions['upload']);?>;
var uploadExtensions=<?php if(@$options['extensions']) echo json_encode($options['extensions']); else echo '""';?>;
var uploadMaxFiles=<?php echo (int)@$options['maxFiles'];?>;
function initUpload(){
	var ui=dgi('ubId');
	if((ui.value)=="") {alert('<?php echo cfCaptionJS('explorerUploadNoFile');?>'); return;}
	if(!uploadCheckExt(ui.value)) return;
	dgi('postUlFile').value=dgi('preUlFile').value=ui.value;
	var dcf=D.comForm;
	if(!dcf) return initUploadProceed('proceed');
	dgi('ulDir').value=dgi('preUlDir').value=dcf.data1.value;
	dcf.data2.value=ui.value.replace(/c:[\\//]fakepath[\\//]/i,'');
	dcf.data3.value='uploadCheckOverwrite';
	asyncSubmit();
}
// Check file extension
function uploadCheckExt(name){
	var f=0,ex,e;
	if(ex=uploadExtensions){
		name=name.substr(name.lastIndexOf('.')+1).toLowerCase();
		for(e in ex) if(name==e) f++;
	}
	else f=1;
	return f;
}
function initUploadProceed(action,caption){
	if(action=='forbidden') return alert(caption);
	if(action=='confirm' && !confirm(caption)) return;
	var now=new Date();
<?php
	if($singleUpload) echo 'windowId="upload'.$_SESSION['activeResourceId']."\";\n"; else echo 'windowId="upload"+now.getTime();';

	// Frameless display
	if(!cfTGetVar('frames')){?>D.uploadForm.submit();<?php }
	else{
?>
	var noPopup=<?php echo (int)cfHGetVar('noPopup');?>,
	dest=((W.parent)?parent:W),
	width=220,height=190,w,h,t;

	if(noPopup && W.parent && W.parent.winOpenLocation){
		w=dest.dd.getWndW();
		h=dest.dd.getWndH();
		dest.winOpenLocation("",windowId,"...",winMyId);
		with(dest.win(windowId)){
			moveTo(Math.floor((w-width)/2),Math.floor((h-height)/2));
			resizeInnerTo(width,height); freezeSize(); show(); bringToFront();
		}
		t=dest.dgi(windowId+'_if').name;
	}
	else{
		w=screen.availWidth;
		h=screen.availHeight;
		W.open("about:blank",windowId,"toolbar=no,location=no,status=no,scrollbars=no,resizable=no,menuBar=no,width="+width+", height="+height+", top="+Math.floor((h-height)/2)+", left="+Math.floor((w-width)/2));
		t=windowId;
	}

	D.preUploadForm.target=D.uploadForm.target=t;
	D.preUploadForm.submit();
<?php
}
?>
}
function doUploadFile(uploadId){
	if(!uploadId) return;
	dgi('ulTransferId').value=uploadId;
	dgn('UPLOAD_IDENTIFIER').value=uploadId;
	try{frames["transfersIFrame"].transfersRefresh(1);} catch(e){}
	D.uploadForm.submit();
}
function refreshPage(){location=location.href;}
</script>
</div>
<?php

//if(@$options['onlyClassical']) return;

/**
 * FLASH upload HTML and script
 */
?>
<script type="text/javascript">
var initFlashUploadDone=0;
function initFlashUpload(vd){
	if(initFlashUploadDone) return;
	initFlashUploadDone=1;
	dgi('htmlUploadStuff').style.display='none';
	dgi('flashUploadStuff').style.display='';
}
function flashUploadOpen(){
	windowId="upload"+(new Date().getTime());
	var width=600, height=200;

	if(noPopup && parent && parent.winOpenLocation){
		parent.winOpenLocation("<?php echo dirname($_SERVER['SCRIPT_NAME']);?>/uploadFlash.php",windowId,"<?php echo cfCaption('genFileUpload');?>",winMyId);
		with(parent.win(windowId)){resizeInnerTo(width,height); center(); show(); bringToFront();}
	}
	else{
		var w=screen.availWidth,h=screen.availHeight;
		W.open("<?php echo dirname($_SERVER['SCRIPT_NAME']);?>/uploadFlash.php",windowId,"toolbar=no,location=no,status=no,scrollbars=auto,menuBar=no,width="+width+", height="+height+", top="+Math.floor((h-height)/2)+", left="+Math.floor((w-width)/2));
	}
}
</script>
<?php


	/**
	 * HTML5 Drag & Drop upload HTML and script
	 */
?>
<div id="htmlUploadDDStuff" style="display:none">
<table style="width:100%"><tr><td>
<div id="uploadDDDiv" class="dropTarget">
<div>
<?php
if(@$options['compact']){
	echo outImage(outIcon('dropFiles'),false,false,'vertical-align:middle;margin:0 0.5em 0 0.5em;float:left').cfCaption('uploadDragDrop');
}
else{
	echo outImage(outIcon('dropFiles'),false,false,'vertical-align:middle;margin-right:1em').cfCaption('uploadDragDrop');
}
?>
</div>
</div>
<?php // Upload progress bar template ?>
<div id="uploadDDPBTemplate" style="display:none"><?php echo outProgressBar(0,'100%','CAPT','PBID',false)?></div>
</td><td style="width:1px;text-align:right;vertical-align:top">
<?php

	// Add control buttons
	$disabled=(!cfRGetVar('uploadAllowed') || (!@$dirPermissions['upload'] && !@$options['allowedDestDir']));

	// Cancel upload button
	echo outButtonSmall((@$options['compact'])?'':cfCaption('genCancel'),'javascript:uploadDD[\'uploadDDDiv\'].cancelAll()',outIcon('eraser'),cfCaption('genCancel'),'uploadDDCancelBt').'<br><br>';

	// Toggle mode button
	if(@$options['compact'])
		echo outButtonSmall('','javascript:uploadDD[\'uploadDDDiv\'].hide()',outIcon('exit'),cfCaption('standardMode'),'uploadDDHideBt').'<br>';
	else
		echo '&nbsp;'.outButtonSmall(cfCaption('standardMode'),'javascript:uploadDD[\'uploadDDDiv\'].hide()',outIcon('exit'),cfCaption('standardMode'),'uploadDDHideBt','style=""'); // &nbsp; is a display WA


?>
</td>
</tr>
</table>
</div>
<script type="text/javascript">
wl.capt.transferCanceled="<?php echo cfCaptionJS('transfersCanceled');?>";
wl.capt.transferError="<?php echo cfCaptionJS('explorerErrorFileSend2');?>";
wl.capt.genCancel="<?php echo cfCaptionJS('genCancel');?>";
wl.capt.replaceFile="<?php echo cfCaptionJS('genFileAlreadyExistReplace');?>";
wl.capt.genYes="<?php echo cfCaptionJS('genYes');?>";
wl.capt.genNo="<?php echo cfCaptionJS('genNo');?>";
wl.capt.dontAskAnymore="<?php echo cfCaptionJS('dontAskAnymore');?>";
<?php echo cfJSCaptionVar("genConfirmCopy"); ?>
wl.ico.alert="<?php echo outIcon('dontAskAnymore');?>";
var overwriteConfirmChecked=<?php echo ((cfRGetVar('overwriteConfirmChecked'))?'1':'0');?>; // Server-side default prompt storage
var uploadDragDropDisabled=<?php echo (int)@$options['noDragDrop']; ?>; // Disabled drag & drop upload
<?php
// Add file mime type filter
echo 'var uploadDDFileTypeFilter=';
if(cfRGetVar('fileTypeFilter')){
	echo '{';
	foreach (cfRGetVar('fileTypeFilter') as $v) echo ((@$nb++)?',':'')."'".$v."':1";
	echo '};';
}
else echo '0;';

// And file extension filter
echo 'var uploadDDMaxFiles='.(int)@$options['maxFiles'].';';
echo 'var uploadDDExtensionFilter=';
if(@$options['extensions']){
	echo '{';
	foreach ($options['extensions'] as $k=>$v) {
		if(is_numeric($v)) $v=$k; // Dirty format fix
		echo ((@$nb++)?',':'')."'".$v."':1";
	}
	echo '};';
}
else echo '0;';

// Enable drag & drop upload div if browser supports it
	?>
(function(){
	var ddc=wl.uploadDragDropCapable() && !uploadDragDropDisabled;
	dgi('htmlUploadDDStuff').style.display=(ddc)?'':'none';
	dgi('htmlUploadStuff').style.display=(ddc)?'none':'';
	if(ddc) {
		var s=wl.includeScript('uploadDD.js',1);
		s.onload=function(){setUploadReceiver('uploadDDDiv',0)}
	}
})();
</script>
	<?php
}

/**
 * @desc display subdirectories and upperdir browser
 *
 * @param string $dir :
 * @param array $filesArray : efGetDirContent formated file array
 * @param string $class : table class
 * @param integer $rows : number of rows
 */
function outDirectoryTable($dir, $filesArray, $class=false,$rows=4,$extraHTML=false,$extraStyle=false){
	$outputStarted=false;

	// "Up" control
	if(cfFileRights($dir."/..",'state')=='authorized' && $dir!='computerRoot') {
		echo outDivFrame($class,$extraHTML,$extraStyle);
		if($class) echo '<div class="'.$class.'Header">'; else echo '<div>';
		echo outThemeImage(outIcon('browse'),false,false,'vertical-align:middle; margin-right:0.5em;').cfCaption('explorerBrowse').'</div>';
		echo '<table style="width:100%;">';
		echo '<tr><td align="left">';
		echo outButton(cfCaption('explorerParentDir'),'javascript:go(\'..\')',outIcon('icoUp'),false,false,false,true).'</td></tr>';
		$outputStarted=true;
	}

	// Sub directories
	$dirIndex=0;
	foreach ($filesArray as $value){
		if($value['type']=='dir' || $value['type']=='drive'){
			// Frame
			if(!$outputStarted) {
				$outputStarted=true;
				if($class) echo outDivFrame($class,$extraHTML,$extraStyle).'<div class="'.$class.'Header">'; else echo outDivFrame($class).'<div>';
				echo outImage(outIcon('browse'),false,false,'vertical-align:middle; margin-right:0.5em;').cfCaption('explorerBrowse').'</div>';
				echo '<table style="width:100%;">';
			}
			if(($dirIndex/$rows)==round($dirIndex/$rows)) echo "<tr>\n";
			echo '<td align="left" width="'.round((100/$rows),0).'%">';

			// Folder button
			$icon=$value['icon'];
			if($value['type']=='dir') $text=$value['name']; else $text=$value['name'].' '.$value['volumeName'];
			echo outButton(cfUTF8Encode($text),'javascript:go(\'dirId'.$dirIndex.'\')',$icon,false,false,'style="width:80%"',true);
			//echo outbt(cfUTF8Encode($text),$icon,'javascript:go(\'dirId'.$dirIndex.'\')',false,false,false,'up','230px','left');
			echo '<span style="display:none;" id="dirId'.$dirIndex.'">'.cfProtectFilename($value['name'])."</span></td>\n";
			$dirIndex++;
			if(($dirIndex/$rows)==round($dirIndex/$rows)) echo "</tr>\n";
		}
	}
	if($dirIndex % $rows > 0){
		for($i=($dirIndex % $rows); $i<$rows-1; $i++){
			echo'<td></td>';
		}
	}
	if($outputStarted) echo '</table></div>';
}



/**
 * @desc Multiple file download file list and buttons
 *
 * @param string $class : class of list
 */

/**
 * @desc : Multiple file download file list and buttons / playlist
 *
 * @param string $class : class of outer HTML frame
 * @param mixed $showMusicControls : false if not a playlist, 'folder' if folder-browsing music resource (playlist),
 * 			'db' if not folder-browsing music resource (playlist)
 * @param boolean $compactDisplay: true if captions must be removed from buttons
 * @param string $extraHTML: outer HTML frame extra HTML
 * @param string $extraStyle: outer HTML frame extra style
 * @param string $extraStyleDrop: drop zone frame extra style
 * @param string $innerFrameClass: inner HTML frame CSS Class
 * @param array $extraOptions: 'noCollapse'=>1 to forbid toggle collapse button, 'noOuterDiv' not to include outer div, 'inlinePlayerFold' to fold with inline audio player, 'm3u" to display m3u direct play button
 * @return string HTML code
 */
function outMultipleDownloadList($class, $showMusicControls, $compactDisplay=false,$extraHTML=false, $extraStyle=false, $extraStyleDrop=false, $innerFrameClass=false, $extraOptions=array()){
	$output='';
	if(cfRGetVar('mDownloadList') && count(cfRGetVar('mDownloadList'))) $empty=false; else $empty=true;

	// Main outer div
	if($compactDisplay==='vertical' || cfRGetVar('displayMultipleDownload')) $folding=''; else $folding=' collapsed';
	if(!isset($extraOptions['noOuterDiv'])) $output.=outDivFrame($class.$folding,'id="mdlOuterFrame" '.$extraHTML,$extraStyle);

	// Reloading icon
	$l=outThemeImage(outIcon('loading'), false, 'id="mdlLoadingIcon"','vertical-align:middle;display:none;margin-right:0.5em');

	// Play all (music only)
	if($showMusicControls) {
		// playlist icon and play all button
		$l.=outThemeImage(outIcon('icom3u'), false, 'id="mdlIcon"','vertical-align:middle;margin-right:0.5em');
		$l.=outButton(cfCaption('explorerAudioPlayPlaylist'),'javascript:plPlayAll('.DEFAULT_AUDIO_WINDOW_WIDTH.','.DEFAULT_AUDIO_WINDOW_HEIGHT.')',outIcon('play'),false,'plPlayAllBtE',(($empty)?'style="display:none"':''));
		$l.=outButtonDisabled(cfCaption('explorerAudioPlayPlaylist'),false,outIcon('play'),false,'plPlayAllBtD',(($empty)?'':'style="display:none"'));

	}
	else{
		// mdl icon
		$tooltip=(($compactDisplay=='vertical')?' onmouseover="tooltip(this,\''.cfCaption('explorerMultipleDownload').' \'+dgi(\'mDlTotals\').innerHTML,1)"':'');
		$l.=outThemeImage(outIcon('mdli'), false, 'id="mdlIcon"'.$tooltip,'vertical-align:middle;margin-right:0.5em').'&nbsp;';
		// mdl title
		if($compactDisplay!=='vertical') $l.=(($compactDisplay)?cfCaption('explorerMDShort'):cfCaption('explorerMultipleDownload'));

		// mdl total size
		if(cfHGetVar('mDownloadPrecomputeSize')){
			list($nbFiles,$totalSize)=cfRGetVar('mDownloadTotals');
			$l.=' <span id="mDlTotals"'.(($compactDisplay=='vertical')?' style="display:none"':'').'> ('.cfCaption('explorerFiles',$nbFiles).' / '.efFileSizeFormat($totalSize).')</span>';
		}
	}

	$r='';
	// Shuffle playlist
	if($showMusicControls) {
		if(isset($extraOptions['m3u']) && cfBGetVar('playlistFormat')) $r.=outButton(false,'javascript:plPlayAll(0,0,\'forceM3U\')',outIcon('m3u'),cfCaption('musicCfgExternal'),'plPlayM3UBt');

		$r.=outButton('','javascript:plToggleShuffle(\'ns\')',outIcon('random'),cfCaption('explorerShuffle'),'plShuffleBtOn',((!cfRGetVar('shufflePlaylist'))?'style="display:none;margin-right:1em"':'style="margin-right:1em"'));
		$r.=outButton('','javascript:plToggleShuffle(\'s\')',outIcon('randomNo'),cfCaption('explorerShuffleNo'),'plShuffleBtOff',((!cfRGetVar('shufflePlaylist'))?'style="margin-right:1em"':'style="display:none;margin-right:1em"'));
	}


	// if list is not empty, show enabled buttons
	if(cfRGetVar('mDownloadList') && count(cfRGetVar('mDownloadList'))){
		if(cfRGetVar('multipleDownloadAllowed')){
			// Zip download
			$r.=outButton(((!$compactDisplay)?cfCaption('explorerStartDownload'):''),'javascript:mDownload();',outIcon('dl2'),(($compactDisplay)?cfCaption('explorerStartDownload'):''),'mDlDownloadE')
			  .outButtonDisabled(((!$compactDisplay)?cfCaption('explorerStartDownload'):''),false,outIcon('dl2'),(($compactDisplay)?cfCaption('explorerStartDownload'):''),'mDlDownloadD','style="display:none"');
			// Bittorrent download (if bittorrent Enabled and not db music playlist)
			if(cfHiGetVar('bittorrentEnabled') && $showMusicControls!='db')
				$r.=outButton(((!$compactDisplay)?cfCaption('genDownload').' / '.cfCaption('bittorrent'):''),'javascript:mBtDownload();',outIcon('bittorrent'),(($compactDisplay)?cfCaption('genDownload').' / '.cfCaption('bittorrent'):''),'btDlDownloadE')
				.outButtonDisabled(((!$compactDisplay)?cfCaption('bittorrent'):''),false,outIcon('bittorrent'),(($compactDisplay)?cfCaption('bittorrent'):''),'btDlDownloadD','style="display:none"');
		}

		// Clear list
		$r.=outButton(((!$compactDisplay)?cfCaption('explorerClearList'):''),'javascript:mSuppAll();',outIcon('eraser'),(($compactDisplay)?cfCaption('explorerClearList'):''),'mDlClearE','style="margin-left:1em"')
		  .outButtonDisabled(((!$compactDisplay)?cfCaption('explorerClearList'):''),false,outIcon('eraser'),(($compactDisplay)?cfCaption('explorerClearList'):''),'mDlClearD','style="display:none;margin-left:1em"');

		  // Add all files in current directory to list
		if($showMusicControls!='db')
			$r.=outButton(((!$compactDisplay)?cfCaption('explorerAudioSelectAll'):''),'javascript:mAddAll()',outIcon('addMult'),(($compactDisplay)?cfCaption('explorerAudioSelectAll'):''),'mDlAddAllE');
	}
	// else, show disabled buttons
	else{
		if(cfRGetVar('multipleDownloadAllowed')){
			// Zip download
			$r.=outButton(((!$compactDisplay)?cfCaption('explorerStartDownload'):''),'javascript:mDownload();',outIcon('mdlS'),cfCaption('explorerStartDownload'),'mDlDownloadE','style="display:none"')
			  .outButtonDisabled(((!$compactDisplay)?cfCaption('explorerStartDownload'):''),false,outIcon('dl'),false,'mDlDownloadD');
			// Bittorrent download (if bittorrent Enabled and not db music playlist)
			if(cfHiGetVar('bittorrentEnabled') && $showMusicControls!='db')
				$r.=outButton(((!$compactDisplay)?cfCaption('bittorrent'):''),'javascript:mBtDownload();',outIcon('bittorrent'),cfCaption('explorerStartDownload').' ('.cfCaption('bittorrent').')','btDlDownloadE','style="display:none"')
				  .outButtonDisabled(((!$compactDisplay)?cfCaption('bittorrent'):''),false,outIcon('bittorrent'),false,'btDlDownloadD');
		}
		// Clear list
		$r.=outButton(((!$compactDisplay)?cfCaption('explorerClearList'):''),'javascript:mSuppAll();',outIcon('eraser'),cfCaption('explorerClearAll'),'mDlClearE','style="display:none;margin-left:1em"')
		  .outButtonDisabled(((!$compactDisplay)?cfCaption('explorerClearList'):''),false,outIcon('eraser'),false,'mDlClearD','style="margin-left:1em"');

		// Add all files to list
		if($showMusicControls!='db') $r.=outButton(((!$compactDisplay)?cfCaption('explorerAudioSelectAll'):''),'javascript:mAddAll();',outIcon('addMult.png'),cfCaption('addCurrentDirectoryContent'),'mDlAddAllE');
	}

	// Store playlist button
	if(cfRGetVar('storePlaylistAllowed')) $r.=outButton((($compactDisplay)?'':cfCaption('storePlaylistDesc')),'javascript:plStore()',outIcon('favorite'),(($compactDisplay)?cfCaption('storePlaylistDesc'):''),'plStoreBt');

	// Help button
	if(!$showMusicControls) $r.=outHelpButton('helpMD');

	// Toggle collapse button
	if(isset($extraOptions['inlinePlayerFold'])) $r.='<b style="padding-left:3em"></b>'.outButtonToggleFrameCollapse(array('mdlOuterFrame','inlinePlayer'),'displayMultipleDownload');
	else if($compactDisplay!=='vertical' && !isset($extraOptions['noCollapse'])) $r.='<b style="padding-left:3em"></b>'.outButtonToggleCollapse('mdlInnerFrame','displayMultipleDownload');

	// Header
	$extraHTML='';
	//if(isset($extraOptions['inlinePlayerFold'])) $extraHTML.=' onmousedown="if(dgi(\'mdlOuterFrameBtMinus\').style.display!=\'none\') dgi(\'mdlOuterFrameBtMinus\').onmousedown(); else dgi(\'mdlOuterFrameBtPlus\').onmousedown()"';
	$output.=outFrameHeaderTable($class.'Header',$l,$r,$extraHTML,'vertical-align:top;white-space:nowrap');

	// Files frame
	if($compactDisplay==='vertical')
		$output.=outCollapsableFrame("mdlInnerFrame", false,'class="'.$innerFrameClass.'"' ,false,false);
	else{
		$output.=outCollapsableFrame("mdlInnerFrame", 'displayMultipleDownload',(($innerFrameClass)?'class="'.$innerFrameClass.'"':''),'hidden');
	}

	// Music playlist inner part
	if($showMusicControls){
		// Add item to indicate JS wether it's a playlist or not
		$output.='<p style="display:none" id="mDlPlaylist"></p>';
		$output.=outDivFrame('frame2','id="mDlPlaylistInnerFrame"');
		$output.='<div id="mDlList" style="position:relative;width:100%;height:'.DEFAULT_INLINEPLAYER_HEIGHT.'; overflow:auto;'.$extraStyleDrop.'" class="dropTarget" '.outEHoverFrameAttr().'>';
		$output.=outMultipleDownloadListInnerHTML();
		$output.='</div></div>';
	}
	// Regular mdl inner part
	else{
		$output.='<div style="position:relative;height:8em;text-overflow:ellipsis;overflow:auto;'.$extraStyleDrop.'" id="mDlList" class="dropTarget" '.outEHoverFrameAttr().'>';
		/**
		 * CONTENT
		 */
		$output.=outMultipleDownloadListInnerHTML();
		$output.='</div>';
	}
	$output.='</div>'; // end of mdlInnerFrame
	if(!isset($extraOptions['noOuterDiv'])) $output.='</div>'; // end of main outer div

	// Add frame to draggable items list
	cfDragAddItem("mDlList",'+NO_DRAG+RECEIVER+AUTOSIZE+EASYDROP');
	return $output;
}

/**
 * @desc : multiple file download file list (used both by outMultipleDownloadList (below)
 * 			and by async call (efMultipleDownloadAdd, efMultipleDownloadSupp, efMultipleDownloadClear))
 *
 */
function outMultipleDownloadListInnerHTML(){
	$output=''; $i=0;
	if(cfGetBrowser()=='ie') $w=100; else $w=98;
	if(cfRGetVar('mDownloadList')){
		foreach (cfRGetVar('mDownloadList') as $name=>$completeFileName){
		$output.='<div class="eTC" id="mDlDiv'.$i.'" style="width:'.$w.'%;white-space:nowrap" eH=1>';
		$output.=outButtonSmall('','javascript:mSupp(\'mDl'.$i.'\')',outIcon('eraser'));
		if(is_dir($completeFileName)) $icon='/gfx/fi/folder.gif'; else $icon=efIcon($completeFileName);
		$output.=outImage($icon,false,false,'vertical-align:middle;margin-left:0.5em;');
		$output.='<span id="mDl'.$i.'" style="margin-left:0.5em">'.cfUTF8Encode($name).'</span></div>';
		$i++;
		}
	}
	else $output.='<div style="text-align:center; width:'.$w.'%; display:block; position:relative; top:45%">'.cfCaption('explorerNoFile').'</div>';
	return $output;
}

/**
 * @desc insert div stuff for image preview on view icon hover
 *
 * @return srting : HTML code
 */
function outFilePreviewDiv(){
	cfDragAddItem("previewFrame");
	return '<div id="previewFrame" style="position:absolute;top:-100px;"></div>';
}

/**
 * @desc display Inline player HTML code
 * @return void
 *
 */
function outDisplayInlineAudioPlayer($options=array()){
	require(INCLUDE_DIR.'viewFunctions.php');

	//outShadowBefore(DEFAULT_INLINEPLAYER_WIDTH.'px');
	if(cfRGetVar('displayMultipleDownload')) $folding=''; else $folding=' collapsed';
	echo outDivFrame('frame1'.$folding,'id="inlinePlayer"','padding-bottom:0');

	// Player's header
	//$l=outThemeImage(dirname($_SERVER['PHP_SELF']).'/player.png',false,false,'vertical-align:middle; margin-right:1em').cfCaption('explorerAudioPlayer').outButton('','#',false,false,false,'style="visibility:hidden"');
	//$l=outBt('',outIcon('html5Audio'),"javascript:var s=((btState('fH5P'))=='up')?1:0;void(wl.asr.send('forceHTML5AudioPlayer='+s))",'fH5P','HTML 5 Audio',false,((cfHGetVar('forceHTML5AudioPlayer'))?'on':'up'));
	$l='<div style="position:absolute;left:0;top:0">';
	$l.=outBlockItem('fH5P',outImage(outIcon('flashAudio')),false,'padding:1px',array('onclick'=>"html5AP.show(0);wl.asr.send('forceHTML5AudioPlayer=0')"),((cfHGetVar('forceHTML5AudioPlayer'))?'unsel':'sel'));
	$l.=outBlockItem('fH5P',outImage(outIcon('html5Audio')),false,'padding:1px',array('onclick'=>"html5AP.show(1);wl.asr.send('forceHTML5AudioPlayer=1')")	,((cfHGetVar('forceHTML5AudioPlayer'))?'sel':'unsel'));
	$l.='</div>';

	// Birtate selection list
	$mbr=cfRGetVar('maxOutputBitRate'); if($mbr==='original') $mbr=999;
	$br=cfRGetVar('outputBitRate');
	$r='<select onchange="wl.asr.send({\'outputBitRate\':this.value})"'.outToolTip(cfCaption('musicCfgSrcQuality')).'>';
	$r.='<option value="32"'.(($br==32)?' selected':'').'>'.cfCaption('explorerAudio32').'</option>';
	$r.='<option value="64"'.(($br==64)?' selected':'').'>'.cfCaption('explorerAudio64').'</option>';
	if($mbr>64) $r.='<option value="128"'.(($br==128)?' selected':'').'>'.cfCaption('explorerAudio128').'</option>';
	if($mbr>128) $r.='<option value="original"'.(($br=='original')?' selected':'').'>'.cfCaption('explorerAudioOriginal').'</option>';
	$r.='</select>';

	if($mbr>32)
		echo outFrameHeaderTable('frame1Header','',$r);
	else
		echo '<div class="frame1Header">&nbsp;</div>';

	// Folding frame
	echo outCollapsableFrame("inlinePlayerInnerFrame", 'displayMultipleDownload','class="frame2"','hidden');

	echo outDivFrame('dropTarget', 'id="inlinePlayerDiv" name="playListFrame"','height:'.DEFAULT_INLINEPLAYER_HEIGHT.';'); // Used to set same height than playlist frame
	echo outTableTransparent('frame2').'<tr><td width="1" style="vertical-align:top"><div style="position:relative;margin:-4px 0.5em 0.5em -4px ;width:88px;height:88px">';
	echo outImage('/gfx/icons/blankIcon.gif',false,'id="inlinePlayerImg"','position:relative;left:4px;top:4px;background:#4d4d4d');
	echo outImage(dirname($_SERVER['PHP_SELF']).'/../music2/artistBg.png',false,'','position:absolute;top:0;left:0');
	echo '</div>';
	echo '</td><td style="vertical-align:top">';
	echo '<b>'.cfCaption('explorerAudioArtist').'</b><div id="inlineArtist">-</div>';
	echo '<b>'.cfCaption('explorerAudioAlbum').'</b><div id="inlineAlbum">-</div>';
	echo '<b>'.cfCaption('explorerAudioTitle').'</b><div id="inlineTitle">-</div>';
	echo '</td></tr></table>';
	?><script type="text/javascript">function flashCommand(cmd,args) {<?php if(cfRGetVar('inlinePlayer')) echo 'if(cmd=="playSong") inlineUpdatePlayerInfo(args);';?>}</script><?php
	echo '</div></div>';
	// Player
	echo '<div id="playerDiv" class="foldableInlinePlayer">'.vfInsertAudioPlayer('','normal',false,'95%',25).'</div>';

	// Player selection controls (I know it's a weird place to insert those controls, but there is a problem with IE if it's in frame1Header
	echo $l;

	echo '</div>';
}

/*
 ***************************************************************************************************************************
 * Image gallery specific HTML controls output scripts
 ***************************************************************************************************************************
 */

/**
 * @desc Full thumbnails interval selection controls
 * @param integer $index: index of control set, used if more than one on the page
 * @param bool $async: true to asynchronously update files
 * @return string
 */
function outExplorerThumbnailsSelector($index=1,$async=false){
	if(cfIsAsync()) {
		$r='';
		$act='';
		// Show / hide selector control depending of number of files
		if(isset($_ENV['weezoResEfFiles']['nbFiles']))
			$act=cfAsyncXMLJSaction('dgi("thumbnailsSelector'.(($index!=1)?$index:'').'").style.display="'.((cfRGetVar('resourceLayout')=='useInterval' && $_ENV['weezoResEfFiles']['nbFiles']>10)?'':'none').'"');
	}
	else
		$r='<span style="white-space:nowrap'.((cfRGetVar('resourceLayout')=='useInterval' && efDirGetNbOfFiles($_ENV['weezoResEfFiles'])>10)?'':';display:none').'" id="thumbnailsSelector'.(($index!=1)?$index:'').'">';

	//previous thumbnails set
	$r.=outExplorerPrevThumbnails($async);
	// thumbnails 'from - to'
	$r.=outExplorerThumbnailsSelectList($index,$async);
	//next thumbnails set
	$r.=outExplorerNextThumbnails($async);

	// If async, return code needed for async update of controls
	if(cfIsAsync()) return cfAsyncXMLInnerHTMLbyId($r,'thumbnailsSelector'.(($index!=1)?$index:'')).$act;
	// Else, return controls embeded into known span
	return $r.'</span>';

}

/**
 * @desc Previous thumbnails button. Button is enabled or disabled depending on thumbnailsIndex value
 *
  * @return string : HTML code
 */
function outExplorerPrevThumbnails($async=false){
	if($async) $async=',1';
	if(cfRGetVar('itemsPerPage') && 0+cfRGetVar('thumbnailsIndex')>0)
		return outButton('', 'javascript:goThumbnail('.max(0,0+cfRGetVar('thumbnailsIndex')-cfRGetVar('itemsPerPage')).$async.');',outIcon('prev'));
	else return outButtonDisabled('',false,outIcon('prev'));
}

/**
 * @desc Next thumbnails button. Button is enabled or disabled depending on thumbnailsIndex, itemsPerPage and number of files values
 *
  * @return string : HTML code
 */
function outExplorerNextThumbnails($async=false){
	if($async) $async=',true';
	if(cfRGetVar('itemsPerPage') && cfRGetVar('thumbnailsIndex')+cfRGetVar('itemsPerPage')<efDirGetNbOfFiles($_ENV['weezoResEfFiles']))
		return outButton('', 'javascript:goThumbnail('.((int)cfRGetVar('thumbnailsIndex')+(int)cfRGetVar('itemsPerPage')).$async.');',outIcon('next'));
	else return outButtonDisabled('',false,outIcon('next'));
}

/**
 * @desc Display a <select> control with "from-to" thumbnails list
 *
  * @return string : HTML code
  * @param  $index optional : control index if multiple select list displayed in same page
 */
function outExplorerThumbnailsSelectList($index=1,$async=false){
	if($async) $async=',true';
	$output='<select name="newRange" id="efNewRange_'.$index.'" size="1" style="vertical-align:middle;width:8.5em" onchange="goThumbnail(this.value'.$async.')">';
	$nbTotalImages=efDirGetNbOfFiles($_ENV['weezoResEfFiles']);

	$compact=cfTGetVar('compactDisplay')||(cfRGetVar('fileTypeFilter')!=='image');

	$nbGroups=((cfRGetVar('itemsPerPage'))?efDirGetNbOfFiles($_ENV['weezoResEfFiles'])/cfRGetVar('itemsPerPage'):1);
	$output.='<option value="">'.cfCaption('explorerItemsPerPage').'</option>';
	$output.='<option value="ipp10">10</option>';
	$output.='<option value="ipp20">20</option>';
	$output.='<option value="ipp50">50</option>';
	$output.='<option value="ipp100">100</option>';
	$output.='<option value="ipp0">'.cfCaption('genAll').'</option>';
	if(cfRGetVar('itemsPerPage')>0){
		$output.='<option value="">&nbsp;</option>';
		for($i=0;$i<$nbGroups;$i++){
			if($i*cfRGetVar('itemsPerPage')==cfRGetVar('thumbnailsIndex')) $output .= '<option selected '; else $output .= '<option ';
			$output .= 'value="'.($i*cfRGetVar('itemsPerPage')).'">';
			$from=cfUTF8Encode(1+$i*cfRGetVar('itemsPerPage'));
			$to=cfUTF8Encode(((($i+1)*cfRGetVar('itemsPerPage'))>$nbTotalImages)?$nbTotalImages:(($i+1)*cfRGetVar('itemsPerPage')));
			if($compact) $output.=$from.' - '.$to.'</option>';
			else $output .= cfCaption('explorerImagesFromTo',$from,$to).'</option>';
		}
	}
	$output .= "</select>\n";
	return $output;
}

/**
 * @desc Display thumbnail <img>
 *
 * @param array $file
 * @param integer $fileIndex: index in
 * @param string $extraStyle
 * @param string $extraHTML
 * @param mixed $draggable: true, false, 'div' (to embed draggable image into a div)
 *
 */
function outThumbnail($file, $fileIndex, $extraStyle='', $extraHTML=false, $draggable=true){
	// Cache vars
	static $fade=-1; static $tnMWC=0; static $tnMHC=0;

	if($fade==-1) {
		$fade=cfBGetVar('imgFadeSteps');
		$tnMHC=$tnMWC=efThumbnailsMaxWidth();
	}

	$output='';$tnW=0;

	// Use icon instead of actual image (music icon...)
	if(isset($file['thumbnailIcon'])){
		$src=$file['thumbnailIcon'];
		if(isset($file['width'])) $tnW=$file['width'];
		if(isset($file['height'])) $tnH=$file['height'];
	}
	// Use pre-generated tn
	elseif(isset($file['thumbnail'])){
		// Compute thumbnail dimensions, using source image width & height
		if($tnMWC || $tnMHC){
			if(isset($file['width']) && isset($file['height']) && $file['width']*$file['height']!=0){
				list($tnW,$tnH)=cfThumbnailDimensions($file['width'],$file['height'],$tnMWC,$tnMHC);
			}
			// If unknown image w or h, get dimension from thumbnail file
			elseif (file_exists($file['thumbnail'])){
				list($tnW, $tnH, $type, $attr) = getimagesize($file['thumbnail']);
				if($tnW*$tnH>0) list($tnW,$tnH)=cfThumbnailDimensions($tnW,$tnH,$tnMWC,$tnMHC);
				else {
					$tnW=$tnMWC;
					$tnH=$tnMWC*3/4;
				}
			}
			else {
				$tnW=$tnMWC;
				$tnH=$tnMWC*3/4;
			}
		}
		// output image
		$src=cfExtImage($file['thumbnail'],$tnW,$tnH);
	}
	// Directly extract from file
	else{
		if(efFileType($file)=='image'){
			list($tnW, $tnH, $type, $attr) = getimagesize($file['completeFileName']);
			list($tnW,$tnH)=cfThumbnailDimensions($tnW,$tnH,$tnMWC,$tnMHC);
		}
		else {$tnW=$tnMWC;$tnH=$tnMHC;}
		$src=cfExtImage($file['completeFileName'],$tnW,$tnH);
	}
	// If preview image displayed in resource, add link to thumbnail
	if(cfRGetVar('previewDisplay')) {
		$extraStyle.=';cursor:pointer';
		$extraHTML.=' onclick="preview(\'id'.$fileIndex.'\')"';
		if(cfRGetVar('fileViewAllowed')) $extraHTML.=' onDblClick="view(\'id'.$fileIndex.'\')"';
	}
	elseif(cfRGetVar('fileViewAllowed')) {
		$extraStyle.=';cursor:pointer';
		$extraHTML.=' onclick="view(\'id'.$fileIndex.'\')"';
	}

	if($tnW>0) $dimStyle='width:'.floor($tnW).';height:'.floor($tnH).';'; else $dimStyle='';
	// Embed image into div
	if($draggable==='div'){
		$extraStyle.=';position:relative;height:'.$tnMHC.';padding-top:'.floor(($tnMHC-$tnH)/2);
		$output='<div name="_id'.$fileIndex.'" id="_id'.$fileIndex.'" style="'.$extraStyle.'" '.$extraHTML.'><img src="'.$src.'" alt="" style="'.$dimStyle.(($fade)?'visibility:hidden" onload="fade(this)"':'').'></div>';
	}
	else{
		if($fade && !$draggable) $extraHTML.=' onload="fade(this)"';
		if($fade) $extraStyle.=';visibility:hidden';
		//if($fade) $extraStyle.=';'.outCssOpacity(1);
		$output='<img name="_id'.$fileIndex.'" style="'.$dimStyle.$extraStyle.'" '.$extraHTML.' src="'.$src.'" alt="">';
	}


	// Add image to dragable items list
	if($draggable) cfDragAddItem("_id".$fileIndex,'+CURSOR_HAND+ELASTIC+TRANSPARENT+INERTIA'.(($fade && $draggable===true)?'+IMGFADEIN':''));
	return $output."\n";
}

/**
 * @desc Return name/date/size sort controls
 *
 * @return string
 */
function outFileSortControls($compact=false){

	// Context menu sort control
	if($compact){
		$r='<span class="link" style="" id="sortBt">'.cfCaption('genSortBy').'...'.outImage(outIcon('down'),false,false,'vertical-align:middle"').'</span>';

		$items='[';
		$items.=' ContextMenuItem("'.outIcon('sortNameAsc').'","'.cfCaption('explorerFilename').'","srt(\'name\',\'asc\')")';
		$items.=',ContextMenuItem("'.outIcon('sortNameDesc').'","'.cfCaption('explorerFilename').'","srt(\'name\',\'desc\')")';
		$items.=',ContextMenuItem("sep")';
		$items.=',ContextMenuItem("'.outIcon('sortSizeAsc').'","'.cfCaption('genSize').'","srt(\'size\',\'asc\')")';
		$items.=',ContextMenuItem("'.outIcon('sortSizeDesc').'","'.cfCaption('genSize').'","srt(\'size\',\'desc\')")';
		$items.=',ContextMenuItem("sep")';
		$items.=',ContextMenuItem("'.outIcon('sortDateAsc').'","'.cfCaption('explorerDate').'","srt(\'date\',\'asc\')")';
		$items.=',ContextMenuItem("'.outIcon('sortDateDesc').'","'.cfCaption('explorerDate').'","srt(\'date\',\'desc\')")';
		$items.=',ContextMenuItem("sep")';
		if(cfGGetVar('ratingAllowed') && cfRGetVar('ratingAllowed')) {
			$items.=',ContextMenuItem("'.outIcon('sortRatingAsc').'","'.cfCaption('rating').'","srt(\'rating\',\'asc\')")';
			$items.=',ContextMenuItem("'.outIcon('sortRatingDesc').'","'.cfCaption('rating').'","srt(\'rating\',\'desc\')")';
		}
		$items.=']';

		$r.='<script type="text/javascript">';
		$r.="ContextMenuAttach(dgi('sortBt'),".$items.",{'align':'right','leftClick':1,'button':1,'style':{'width':130}})";
		$r.='</script>';

	}
	elseif($compact){
		$r='<span style="top:2px;position:relative">';
		// Name
		$r.='<span '.outToolTip(cfCaption('genSortBy').cfCaption('genSeparator').cfCaption('explorerFilename')).'>';
			$r.=outImage(outIcon('sortName'));
			$r.=outImage(outIcon('sup'),'javascript:srt(\'name\',\'asc\');',false).outImage(outIcon('sdown'),'javascript:srt(\'name\',\'desc\')',false,'margin-right:1em');
		$r.='</span>';
		// Size
		$r.='<span '.outToolTip(cfCaption('genSortBy').cfCaption('genSeparator').cfCaption('genSize')).'>';
			$r.=outImage(outIcon('sortSize'));
			$r.=outImage(outIcon('sup'),'javascript:srt(\'size\',\'asc\');',false).outImage(outIcon('sdown'),'javascript:srt(\'size\',\'desc\')',false,'margin-right:1em');
		$r.='</span>';
		// Date
		$r.='<span '.outToolTip(cfCaption('genSortBy').cfCaption('genSeparator').cfCaption('explorerDate')).'>';
			$r.=outImage(outIcon('sortDate'));
			$r.=outImage(outIcon('sup'),'javascript:srt(\'date\',\'asc\');',false).outImage(outIcon('sdown'),'javascript:srt(\'date\',\'desc\')',false,'margin-right:1em');
		$r.='</span>';
		// Note
		if(cfGGetVar('ratingAllowed') && cfRGetVar('ratingAllowed')) {
			$r.='<span '.outToolTip(cfCaption('genSortBy').cfCaption('genSeparator').cfCaption('rating')).'>';
				$r.=outImage(outIcon('sortRating'));
				$r.=outImage(outIcon('sup'),'javascript:srt(\'rating\',\'asc\');',false).outImage(outIcon('sdown'),'javascript:srt(\'rating\',\'desc\')',false,'margin-right:1em');
			$r.='</span>';
		}
		$r.='</span>';
	}
	else{
		$r='<span class="smallFont" style="white-space:nowrap">'.cfCaption('explorerFilename').outImage(outIcon('sup'),'javascript:srt(\'name\',\'asc\');',false,'margin-left:5px').outImage(outIcon('sdown'),'javascript:srt(\'name\',\'desc\')',false,'margin-right:1em');
		$r.=cfCaption('genSize').outImage(outIcon('sup'),'javascript:srt(\'size\',\'asc\');',false,'margin-left:5px').outImage(outIcon('sdown'),'javascript:srt(\'size\',\'desc\')',false,'margin-right:1em');
		$r.=' '.cfCaption('explorerDate').outImage(outIcon('sup'),'javascript:srt(\'date\',\'asc\');',false,'margin-left:5px').outImage(outIcon('sdown'),'javascript:srt(\'date\',\'desc\')',false).'</span>';
		if(cfGGetVar('ratingAllowed') && cfRGetVar('ratingAllowed')) $r.=' '.outImage(outIcon('rating'),false,false,'vertical-align:middle;margin-left:10px').outImage(outIcon('sup'),'javascript:srt(\'rating\',\'asc\');',false,'margin-left:5px').outImage(outIcon('sdown'),'javascript:srt(\'rating\',\'desc\')',false).'</span>';
	}
	return $r;
}

/*
 ***************************************************************************************************************************
 * Audio files explorer specific HTML controls output scripts
 ***************************************************************************************************************************
 */

function outWaitCursorMask(){
	$output='<div id="waitCursorMask" style="position:absolute; background: white; top:0;left:0;width:99%;height:100%;filter:Alpha(opacity=1, style=0); -moz-opacity:0.01; -khtml-opacity:0.01; opacity:0.01; cursor:wait; display:none">&nbsp;</div>';
	$output.='<script type="text/javascript" language="javascript">function setWaitCursor(){dgi("waitCursorMask").style.display="block";}'."\n";
	return $output.='function removeWaitCursor(){document.getElementById("waitCursorMask").style.display="none";}</script>'."\n";
}


/**
 * @desc Display a mask hidding page during loading, and show page when page and all iframes are loaded
 *
 * @desc boolean $waitForIframes: true to wait for iframes to load, false just to wait for main page
 * @return string HTML code
 */
function outLoadMask($waitForIframes=true, $options=array()){
	$o='<div id="loadMask">';
	$o.=outImage(outIcon('loading'),false,false,'position:absolute;top:50%;left:50%;margin:-8px 0 0 -8px');
	if(isset($options['text'])) $o.='<div style="position:absolute;top:50%;left:0;width:100%;margin-top:1em;text-align:center"><div class="popup">'.$options['text'].'</div></div>';
	$o.='</div>';

	$o.='<script type="text/javascript">';
	if($waitForIframes)
		$o.='var loadMaskIframesNb=0;connect(W,"load",function(){var i, ifs=D.getElementsByTagName("IFRAME"); for(var i in ifs) if(i!="length") {connect(ifs[i],"load",function(){if(--loadMaskIframesNb==0) $("loadMask").e.remove()}); loadMaskIframesNb++}; if(!loadMaskIframesNb) $("loadMask").e.remove() })';
	else
		$o.='connect(W,"load",function(){$("loadMask").e.remove()})';
	$o.='</script>';
	return $o;
}

/*
 ***************************************************************************************************************************
 * Other...
 ***************************************************************************************************************************
 */

/**
 * @desc return HTML code for theme selection for given level, through included POST form
 *
 * @return string : HTML code
 * @param string $standaloneID: id of control, used when displaying multiple standalone theme controls
 * @param string $standaloneDefaultTheme: name of default selected theme, used when themeLevel is "standalone"
 * @param string $themeLevel : 'general', 'user', 'resource', 'standalone'
 * @param boolead $targetParentFrame : true if form is submited in parent frame (/index.php)
 * @param mixed $displayLabel: true if "Theme" label is displayed, false if not, 'noSmallFont' if smallFont class should not be applied
 * @param boolead $displayIcon: true if form is submited in parent frame (/index.php)
*/
function outThemeSelectionControl($themeLevel, $targetParentFrame=false,$method='POST',$extraHTML='',$displayLabel=true, $displayIcon=true, $standaloneDefaultTheme='', $standaloneID=false){
	global $useWin;
	if($themeLevel=='general')			$theme=cfGGetVar('theme');
	elseif($themeLevel=='standalone')	$theme=$standaloneDefaultTheme;
	else $theme=cfActualTheme();
	$dir=cfAppDocRoot().'/themes';

	// Form
	$output= '<form action="/index.php" method="'.$method.'" enctype="multipart/form-data" name="themeForm"'.($targetParentFrame?' target="_parent"':'').' style="display:inline"'.' '.$extraHTML.">\n";

	if($themeLevel=='general')	 $output.='<input type="text" name="themeLevel" value="general" style="display:none">';
	elseif ($themeLevel=='user') $output.='<input type="text" name="themeLevel" value="user" style="display:none">';
	elseif ($themeLevel=='resource') $output.='<input type="text" name="themeLevel" value="resource" style="display:none">';

	// Display control
	// icon
	if($displayIcon) $output .= outImage(outIcon('theme'),false, false,'margin-right:0.5em;vertical-align:middle');
	// Label
	if($displayLabel) $output .= '<span'.(($displayLabel=='noSmallFont')?'':' class="smallFont"').' style="margin-right:1em;vertical-align:middle;">'.cfCaption('genTheme').'</span>';
	// Select
	$output .= '<select class="textinput smallFont" name="themeUpdate"'.(($standaloneID)?' id="'.$standaloneID.'"':'').' size="1" onChange="if(W.themeSelectionControlChanged) themeSelectionControlChanged(this.value, this); else document.themeForm.submit()" style="vertical-align:middle">'."\n";
	// Display "no theme" item (only if $theme parameter is not set)
	if($themeLevel!='general'){
		if($theme) $output .= '<option value="noTheme">'.cfCaption('adminNoTheme').'</option>';
		else $output .= '<option selected value="noTheme">'.cfCaption('adminNoTheme').'</option>';
	}
	// Create themes select
	$themes=cfMGetVar('weezoThemes');
	foreach ($themes as $name=>$data) if($name!='skins/default' && $name!='skins/osx' && !isset($data['private'])){
		if($name==$theme) $output .= '<option selected value="'.cfUTF8Encode($name).'">'.cfUTF8Encode($name).'</option>';
		else $output .= '<option value="'.cfUTF8Encode($name).'">'.ucfirst(cfUTF8Encode($name)).'</option>';
	}
	$output .= '</select>';
	$output.="</form>\n";
	return $output;
 }

/**
 * @desc return HTML code for HTTPXmlRequest js inclusion (now included in common.js)
 *
 * @return string : HTML code
*/
function outInsertHTTPXmlRequest(){}

/**
 * @desc display robot-safe clickable email adress
 *
 * @param string $eMail : email adress
 */
function outMailSafe($eMail,$echoOutput=1){
	$o='<a href="javascript:void(0)" onclick="javascript:location.href=\'mail\'+\'to:'.substr($eMail,0,3).'\'+\''.substr($eMail,3).'\'"><span>'.substr($eMail,0,4).'</span><span>'.substr($eMail,4)."</span></a>\n";
	if($echoOutput) echo $o;else return $o;
}

/**
 * @desc draw a progress bar
 *
 * @param float $percent : percentage 0-1 (set to -1 to set error)
 * @param string $width : bar width
 * @param string $caption : bar caption
 * @param string $id : item Id
 * @param string $extraHTML : extra HTML
 * @return string : HTML code
 */
function outProgressBar($percent,$width='100%',$caption=false,$id=false,$extraHTML=false,$invert=false, $extraStyle=false){
	/* code:
	<span class=""
	*/
	$output='';
	if(!is_numeric($percent)) $percent=-1;
	if($percent>1) $percent=1;
	// Outer bar
	if($percent==-1 || $invert) $output.='<span';
	else $output.='<div';
	$output.=' style="width:'.$width.';'.(($extraStyle)?$extraStyle:'').'" class="progressOuter"'.(($id)?' id="'.$id.'"':'').(($extraHTML)?' '.$extraHTML:'').'>';
	// Inner bar
	if($percent==1) $output.='<span style="display:block;position:relative;width:100%" class="progressInner"'.(($id)?' id="'.$id.'_inner"':'').'></span>';
	else $output.='<div style="position:absolute;'.(($invert)?'right':'left').':0px; width:'.(($percent==-1)?'0':(round(100*$percent))).'%" class="progressInner"'.(($id)?' id="'.$id.'_inner"':'').'></div>';
	// Text
	if($caption) $output.='<div class="progressTextO"><div class="progressText"'.(($id)?' id="'.$id.'_text"':'').'>'.$caption.'</div></div>';

	if($percent==-1 || $invert) $output.='</span>'; else $output.='</div>';
	return $output;
}

/**
 * @desc Wiimote keys management
 *
 * @param string $scrollItem : JS item to be scrolled with arrows (leave blank for body)
 */
function outKeysManager($scrollItem='document.body'){
	if(!cfIsWII()) return;
?>
<script language="javascript" type="text/javascript">
var kmInterval=false;
var documentOnKeydown = document.onkeydown;
var documentOnKeyup = document.onkeyup;
var zoomIn=false;
var keyActDown=0;
var scrollInt=50;
function keydown(e){
	if(documentOnKeydown) {if(!documentOnKeydown(e)) return;}
	if(e) kc=e.which; else {if(window.event) kc=window.event.keyCode; else return;}

	if(false && (kc==170 || kc==27) && !zoomIn) {
		if(keyActDown==176){
			if(window.winMe) winMe.reload();
		}
		if(keyActDown==177){
			if(window.winMe) winMe.toggleState();
		}
		else{
			if(dgi("helpDiv") && dgi("helpDiv").innerHTML!="") {
				dgi("helpDiv").innerHTML="";
			}
			else{
				if(window.winMe) winMe.closeMe();
				else{
					if(!window.winScript) window.close();
				}
			}
		}
		return;
	}
	if(kc==175) W.setTimeout("scroll('up',7)",scrollInt);
	if(kc==176) W.setTimeout("scroll('down',7)",scrollInt);
	if(kc==178) W.setTimeout("scroll('left',7)",scrollInt);
	if(kc==177) W.setTimeout("scroll('right',7)",scrollInt);

	if(false && kc==171 || kc==79) {
		if(!zoomIn && window.optionKeyPressed) optionKeyPressed();
		zoomIn=false;
	}
	if(false && kc==174) zoomIn=true;
	keyActDown=kc;
}

function scroll(dir,step){
	if(dir=='down') <?php echo $scrollItem;?>.scrollTop+=Math.pow(2,Math.min(step,5));
	if(dir=='up') <?php echo $scrollItem;?>.scrollTop-=Math.pow(2,Math.min(step,5));
	if(dir=='left') <?php echo $scrollItem;?>.scrollLeft-=Math.pow(2,Math.min(step,5));
	if(dir=='right') <?php echo $scrollItem;?>.scrollLeft+=Math.pow(2,Math.min(step,5));
	if(step>0) window.setTimeout("scroll('"+dir+"',"+(step-1)+")",scrollInt);
}
function keyup(e){
	keyActDown=0;
	//dgi("keysU").innerHTML+=kc+'<br>';
	//if(kmInterval!==false) window.clearInterval(kmInterval);
	//kmInterval=false;
	if(e) kc=e.which; else {if(window.event) kc=window.event.keyCode; else return;}

	if(kc==175) {
		if(window.keyUp) keyUp();
	}
	if(kc==176 || kc==76) {
		if(window.keyDown)  keyDown();
	}
	if(kc==178) {
		if(window.keyLeft) keyLeft();
	}
	if(kc==177) {
		if(window.keyRight) keyRight();
	}

	if(documentOnKeyup) documentOnKeyup(e);
}
document.onkeydown=keydown;
document.onkeyup=keyup;
</script>
<?php
}

/**
 * @desc return CSS/style code for opacity
 *
 * @param integer $percent : opacity (0-100)
 */
function outCssOpacity($percent){
	$percent=round($percent);
	return ((cfGetBrowser()=='ie')?'_filter:Alpha(Opacity='.$percent.');zoom:1;':'').'opacity:'.($percent/100).';';
}

/**
 * @desc Return single ETC line
 *
 * @param string $id : line id
 * @param string $tagExtraHTML : <tr> extra HTML inside tag
 * @param mixed $innerHTML : content
 * @param integer $index : line index
 * @return string : HTML code
 */
function outETC($id, $tagExtraHTML, $innerHTML, $index, $useDIV=false){
	if($useDIV) return '<div id="'.$id.'" class="eTC" eH=1 '.$tagExtraHTML.' '.((cfGetBrowser()!='ie' && $index%2==0)?'lang="i"':'').'>'.$innerHTML.'</div>';

	if(cfGetBrowser()=='ie' || $index%2==0) $del='td'; else $del='th';
	$output ='<tr id="'.$id.'" class="eTC" eH=1 '.$tagExtraHTML.' '.((cfGetBrowser()!='ie' && $index%2==0)?'lang="i"':'').'>';
	$output.='<'.$del.' class="eTCLeft">'.((is_array($innerHTML) && count($innerHTML)>1)?$innerHTML[0]:'&nbsp;').'</'.$del.'>';
	$output.='<'.$del.'>'.((is_array($innerHTML))?$innerHTML[min(1,count($innerHTML)-1)]:'&nbsp;').'</'.$del.'>';
	$output.='<'.$del.' class="eTCRight">'.((is_array($innerHTML) && count($innerHTML)==3)?$innerHTML[2]:'&nbsp;').'</'.$del."></tr>\n";
	return $output;
}

/**
 * @desc  Insert HTML code for a button
 * @param string $caption : button's caption (utf-8 encoded)
 * @param string $img : optional : path to button's image
 * @param string $href : optional : link href
 * @param string $id : optional : button's ID
 * @param string $title : optional : button's title
 * @param string $extraHTML : optional : extra HTML code included in button definition
 * @param mixed $width : optional : button's width : false = auto, integer = fixed width, array(minWidth, maxWidth) : min/max width
 * @param string $state : optional : buton state : "on" (button pressed = activated), "up" (normal), "off" (disabled)
 * 				set to 'style="width:XXpx;text-align:XXX"' to define a fixed-width button
 * @param boolean $forceFullDisplay : optional : display button's caption even for compact themes
 * @return void
 *
*/
function outBt($caption, $img=false, $href=false, $id=false, $title=false, $extraHTML=false, $state="up", $width=false, $textAlign=false, $forceFullDisplay=false){

	if(cfBGetVar('output')=='xhtml-mp') return '<a class="bt"'.(($extraHTML)?$extraHTML:'').' href="'.$href.'">'.(($img)?'<img src="'.$img.'" alt=""/>':'').$caption.'</a>';

	//Format : <fieldset class="bt"><legend></legend><a class="up/on/off" href="" title=""><div><b><IMG/><s>TEXT</s></b><u>&nbsp;</u><i>&nbsp;</i></div></a><q></q></fieldset>

	// Corrects Width for IE (remove aproximate icon Width
	if($width && cfGetBrowser()=='ie'){
		$units=substr($width,-2);
		if($units=='px' || $units=='em' || $units=='pt'){
			$width=substr($width,0,strlen($width)-2);
			if($units=='em') $width=(max($width-2,0)).$units; else $width=(max($width-16,0)).$units;
		}
	}
	elseif ($width) $width+=22;

	if($width && !cfCmpRight($width,'em') && !cfCmpRight($width,'%') && !cfCmpRight($width,'px')) $width.='px';

	// Theme specific button
	if(cfTGetVar('buttonFunction') && !isset($_ENV['weezo']['buttonFunction']))
		$_ENV['weezo']['buttonFunction'] = create_function('$caption,$href,$img,$title,$id,$extraHTML,$forceFullDisplay', cfTGetVar('buttonFunction'));
	if(isset($_ENV['weezo']['buttonFunction'])) return $_ENV['weezo']['buttonFunction']($caption,$href,$img,$title,$id,$extraHTML,$forceFullDisplay);

	if(cfTGetVar('compactDisplay') && !$forceFullDisplay && $img!==false){
		//if(!$title) $title=$caption;
		//$caption='';
	}

	// Button "frame"
	$output ="\n<fieldset".(($id)?' id="'.$id.'"':'').(($extraHTML)?' '.$extraHTML:'').' class="bt">';
	$output.='<legend>'.(($state=='off')?'':$href).'</legend>'; // Store href for disabled button
	if(!$href) $href='javascript:return';

	// Action
	$output.='<a class="'.$state.'" href="'.$href.'"'.(($title)?' title="'.$title.'"':'').'>';
	$output.='<div'.(($width || $textAlign)?' style="'.(($width)?'width:'.$width.';':'').(($textAlign)?'text-align:'.$textAlign:'').'"':'').'><b>';

	/**
	 * Icon
	 */

	// IE6 & png: use MS filters
	if(cfFileExtension($img)=='png' && !cfBGetVar('png')) {
		$output.='<span class="ieBtPng" style="filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='.$img.');';
		if(strlen($caption)>0) $output.='margin-right:0.4em;';
		$output.='"></span>';
	}
	// Regular image
	else {
		$output.='<img alt="" src="';
		if($caption) $mr='margin-right:0.4em;'; else $mr='';
		if($img) $output.=$img.'" '.(($mr)?'style="'.$mr.'"':''); else $output.='/gfx/v.gif"';
		$output.='>';
	}

	// Caption
	if($caption) $output.='<s'.(($width && cfGetBrowser()=='ie')?' style="'.'width:'.$width.'"':'').'>'.$caption.'</s>';
	elseif (cfGetBrowser()=='ie') $output.='<s>&nbsp;</s>';
	else $output.='<s style="margin-right:-0.2em">&nbsp;</s>';
	return $output.="</b><u>&nbsp;</u><i>&nbsp;</i></div></a><q></q></fieldset>\n";

	if(cfIsInApp() && cfGetBrowser()=='ie' && stripos(($_SERVER['HTTP_USER_AGENT']), "msie 7.0")!==false && $caption=='')
		return $output.'<span> </span></b><u>&nbsp;</u></a>';
	return $output.'<span style="position:relative;min-width:80px">a'.str_replace(' ','&nbsp;',$caption).'</span></b><u>&nbsp;</u></a>';
}

/**
 * @desc Insert Resource configuration button
 *
 */
function outResourceConfigBt($showCaption=true){
	if(cfIsWII()) return false;
	if(!cfRGetVar('definition','configFile') && !file_exists(dirname($_SERVER['SCRIPT_FILENAME'].'/config.php'))) return;

	// Configuration script URL
	// Note: limitedConfig is indicative, actual configuration access rights are not based on it
	$configScript=dirname($_SERVER['PHP_SELF']).'/'.((cfRGetVar('definition','configFile'))?cfRGetVar('definition','configFile'):'config.php?limitedConfig=1');

	$output=outButton(($showCaption)?cfCaption('genParameters'):'','javascript:configureResource()',outIcon('gears'),(!$showCaption)?cfCaption('genParameters'):'');
	$output.="\n<script type=\"text/javascript\">function configureResource(){\n";
	$output.="var winConfId=\"configWindow".$_SESSION['activeResourceId']."\";\n";
	$output.="if(W.beforeConfigureResource) beforeConfigureResource();"; // If a function called "beforeConfigureResource" is present, execute
	$output.='var confWin=wl.popup("'.$configScript.'","'.cfCaptionJS('genParameters').'",{"id":winConfId});';
	$output.="\nwith(parent.win(winConfId)){\n";
	//$output.='resizeInnerTo('.$width.','.$height.');'."\n";
	$output.='try{with(confWin){activate();resizable(false);reloadable(false)}}catch(e){}';
	$output.="}}</script>\n";
	return $output;
}

/**
 * @desc Load a file from an URL, and display loading progress bar
 *
 * @param string $url
 * @param string $localCompleteFilename
 * @param boolean $showCancelButton
 * @return boolean true on success
 */
function outLoadFileProgress($url, $localCompleteFilename, $showCancelButton=true){
	function outLFPupdatePage($id,$text){
		if($id) echo '<script type="text/javascript">dgi("'.$id.'").innerHTML="'.str_replace('"','\\"',$text).'"</script>';
		else echo '<script type="text/javascript">'.$text.'</script>';
	flush();
}

	// Display progress bar
?>
<script type="text/javascript">function uPB(pc){progressSetPerc("pBar",pc);progressSetText("pBar",pc+"%")}
</script>
<div id="progress" style="text-align:center">
<?php echo outProgressBar(0,'100%','0%','pBar'); ?>
</div>
<br>
<?php
	if($showCancelButton)	echo '<center id="cancelButtonOuter">'.outButton(cfCaption('genCancel'),'javascript:wl.UICommand(\'close\')',outIcon('cancel'),false,'cancelButton','style="margin-bottom:5px"').'</center>';
?>
<form name="proceedForm" method="POST" action="" style="display:none"><input name="doUpdate" value="download"></form>
<?php
	flush();
	if(file_exists($localCompleteFilename)) @unlink($localCompleteFilename);

	// Max 2 redirections
	$proceed=2;
	while($proceed){
		if(!($handle=cfSocketHTTPRequest($url,7,false,$foo1,$foo2,false,true))) return false;
		$response='';
		while(strlen($response)<1023 && !feof($handle)) $response .= fread($handle, 1024);

		$pos=strpos($response,"\r\n\r\n");
		if($pos === false){
			fclose($handle);
			displayErrorPage();
		}
		$header=substr($response,0,$pos);
		$response=substr($response,$pos+4);

		$headers = array();
		$lines = explode("\r\n", $header);
		$fSize=0;
		if(count($lines)==0) displayErrorPage();

		// HTTP response code
		$responseCode=substr($lines[0],9,3);

		// Seek file size or redirection
		foreach($lines as $value){
			if(strtolower(substr($value,0,15))=='content-length:') $fSize=trim(substr($value,15));
			if(strtolower(substr($value,0,9))=='location:') $redir=trim(substr($value,9));
		}
		// If redirection
		if(isset($redir)) {
			$proceed--;
			if($proceed<0) return false;
			if(substr($redir,0,1)=='/') {
				$url=substr($url,0,strpos($url,'/',10)).$redir;
			}
			else $url=$redir;
			unset($redir);
		}
		// ! 200 OK
		elseif($responseCode!='200') displayErrorPage();
		// No redirection and response code 200 OK
		else $proceed=0;
	}

	// Proceed Download
	$handlew = fopen($localCompleteFilename, "w");
	if(!$handlew) {fclose($handle);return false;}
	$i=-1;
	fwrite($handlew,$response);
	$dlSize=strlen($response);
	while (!feof($handle)) {
		$response=fread($handle, 8192);
		fwrite($handlew,$response);
		$i++;$dlSize+=strlen($response);
		if($i%9==0) outLFPupdatePage(false,'uPB('.floor(100*$dlSize/$fSize).');');
	}
	outLFPupdatePage(false,'uPB(100);');
	fclose($handlew);
	fclose($handle);
	return true;
}

/**
 * @desc Return HTML for a list of big icons
 *
 * @param array $list: (link,image,caption)
 * @return string
 */
function outIconsList($list){
	$output='';
	foreach ($list as $icon){
		$output.='<fieldset class="iconDiv" style="border:0;text-align:center;position:relative;display:inline"><a class="icon" style="width:70;height:70;color:inherit !important" onclick="'.$icon['link'].'"><img src="'.$icon['image'].'"><br>';
		$output.='<div style="position:relative">';
		$output.='<span class="iconCS" style="left:-1;top:0">'.$icon['caption'].'</span>';
		$output.='<span class="iconCS" style="left:1;top:0">'.$icon['caption'].'</span>';
		$output.='<span class="iconCS" style="top:-1;left:0">'.$icon['caption'].'</span>';
		$output.='<span class="iconCS" style="top:1;left:0">'.$icon['caption'].'</span>';
		$output.='<span class="iconCS" style="top:1;left:1">'.$icon['caption'].'</span>';
		$output.='<span class="iconC" style="top:0;left:0">'.$icon['caption'].'</span>';
		$output.="</div></a></fieldset>\n";
	}
	$output.='<script type="text/javascript">function outIconListAlign(){var n,l=D.getElementsByTagName("fieldset");for(var i=0;i<l.length;i++) if(l[i].className=="iconDiv") {n=l[i].firstChild.lastChild;';
	if(cfGetBrowser()=='ie') $output.='n.style.left=-0.5*n.firstChild.offsetWidth';
	else $output.='n.style.left=(n.offsetWidth-n.firstChild.offsetWidth)/2';
	$output.='}} outIconListAlign();</script>';
	return $output;
}

/**
 * @desc Return script required for RSA ciphering
 * @param string mode: 'auto' to determine actual ciphering from protocole
 * @param bool $doesCipher: return true if text will actually be RSA ciphered
 *
 * @return string
 */
function outRSAScripts($mode='auto',&$doesCipher=null){
	function lRSAKP(){
		if(!cfGGetVar('loginKeyLength')) return array('keyLength' => 0);
		// Read file
		$keyFile=cfAppDataDir().'/RSAKey.cnf';
		if(!file_exists($keyFile)){
	    	// Try to regenerate now the keys
	    	require(cfAppDocRoot().'/local/genRSAKey.php');

	    	// If keys could not be generated, abort
	    	if(!file_exists($keyFile)){
				cfLog('login.php: Missing RSAKey.cnf file. Relaunch application to generate file.',LOG_ERROR);
				return false;
	    	}
		}
		$kp=parse_ini_file($keyFile);
		if(!isset($kp['keyLength']) || !isset($kp['modulo']) || !isset($kp['publicExponent']) || !isset($kp['privateExponent'])){
			cfLog('login.php: corrupted RSAKey.cnf file. Delete file and relaunch application.',LOG_ERROR);
			return false;
		}
		// Set maxDigits
		if($kp['keyLength']<=128) $kp['maxDigits']=19;
		elseif($kp['keyLength']<=384) $kp['maxDigits']=57;
		elseif($kp['keyLength']<=512) $kp['maxDigits']=76;
		elseif($kp['keyLength']<=1024) $kp['maxDigits']=154;
		else $kp['maxDigits']=308;
		return $kp;
	}

	$doesCipher=true;
	if($mode=='auto'){
		if(cfGGetVar('protocol')=='https') $doesCipher=false;
		elseif(!cfGGetVar('encryptPassword')||
			(cfGGetVar('disablePasswordCipheringForSlowJSDevices') && cfBGetVar('slowJS'))) $doesCipher=false;
		elseif (!($RSAKeyPair = lRSAKP())) $doesCipher=false;
	}
	elseif (!($RSAKeyPair = lRSAKP())) $cipher=false;

	$output='<script type="text/javascript">';
	if(!$doesCipher) $output.='function RSACrypt(plainText,percNode,callback) {callback(plainText)}</script>';
	else{
		$output.='var keyLength='.$RSAKeyPair['keyLength'].',';
		$output.='keyModulo="'.$RSAKeyPair['modulo'].'",';
		$output.='keyPExp="'.$RSAKeyPair['publicExponent'].'",';
		$output.='keyMaxDigits='.$RSAKeyPair['maxDigits'].';';
		$output.='</script>';
		$output.=cfScriptLink('BigInt-RSA.js');
	}
	return $output;
}

/**
 * @desc Display a progress bar and generate video thumbnails
 *
 * @param string $caption
 * @param string $icon
 * @param integer $total
 * @param integer $standalone: 0 to in to integrate HTML head and body, 1 to integrate only progress box, 2 to integrate only progress bar
 * @param array $options:
 * 			'display'=>'%' to display progress percentage or 'size' to display dl/totalsize
 * 			'suspendSession'=>true to suspend current session during download
 */
function outSyncProgress($caption=false,$icon=false,$total=100,$standalone=1,$options=array()){
	require_once(INCLUDE_DIR.'explorerFunctions.php');
	$options+=array('display'=>'%');
	if($caption===false) $caption=cfCaption('transferCfgProgress');
	if($icon===false) $icon=outIcon('blankFile');

	$_ENV['outSyncProgress']['standalone']=$standalone;
	$_ENV['outSyncProgress']['total']=$total;
	$_ENV['outSyncProgress']['display']=$options['display'];

	/**
	 * Display synchronous monitoring stuff
	 */

	// Page header
	if($standalone===0){
		if(cfIsMobile()) mfInsertHead(); else cfInsertHEAD();
		echo '<body>';
	}

	// Progress indocator (mobile
	if(cfIsMobile()){
		require_once(INCLUDE_DIR.'mobileFunctions.php');
		echo mfListTable('id="syncProgMon"');
		echo mfListItem('',(($icon)?'<img src="'.$icon.'" style="width:32px;height:32px">':''),false,'<div id="progTxt">'.$caption.'</div>',false,'<div id="progPerc">0%</div>',false);
		echo '</table>';
?>
<script type="text/javascript">
function oSPU(t,p){if(p) dgi('progPerc').innerHTML=p+'%'}
</script>
<?php
	}
	// Mask & progress indicator (computer)
	else{
		if($standalone<2){
?>
<div id="syncProgMon">
<div class="mask">&nbsp;</div>
<div style="position:absolute;top:50%;left:50%"><div class="popup" style="width:24em;left:-12em;top:-4em">
<?php echo outFrameHeaderTable('popupHeader',outImage(outIcon('loading'),false,false,'vertical-align:middle;margin-right:0.5em').$caption,outImage($icon,false,false,'vertical-align:middle'));?>
<br>
<center><?php
		}
		// Progress bar
		echo outProgressBar(0,'80%','0/'.$total,'syncProg');

		if($standalone<2){
?></center>
<br>
</div></div></div>
<?php
		}
?>
<script type="text/javascript">
function oSPU(txt,perc){progressSetText('syncProg',txt);progressSetPerc('syncProg',perc);}
</script>
<?php
	}

	set_time_limit(0);
	if(isset($options['suspendSession'])) {
		cfSessionPause();
		$_ENV['outSyncProgress']['suspendSession']=1;
	}
}

/**
 * @desc Update progress bar
 *
 * @param number $nb: progress, or "done" if done
 * @param number $total
 */
function outSyncProgressUpdate($nb,$total=false){
	// Done
	if($nb==='done'){
		if($_ENV['outSyncProgress']['standalone']==0) die('<script type="text/javascript">D.location.href=D.location</script>');
		if(isset($_ENV['outSyncProgress']['suspendSession'])) cfSessionRestart();
		set_time_limit(ini_get('max_execution_time'));

		// Remove progress indicator
		if(cfIsMobile())
			echo '<script type="text/javascript">dgi("syncProgMon").style.display="none"</script>';
		else
			echo '<script type="text/javascript">if(dgi("syncProgMon")) removeNode(dgi("syncProgMon")); else removeNode(dgi("syncProg"))</script>';
		unset($_ENV['outSyncProgress']);
		return;
	}

	// Update total if needed
	if($total!==false) $_ENV['outSyncProgress']['total']=$total;
	if($_ENV['outSyncProgress']['display']=='%') $txt=floor(100*$nb/$_ENV['outSyncProgress']['total']).' %'; else $txt=efFileSizeFormat($nb).' / '.efFileSizeFormat($_ENV['outSyncProgress']['total']);
	echo '<script type="text/javascript">oSPU("'.$txt.'",'.@floor(100*$nb/$_ENV['outSyncProgress']['total']).');</script>';
	flush();

	// Ping app
	cfPingUpdate();
}

/**
 * @desc Insert Accordion
 *
 * @param array $tabs: array of tabs ('caption'=>, 'src'=>)
 * @param integer $width
 * @param integer $height
 * @param array options: 'hideFolded'=>0/1 : if 1, set display:none on folded iframes, 'callback'=>: callback on fold/unfold completition,
 * 						 'delayedIFrameSrc'=> true to use JS to set iframes src's so IE doesn't emit a clic sound
 */
function outAccordion($tabs,$width,$height,$options=array()){
	// Embedded IE webbrowser workaround (cookies are not always allowed): insert session id into URL
	if(cfIsInApp()) foreach ($tabs as $id=>$tab) $tabs[$id]['src'].=((strpos($tab['src'],'?')!==false)?'&':'?').'WSESSID='.wSession_id();

?>
<div id="accordionOuter" style="width:<?php echo $width;?>;height:<?php echo $height;?>;position:absolute;overflow:hidden">
<?php
	$selected=false;$nb=0;
	// Find selected tab
	foreach ($tabs as $id=>$tab){
		if($selected===false || isset($tab['selected'])) $selected=$nb;
		$nb++;
	}
	// Insert items
	$nb=0;
	foreach ($tabs as $id=>$tab){
?><div id="<?php echo $id;?>" class="accordion" style="width:<?php echo $width;?>;height:<?php echo $height;?>;visibility:hidden" onclick="accordionToggle(this.id)"><?php
	?><div class="accordionHandle <?php echo $nb.(($nb==$selected)?' sel':'');?>"><?php
		?><div class="accordionCaption"><?php echo $tab['caption'];?></div><?php
		?><img class="accordionHandleIcon" src="<?php
	if($nb<=$selected) echo outIcon('right');
	else echo outIcon('left');
		?>" alt=""><?php
	if(isset($tab['icon'])) echo outImage($tab['icon'],false,'class="accordionIcon"');
	?></div><?php
	?><iframe name="<?php echo $id;?>" id="iFrame<?php echo $id;?>" style="height:100%" src="<?php echo ((@$options['delayedIFrameSrc'])?'':$tab['src']);?>" frameborder="0" scrolling="<?php if(!isset($tab['scroll']) || $tab['scroll']==false) echo 'no'; else echo 'auto'; ?>"></iframe></div>
<?php
		$nb++;
	}
?>
</div>
<script type="text/javascript">
var accordionTabs=[<?php
	$o='';
	foreach ($tabs as $id=>$tab) $o.=($o?',':'').'"'.$id.'"';
	echo $o;
?>];
<?php if(isset($options['callback'])) echo 'var accordionCallback="'.$options['callback'].'"';?>;
var accordionSelected=<?php echo (int)$selected;?>;
var accordionHideFolded=<?php echo (int)@$options['hideFolded'];?>;
var accordionHandleWidth=dgi(accordionTabs[0]).firstChild.offsetWidth-1, accordionMove, accordionOuterWidth;
function accordionInit(){
	var s=0,off=0,id,nb=accordionTabs.length;
	accordionOuterWidth=dgi('accordionOuter').offsetWidth-1;
	for(var i=0;i<nb;i++){
		id=accordionTabs[i];
		dgi(id).firstChild.nextSibling.style.width=(accordionOuterWidth-(accordionHandleWidth)*nb)+'px';
		if(accordionHideFolded && accordionSelected!=i) dgi(id).getElementsByTagName('IFRAME')[0].style.visibility='hidden';
		if(s){
			off=accordionOuterWidth-(1+accordionHandleWidth)*(nb-i);
			with(dgi(id).style) {left=off;visibility='visible'}
		}
		else {
			with(dgi(id).style) {left=off;visibility=''}
			if(accordionSelected==i) {
				//if(i==0) dgi(id).firstChild.style.cursor='default'
				s=1;
			}
			else off+=1+accordionHandleWidth;
		}
	}
<?php
	if(@$options['delayedIFrameSrc']){
		foreach ($tabs as $id=>$tab) echo 'connect(W,"onload",function(){$("iFrame'.$id.'").e.updateIFrameSrc("'.$tab['src'].'")});';
	}
?>
}
function accordionInitWA(){accordionInit();return true}
function accordionPosFromId(id){for(var i=0;i<accordionTabs.length;i++) if(accordionTabs[i]==id) return i;}
function accordionRefresh(){
	dgi(accordionTabs[accordionSelected]).getElementsByTagName('IFRAME')[0].style.visibility='visible';
}
// start
function accordionToggle(id){
	var i,ns=accordionPosFromId(id),nb=accordionTabs.length,h;

	// Apply "sel" class to selected accordion's handle, and remove it from other ones
	for(i=0;i<nb;i++) {
		h=$(dgi(accordionTabs[i]).firstChild);
		if(id===accordionTabs[i]) h.e.className().add('sel'); else h.e.className().remove('sel');
	}

	if(accordionHideFolded) setTimeout('accordionRefresh()',10)
	if(ns==accordionSelected && ns>0) ns--;
	for(i=0;i<nb;i++) with(dgi(accordionTabs[i]).firstChild){
		// Apply "sel" class to selected accordion's handle, and remove it from other ones
		h=$(dgi(accordionTabs[i]).firstChild);
		if(i==ns) h.e.className().add('sel'); else h.e.className().remove('sel');
		// Change arrow direction
		firstChild.nextSibling.src=(i<=ns)?'<?php echo outIcon('right');?>':'<?php echo outIcon('left');?>';
	}
	accordionMove={};

	if(ns>accordionSelected){
		for(i=accordionSelected+1;i<=ns;i++) accordionMove[accordionTabs[i]]=i*(1+accordionHandleWidth);
	}
	else if(ns<accordionSelected){
		for(i=ns+1;i<=accordionSelected;i++) accordionMove[accordionTabs[i]]=accordionOuterWidth-(accordionTabs.length-i)*(1+accordionHandleWidth);
	}
	else return;
	if(W.accordionCallback) eval(accordionCallback+"(accordionTabs[ns],accordionTabs[accordionSelected])");
	accordionSelected=ns;
	accordionAnim(20);
}
// anim
function accordionAnim(step){
	// done
	if(!step){
		for(var tid in accordionMove) dgi(tid).style.left=accordionMove[tid];
		if(accordionHideFolded) for(var i=0;i<accordionTabs.length;i++) if(i!=accordionSelected) dgi(accordionTabs[i]).getElementsByTagName('IFRAME')[0].style.visibility='hidden';
		return;
	}
	for(var tid in accordionMove) {
		dgi(tid).style.left=Math.floor(accordionMove[tid]+(actualOffsetLeft(dgi(tid))-accordionMove[tid])*3/4);
	}
	setTimeout("accordionAnim("+(step-1)+")",10);
}
accordionInit();
connect(W,'resize',accordionInitWA);

</script>
<?php
}

/**
 * @desc Update panel position from POST command
 */
function outCommentsPanelUpdate(){
	if(isset($_POST['setCommentsPanelState'])) {
		$s=$_POST['setCommentsPanelState'];
		if($s!=0 && $s!=1 && $s!=2) $s=1;
		if(cfIsResource())
			cfRSetVar('commentsPanelSate',$s);
		else
			cfUSetVar('commentsPanelSate',$s);

        cfAsyncHeader();
        cfAsyncFooter();
	}
}

/**
 * @desc Display an help tag with help text on hover
 *
 * @param string $helpText
 * @param string $title
 * @param string $icon: help icon src
 * @return string
 */
function outHelpTag($helpText,$title=false,$icon=false){
	if($icon) $icon=outIcon('help');
	return outImage(outIcon('helpTag'),false,outToolTip($helpText,$title,$icon,true),'margin:0 0 2px 8px;vertical-align:bottom;cursor:help');
}

/**
 * @desc Insert a debug button
 *
 * @param string $script: debug script
 */
function outDebugBt($script){
	echo outButton(false,'javascript:'.$script,outIcon('bugs'),false,'debugBt','style="position:absolute;right:0;top:0"');
}

/**
 * @desc Insert a set of links to share an URL through social networks
 *
 * @param string $url
 * @param array $opt: list of networks to exclude (clipboard, mail, facebook, twitter, addthis),
 * 					  'noSpace' to exclude spaces between buttons
 * 					  'fullText' to replace when possible URL by a more precise text
 * 					  'password'=>(clear text) password to include &password= in QR code
 * @param array $text: caption sent with shared link (not UTF8 encoded), subject for email
 * @return string
 */
function outSocialURLShare($url,$opt=array(),$text=''){
	$out='';
	$opt=cfArrayValuesToKeys($opt);
	$sep=(@$opt['noSpaces'])?'':'&nbsp;';

	if(!function_exists("outSocialURLShareBt")){
		function outSocialURLShareBt($type,$href,$ico,$blankTarget,$extraStyle,$extraHTML,$tooltipCaption,$btOptions=array()){

			if($btOptions['bwIco']) $ico=cfFilteredImage($ico,0,0,array('r'=>0,'g'=>0,'b'=>180,'p'=>0),255,80);

			if($type=='mobile') {
				$btOptions['extraClass']='img64';
				return mfButton('',$href,$ico,$extraHTML.' style="'.$extraStyle.'"'.(($blankTarget)?' target="_blank"':''),false,$btOptions);
			}
			/*
			if($type=='raw16') {
				return '<a href="'.$href.'" '.(($blankTarget)?'target="_blank"':'').'><img style="width:14px;height:14px;margin:2px;vertical-align:bottom;'.$extraStyle.'" src="'.$ico.'" '.outToolTip($tooltipCaption).'></a>';
			}
			*/
			if($type=='buttonSmall') {
				if($blankTarget) $btOptions['forceHRef']=true;
				return outButtonSmall('',$href,$ico,false,false,$extraHTML.(($blankTarget)?' target="_blank"':'').' style="'.$extraStyle.'"',$btOptions);
			}
		}
	}

	if(cfIsMobile()) {
		$type='mobile';
		$extraStyle='';
	}
	elseif(@$opt['button']) {
		$type='button';
		$extraStyle='';
	}
	elseif(@$opt['raw16']) {
		$type='raw16';
		$extraStyle='';
	}
	else {
		$type='buttonSmall';
		$extraStyle='margin:0 1px 0 0';
	}

	// Url with password
	$passwordUrl=$url;
	if(@$opt['password']) $passwordUrl=$url.((strpos($url,'?')>0)?'&':'?').'password=b64.'.cfBase64_url_encode($opt['password']);

	// Partly url-encoded link(for clipboard copy)
	if($pos=strpos($url,'?')) $partEncodedURL=substr($url,0,$pos+1).str_replace('%3D','=',str_replace('%26','&',urlencode(substr($url,$pos+1)))); else $partEncodedURL=$url;

	// Clipboard copy
	if(cfCmpLeft(cfGetBrowser(),'ie') && !@$opt['clipboard']) $out.=(($out)?$sep:'').outSocialURLShareBt($type,'javascript:void(0);',outIcon('paste'),false,$extraStyle,'onmouseup="wl.clipboardCopy(\''.$partEncodedURL.'\')" ',cfCaption('genCopy'),array('bwIco'=>@$opt['bwIco']));

	// Mail
	if($type=='buttonSmall' || $type=='raw16') $ico=outIcon('mailOut');
	else if($type=='mobile') $ico=outIcon('mailOut64');
	if(!@$opt['mail']) $out.=(($out)?$sep:'').outSocialURLShareBt($type,cfMailTo(false,cfUTF8Encode($text),cfUTF8Encode((isset($opt['fullText'])))?$opt['fullText']:$url),$ico,!cfIsInApp(),$extraStyle,false,cfCaption('sendByMail'),array('forceHRef'=>1,'bwIco'=>@$opt['bwIco']));

	// Facebook
	if($type=='buttonSmall' || $type=='raw16') $ico=outIcon('facebook14');
	else if($type=='mobile') $ico=outIcon('facebook64');
	if(!@$opt['facebook']) $out.=(($out)?$sep:'').outSocialURLShareBt($type,'http://www.facebook.com/share.php?u='.urlencode($url).(($text)?'&t='.urlencode(cfUTF8Encode($text)):''),$ico,true,$extraStyle,false,cfCaption('genShareWith','Facebook'),array('forceHRef'=>1,'bwIco'=>@$opt['bwIco']));

	// Twitter
	if($type=='buttonSmall' || $type=='raw16') $ico=outIcon('twitter14');
	else if($type=='mobile') $ico=outIcon('twitter64');
	if(!@$opt['twitter']) $out.=(($out)?$sep:'').outSocialURLShareBt($type,'http://twitter.com/share?url='.urlencode($url).(($text)?'&text='.urlencode($text):''),$ico,true,$extraStyle,cfCaption('genShareWith','Twitter'),'Twitter',array('forceHRef'=>1,'bwIco'=>@$opt['bwIco']));

	// QR Code button
	if(!@$opt['qrcode'] && !@$opt['displayQRCode']) $out.=(($out)?$sep:'').outSocialURLShareBt($type,"javascript:wl.qrCodeShow('".cfBase64_url_encode($passwordUrl)."','".($id=cfGenerateID(4))."')",outIcon('qrCode14'),false,$extraStyle,'id="'.$id.'"', cfCaption('qrCode'),array('forceHRef'=>1,'bwIco'=>@$opt['bwIco']));

	// AddThis
	if($type=='buttonSmall' || $type=='raw16') $ico=outIcon('addthis14');
	else if($type=='mobile') $ico=outIcon('addthis64');
	if(!@$opt['addthis']) $out.=(($out)?$sep:'').outSocialURLShareBt($type,'http://www.addthis.com/bookmark.php?url='.urlencode($url),$ico,true,$extraStyle.(($type=='buttonSmall')?';left:2px':''),cfCaption('genShareWith','AddThis'),'AddThis',array('forceHRef'=>1,'bwIco'=>@$opt['bwIco']));

	// Inline QR Code (used by mobile remote view)
	if(@$opt['displayQRCode']) $out.='<br><br><img src="/ext/qrCode.php?url='.cfBase64_url_encode($url).'" style="width:40%" alt="QR Code"><br>';

	return $out;
}

/**
 * @desc Return code for horizontal separator (used in user interface)
 *
 * @return string
 */
function outConfigSeparator(){
	return '<div class="configSeparator"></div>';
}

/*
 ***************************************************************************************************************************
 * Asynchronous controls
 ***************************************************************************************************************************
 */

/**
 * @desc insert an image and / or a caption depending on $controlDefinitionArray definition
 *
 * @param array $controlDefinitionArray
 *			$controlDefinitionArray is a one or two elements array of image or caption definition (icd)
 *			(if $controlDefinitionArray is a string then it is treated as a one-element array)
 *			- if icd is false, don't display anything
 *			- if documentRoot/icd or resource's directory/icd is an image, display this image
 *			- else display icd as a caption
 *
 * @param alignLabelControl or alignControlLabel $align : defines wether control is located right or left of caption/image
 *			use this parameter to insert right or left margins
 * @return string: HTML output
 */
function outDisplayLabelAndOrImage($controlDefinitionArray,$align){
	$output='';
	if($align==alignLabelControl) $alignStyle="margin-right:1em;"; else $alignStyle="margin-left:1em;";
	// Check $controlDefinitionArray format
	if(!is_array($controlDefinitionArray)) {$item1=$controlDefinitionArray;$item2=false;}
	elseif (count($controlDefinitionArray)==1) {list($item1)=$controlDefinitionArray;$item2=false;}
	elseif (count($controlDefinitionArray)==2) {list($item1,$item2)=$controlDefinitionArray;}
	else return false;

	if($item1){
		// Check if item1 is an image
		if(file_exists(cfAppDocRoot().$item1)) $output.=outImage($item1,false,false,$alignStyle);
		// Else item1 is a caption
		else  $output.='<span  style="'.$alignStyle.'">'.$item1.'</span>';
	}
	if($item2){
		// Check if item2 is an image
		if(file_exists(cfAppDocRoot().$item2)) $output.='<img alt="" style="'.$alignStyle.'" src="'.$item2.'">';
		elseif(isset($resourceData['type']) && file_exists(cfAppDocRoot().'/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$item2)) $output.='<img alt="" style="'.$alignStyle.'" src="/res/'.$resourceData['type'].'/'.$resourceData['subType'].'/'.$item2.'">';
		// Else item2 is a caption
		else  $output.='<span  style="'.$alignStyle.'">'.$item2.'</span>';
	}
	// Insert spacing after caption/image if needed
	$output.="\n";
	if($align==alignControlLabel && ($item1 || $item2)){
		$item1=strtolower($item1);
		$item2=strtolower($item2);
		if(($item1 && (substr($item1,-4)=='<br>' || substr($item1,-5)=='<br/>'))) return true;
		if(($item2 && (substr($item2,-4)=='<br>' || substr($item2,-5)=='<br/>'))) return true;
		$output.="&nbsp;&nbsp;\n";
	}
	return $output;
}

/**
 * @desc Initialize data array used by ouControl function
 *
 * @param array $data
 */
function outControlSetData($data){
	global $outCD;
	global $outControlAsyncResponse;
	$outControlAsyncResponse=''; // Asynchronous response (used to indicate whether passed value is OK or not)
	$outCD=$data;
}

/**
 * @desc Get new data array after outControl processing
 *
 * @param array $data
 * @return true if data has been modified
 */
function outControlGetData(&$data){
	global $outCD;
	$modified=false;
	foreach ($outCD as $key=>$value) if(!isset($data[$key]) || $outCD[$key]!=$data[$key]) {$modified=true;break;}
	if(!$modified) foreach ($data as $key=>$value) if(!isset($outCD[$key])) {$modified=true;break;}
	$data=$outCD;
	return $modified;
}

/**
 * @desc process a data item:
 *		- set default value
 *		- modify value if POST parameter received
 *		- display control for user input
 *
 * @param string $dataName : name of data
 * @param integer $dataType : type of data :
 * 		- dataText : text data
 * 		- dataPassword : MD5 hashed password
 * 		- dataNumeric : numeric data
 * 		- dataList : fixed list of data
 * 		- dataBoolean boolean
 * 		- dataFolderPath : path to folder
 * 		- dataFilePath : path to file
 * 		- dataImage : image filename
 *
 * @param mixed $defaultValue: default value of data
 * @param mixed $constraints: authorized values for data (see below). No constraint if set to false
 * @param mixed $controlDefinition: user control definition (caption, icons,... see below). Control is not displayed if set to false
 * @param array $extraOptions
 * 				width: slider bar width
 * 				extraStyle
 * @return boolean : true if OK, false if function parameters are not correct
 */
function outControl($dataName, $dataType, $defaultValue, $constraints, $controlDefinition=false,$extraOptions=array()){
	global $outCD;
	global $outControlAsyncResponse;
	static $init=0;
	$output='';

	/*
	***************************************************************************************************************************
	* SET DEFAULT VALUE
	***************************************************************************************************************************
	*/
	if(!isset($outCD[$dataName])) $outCD[$dataName]=$defaultValue;

	/*
	***************************************************************************************************************************
	* PROCESS POST COMMANDS
	***************************************************************************************************************************
	*/

	// Set new value if passed
	if(isset($_POST['asUpdtName']) && $_POST['asUpdtName']==$dataName) {
		$requestedValue=cfUTF8Decode($_POST['asUpdtValue'],true,false,false);
		// If true, indicate that an error has been encountered while modifying the value
		$failed=false;

		switch($dataType){
			// Text data
			case dataText:
				if(is_array($constraints) && count($constraints)==2){
					list($nbMin,$nbMax)=$constraints;
					if(($nbMin===false || strlen($requestedValue)>=$nbMin) && ($nbMax===false || strlen($requestedValue)<=$nbMax)) $outCD[$dataName]=$requestedValue;
					else $failed=true;
				}
				elseif (!$constraints) $outCD[$dataName]=$requestedValue;
				break;

			// Password
			case dataPassword:
				if(is_array($constraints) && count($constraints)==2){
					list($nbMin,$nbMax)=$constraints;
					if(($nbMin===false || strlen($requestedValue)>=$nbMin) && ($nbMax===false || strlen($requestedValue)<=$nbMax)) $outCD[$dataName]=md5($requestedValue);
					else $failed=true;
				}
				elseif (!$constraints) $outCD[$dataName]=md5($requestedValue);
				break;

			// Numeric data
			case dataNumeric:
				if(is_numeric($requestedValue) && is_array($constraints) && count($constraints)==2){
					list($nbMin,$nbMax)=$constraints;
					if(($nbMin===false || $requestedValue>=$nbMin) && ($nbMax===false || $requestedValue<=$nbMax)) $outCD[$dataName]=$requestedValue;
					else $failed=true;
				}
				elseif (is_numeric($requestedValue) && !$constraints) $outCD[$dataName]=$requestedValue;
				else $failed=true;
				break;
			// List
			case dataList:
				if(is_array($constraints) && in_array($requestedValue,$constraints)) $outCD[$dataName]=$requestedValue;
				else $failed=true;
				break;
			// Boolean data
			case dataBoolean:
				if($requestedValue=='true' || $requestedValue=='on') $outCD[$dataName]=true;
				else $outCD[$dataName]=false;
				break;
			// File or folder path
			case dataFolderPath:
			case dataFilePath:
				$requestedValue=str_replace('\\\\','/',$requestedValue);
				if(substr($requestedValue,-1)=='/') $requestedValue=substr($requestedValue,0,strlen($requestedValue)-1);
				if($dataType==dataFolderPath){
					if(is_dir($requestedValue)) $outCD[$dataName]=$requestedValue;
					elseif($requestedValue==cfCaption('explorerComputerRoot') && $constraints!='noComputerRoot') $outCD[$dataName]='computerRoot';
					else $failed=true;
				}
				elseif ($dataType==dataFilePath){
					if(is_file($requestedValue)) $outCD[$dataName]=$requestedValue;
					else $failed=true;
				}
				if(cfIsSubDir(cfAppDataRootDir(),$outCD[$dataName])) $outCD[$dataName]='*appDir*'.substr($outCD[$dataName],strlen(cfAppDataRootDir()));
				break;
			// Image
			// Constraints: array(path to images, array(allowed extensions)) | path to images
			case dataImage;
				if(file_exists(((is_array($constraints))?$constraints[0]:$constraints)).'/'.str_replace('/','',str_replace('\\','',$requestedValue))) $outCD[$dataName]=str_replace('/','',str_replace('\\','',$requestedValue));
				else $failed=true;

				break;
		}
		// Asynchronous response
		$outControlAsyncResponse=cfAsyncXMLJSaction('dgn("asUpdt'.$dataName.'").style.color="'.(($failed)?'red':'').'"');
	}

	/*
	***************************************************************************************************************************
	* DISPLAY CONTROL
	***************************************************************************************************************************
	*/
	if($controlDefinition){
		// Insert javascript and form
		if(!$init){
			$init=1;
			$output.='<script type="text/javascript">var asUpdtTO;';
			// Async update data
			$output.='function asUpdt(dataName,dataValue,delay){';
			$output.='clearTimeout(asUpdtTO);if(delay){asUpdtTO=setTimeout("asUpdt(\'"+dataName+"\',\'"+dataValue.replace(/\'/g,"\\\\\'")+"\')",1000);return}';
			$output.='asUpdtTO=0;dgn("asUpdtName").value=dataName;dgn("asUpdtValue").value=dataValue;asyncSubmitForm("asUpdtForm");if(W.asUpdtFunction) asUpdtFunction(dataName,dataValue);}';
			// Show update image popup
			$output.='function asUpdtImg(dataName){wl.mask.show(0,1);var node=dgi("asUpdtImg"+dataName); if(node) node.parentNode.removeChild(node); if(!dgi("asUpdtImgMV"+dataName)){document.body.insertBefore(node,wl.mask.node()); node.id="asUpdtImgMV"+dataName;} else node=dgi("asUpdtImgMV"+dataName); node.style.visibility="hidden";node.style.display="";wl.mask.moveAbove(node);$(node).e.center();fade(node,0,1);node.focus()}';
			// Hide update image popup
			$output.='function asUpdtImgH(dataName){wl.mask.hide(1);var node=dgi("asUpdtImgMV"+dataName); node.style.display="none";}';
			// Select an update image
			$output.='function asUpdtImgS(dataName,dataValue){asUpdtImgH(dataName);var src=dgn("asUpdt"+dataName).src; dgn("asUpdt"+dataName).src=src.substr(0,src.lastIndexOf("/")+1)+dataValue;asUpdt(dataName,dataValue)}';
			// Update image esc detection
			$output.='function asUpdtImgKP(dataName,evt){if(!evt) evt=W.event;if(evt.keyCode==27) return asUpdtImgH(dataName)}';
			$output.='</script>';
			$output.='<form id="asUpdtForm" method="POST" action="'.$_SERVER['PHP_SELF'].'" style="display:none"><input type="text" name="asUpdt" value="1"><input type="text" name="asUpdtName"><input type="text" name="asUpdtValue"></form>';
		}
		switch($dataType){
			// TEXT OR NUMERIC DATA
			// Control definition :
			// 	array(input lenght, $controlDefinitionArray) : text input (size=input lenght) with image and/or caption
			// 	array($controlDefinitionArray,input lenght) : image and/or caption with text input (size=input lenght)
			// 	array($controlDefinitionArray,input lenght, $controlDefinitionArray) : image and/or caption with text input (size=input lenght) and image and/or caption
			case dataText:
			case dataNumeric:
				if(!is_array($controlDefinition)) $controlDefinition=array($controlDefinition,45); // default control : set size at 45
				if(count($controlDefinition)==2){
					list($item1,$item2)=$controlDefinition;
					// Textbox - caption
					if(is_numeric($item1)){
						$output.='<input onKeyUp="asUpdt(\''.$dataName.'\',this.value,1)" onChange="asUpdt(\''.$dataName.'\',this.value)" type="text" class="textInput" name="asUpdt'.$dataName.'" size="'.$item1.'" value="'.str_replace('"','&quot;',cfUTF8Encode($outCD[$dataName])).'" '.((@$extraOptions['extraStyle'])?'style="'.$extraOptions['extraStyle'].'"':'').'>';
						$output.=outDisplayLabelAndOrImage($item2,alignControlLabel);
					}
					// Caption - Textbox
					elseif (is_numeric($item2)){
						$output.=outDisplayLabelAndOrImage($item1,alignLabelControl);
						$output.='<input onKeyUp="asUpdt(\''.$dataName.'\',this.value,1)" onChange="asUpdt(\''.$dataName.'\',this.value)" type="text" class="textInput" name="asUpdt'.$dataName.'" size="'.$item2.'" value="'.str_replace('"','&quot;',cfUTF8Encode($outCD[$dataName])).'" '.((@$extraOptions['extraStyle'])?'style="'.$extraOptions['extraStyle'].'"':'').'>';
					}
				}
				// Caption - Textbox - Caption
				elseif (count($controlDefinition)==3 && is_numeric($controlDefinition[1])){
						list($item1,$item2,$item3)=$controlDefinition;
						$output.=outDisplayLabelAndOrImage($item1,alignLabelControl);
						$output.='<input onKeyUp="asUpdt(\''.$dataName.'\',this.value,1)" onChange="asUpdt(\''.$dataName.'\',this.value)" type="text" class="textInput" name="asUpdt'.$dataName.'" size="'.$item2.'" value="'.str_replace('"','&quot;',cfUTF8Encode($outCD[$dataName])).'" '.((@$extraOptions['extraStyle'])?'style="'.$extraOptions['extraStyle'].'"':'').'>';
						$output.=outDisplayLabelAndOrImage($item3,alignControlLabel);
				}
			break;
			// PASSWORD
			// See TEXT for control definition
			case dataPassword:
				if(!is_array($controlDefinition)) $controlDefinition=array($controlDefinition,45); // default control : set size at 45
				if(count($controlDefinition)==2){
					list($item1,$item2)=$controlDefinition;
					// Textbox - caption
					if(is_numeric($item1)){
						$output.='<input onKeyUp="asUpdt(\''.$dataName.'\',this.value,1)" onChange="asUpdt(\''.$dataName.'\',this.value)" type="password" class="textInput" name="asUpdt'.$dataName.'" size="'.$item1.'" value="" '.((@$extraOptions['extraStyle'])?'style="'.$extraOptions['extraStyle'].'"':'').'>';
						$output.=outDisplayLabelAndOrImage($item2,alignControlLabel);
					}
					// Caption - Textbox
					elseif (is_numeric($item2)){
						$output.=outDisplayLabelAndOrImage($item1,alignLabelControl);
						$output.='<input onKeyUp="asUpdt(\''.$dataName.'\',this.value,1)" onChange="asUpdt(\''.$dataName.'\',this.value)" type="password" class="textInput" name="asUpdt'.$dataName.'" size="'.$item2.'" value="" '.((@$extraOptions['extraStyle'])?'style="'.$extraOptions['extraStyle'].'"':'').'>';
					}
				}
				// Caption - Textbox - Caption
				elseif (count($controlDefinition)==3 && is_numeric($controlDefinition[1])){
						list($item1,$item2,$item3)=$controlDefinition;
						$output.=outDisplayLabelAndOrImage($item1,alignLabelControl);
						$output.='<input onKeyUp="asUpdt(\''.$dataName.'\',this.value,1)" onChange="asUpdt(\''.$dataName.'\',this.value)" type="password" class="textInput" name="asUpdt'.$dataName.'" size="'.$item2.'" value="" '.((@$extraOptions['extraStyle'])?'style="'.$extraOptions['extraStyle'].'"':'').'>';
						$output.=outDisplayLabelAndOrImage($item3,alignControlLabel);
				}
			break;

			// VALUES LIST : 2 types of controls : <select> droplist or radio buttons
			// Control definition :
			// <select> :
			//		array($controlDefinitionArray, array of items, $controlDefinitionArray)
			// 		items : array(value, caption)
			//  Radio buttons:
			//		array of items
			// 		items : array(left-side $controlDefinitionArray, value, right-side $controlDefinitionArray)
			case dataList:
				if($controlDefinition=='slider'){
					// HTML objects
					$output.='<fieldset class="asCtrlHSliderExt" name="asUpdt'.$dataName.'_ext" onmouseup="controlSliderC'.$dataName.'()" '.((@$extraOptions['extraStyle'])?'style="'.$extraOptions['extraStyle'].'"':'').'>';
					$output.='<legend></legend>';
					$output.='<div class="asCtrlHSliderBlock" '.((isset($extraOptions['width']))?'style="width:'.$extraOptions['width'].'"':'').'></div>';
					$output.='<div class="asCtrlHSliderBar" id="asUpdt'.$dataName.'_bar" onmouseup="controlSliderC'.$dataName.'()"';
					//if(isset($extraOptions['width'])) $output.='style="width:'.$extraOptions['width'].'"';
					$output.='></div>';
					$output.='<div class="asCtrlHSliderThumb" id="asUpdt'.$dataName.'_thumb"></div>';
					$output.='<input class="textInput asCtrlHSliderTxt" type="text" size="1" name="asUpdt'.$dataName.'" disabled="disabled">';
					$output.='</fieldset>';
					// Drag items
					cfDragAddItem('asUpdt'.$dataName.'_bar','+NO_DRAG+CURSOR_HAND');
					cfDragAddItem('asUpdt'.$dataName.'_thumb','+HORIZONTAL+CURSOR_HAND+"DRAGFUNC(controlSlider'.$dataName.')"','controlSliderInit'.$dataName.'()');

					// Associated scripts
					$output.='<script type="text/javascript">';
					$output.='var controlSlider'.$dataName.'Values=new Array('.$outCD[$dataName];
					foreach ($constraints as $v) $output.=','.$v;
					$output.=');';
					$output.='function controlSlider'.$dataName.'(){';
					$output.='var thumb=dd.elements.asUpdt'.$dataName.'_thumb;';
					$output.='var bar=dd.elements.asUpdt'.$dataName.'_bar;';
					$output.='var arr=controlSlider'.$dataName.'Values;';
					$output.='if(thumb.x+thumb.w>bar.x+bar.w) thumb.moveTo(bar.x+bar.w-thumb.w,false);';
					$output.='if(thumb.x<bar.x)thumb.moveTo(bar.x,false);';
					$output.='var pos=(thumb.x-bar.x)/(bar.w-thumb.w);pos=arr[1+Math.floor(pos*(arr.length-2))];';
					$output.='if(arr[0]==pos) return;arr[0]=pos;dgn("asUpdt'.$dataName.'").value=pos;';
					$output.='if(W.my_asUpdt) my_asUpdt("'.$dataName.'",pos);'.((isset($extraOptions['noAsync']))?'':'asUpdt("'.$dataName.'",pos+"",1)').'}';
					$output.='function controlSliderC'.$dataName.'(e){dd.elements.asUpdt'.$dataName.'_thumb.moveTo(mouseEvt.x-dd.elements.asUpdt'.$dataName.'_thumb.w/2,false);controlSlider'.$dataName.'()}';

					// Init function
					$initCommands ='var thumb=dd.elements.asUpdt'.$dataName.'_thumb;';
					$initCommands.='var bar=dd.elements.asUpdt'.$dataName.'_bar;';
					$initCommands.='var arr=controlSlider'.$dataName.'Values;';
					$initCommands.='thumb.setZ(bar.z+1);';
					$initCommands.='thumb.minXPos=bar.x+1;';
					$initCommands.='thumb.maxXPos=bar.x+bar.w-thumb.w;';
					$initCommands.='pos=0; while(pos+2<arr.length && arr[0]>arr[pos+1]) pos++;';
					$initCommands.='if(arr.length>1) pos=pos/(arr.length-1);';
					$initCommands.='thumb.moveTo(thumb.minXPos+pos*(thumb.maxXPos-thumb.minXPos),false);';
					$initCommands.='dgn("asUpdt'.$dataName.'").value=arr[0];';
					$output.='function controlSliderInit'.$dataName.'(){'.$initCommands.'}';
					$output.='</script>';

				}
				elseif(!is_array($controlDefinition) || count($controlDefinition)<2) return false;
				//<select>
				elseif(count($controlDefinition)==3 && is_array($controlDefinition[1]) && ((!is_array($controlDefinition[0])) || count($controlDefinition[0])<3)){
					list($item1,$item2,$item3)=$controlDefinition;
					$output.=outDisplayLabelAndOrImage($item1,alignLabelControl);
					$output.='<select name="asUpdt'.$dataName.'" size="1" class="textInput"  onChange="asUpdt(\''.$dataName.'\',this.value)">';
					foreach ($item2 as $value) {if(is_array($value) && count($value)==2) $output.='<option value="'.$value[0].'"'.(($outCD[$dataName]==$value[0])?' selected':'').'>'.$value[1]."</option>\n";}
					$output.="</select>\n";
					$output.=outDisplayLabelAndOrImage($item3,alignControlLabel);
				}
				else{
				// Radio buttons
					foreach ($controlDefinition as $value){
						if(is_array($value) && count($value)==3){
							list($item1,$item2,$item3)=$value;
							$output.=outDisplayLabelAndOrImage($item1,alignLabelControl);
							$output.='<input type="radio" name="asUpdt'.$dataName.'"  onClick="asUpdt(\''.$dataName.'\',this.value)" value="'.$item2.'" '.(($outCD[$dataName]==$item2)?'checked="checked"':'').">\n";
							$output.=outDisplayLabelAndOrImage($item3,alignControlLabel);
						}
					}
				}
			break;
			// BOOLEAN DATA
			case dataBoolean:
				// Checkbox
				// Control definition :
				// 	array($controlDefinitionArray) : checkbox with image and/or caption
				//	array($controlDefinitionArray,$controlDefinitionArray) : 2 radio buttons with image and/or caption
				if(!is_array($controlDefinition)) $controlDefinition=array($controlDefinition);
				// Checkbox
				if(count($controlDefinition)==1){
					$output.='<input onChange="asUpdt(\''.$dataName.'\',this.checked)" onClick="asUpdt(\''.$dataName.'\',this.checked)" type="checkbox" name="asUpdt'.$dataName.'" '.(($outCD[$dataName])?'checked':'').' />';
					$output.='<span onclick="obj=dgn(\'asUpdt'.$dataName.'\'); if(obj.disabled) return; obj.checked=!obj.checked;asUpdt(\''.$dataName.'\',obj.checked);">';
					$output.=outDisplayLabelAndOrImage($controlDefinition[0],alignControlLabel);
					$output.='</span>';
				}
				// Radio button (Warning : first item is always true, second is always false)
				elseif(count($controlDefinition)==2){
					$first=true;
					foreach ($controlDefinition as $value){
						// radio button

						// "true" button
						if($first) $output.='<input onChange="asUpdt(\''.$dataName.'\',this.value)" onClick="asUpdt(\''.$dataName.'\',this.value)" type = "radio" name="asUpdt'.$dataName.'" value="true" '.(($outCD[$dataName])?'checked':'').'>';
						// "false" button
						else $output.='<input onChange="asUpdt(\''.$dataName.'\',!this.value)" onClick="asUpdt(\''.$dataName.'\',!this.value)" type = "radio" name="asUpdt'.$dataName.'" value="false" '.((!$outCD[$dataName])?'checked':'').'>';
						$output.='<span onclick="obj=document.getElementsByName(\''.$dataName.'\')['.(($first)?'0':'1').'];if(obj.disabled) return;obj.checked=true;asUpdt(\''.$dataName.'\','.(($first)?'!':'').'this.checked);">';
						$output.=outDisplayLabelAndOrImage($value,alignControlLabel);
						$output.='</span>';
						if($first) $first=false;
					}
				}
			break;

			// FILE OR DIRECTORY
			// Control definition:
			//	optional string: caption
			case dataFolderPath:
			case dataFilePath:
				// Corrects file/path case
				$path=cfCorrectFilenameCase(str_replace('*appDir*',cfAppDataRootDir(),$outCD[$dataName]),true);
				if(cfIsSubDir(cfAppDataRootDir(),$path)) $outCD[$dataName]='*appDir*'.substr($path,strlen(cfAppDataRootDir())); else $outCD[$dataName]=$path;

				$output.='<img alt="" src="'.(($dataType==dataFolderPath)?outIcon('folder'):outIcon('blankFile')).'" style="vertical-align:middle; margin-right:0.5em;">';
				// Caption
				if($controlDefinition!==true && !is_array($controlDefinition)) $output.=$controlDefinition;
				elseif($dataType==dataFolderPath) $output.='<b>'.cfCaption('genDirectory').'</b>';
				else $output.=cfCaption('genFile');
				// Input text box
				$output.='<input class="textInput" onKeyUp="asUpdt(\''.$dataName.'\',this.value);this.style.color=\'\';" onChange="asUpdt(\''.$dataName.'\',this.value);" type="text" name="asUpdt'.$dataName.'" id="'.$dataName.'" size="36" style="margin-left:1em;margin-right:1em;'.(($path!='computerRoot' && !file_exists($path))?'color:red;':'').'" value="'.(($path=='computerRoot')?cfCaption('explorerComputerRoot'):cfUTF8Encode($path)).'">';
				if($dataType==dataFolderPath){
					// Folder selection button (application environment only)
					if(cfIsInApp())
						$output.=outButton(cfCaption('genBrowse'),"javascript:wl.UICommand('openFolder','".$dataName."')",outIcon('fi/folder'),cfCaption("genDirectory"),false,'onclick="asUpdt(\''.$dataName.'\',this.value)"');
					// Computer root button
					if($constraints!='noComputerRoot') $output.=outButton('','javascript:dgi(\''.$dataName.'\').value=\''.cfCaption('explorerComputerRoot').'\';asUpdt(\''.$dataName.'\',this.value);', outIcon('computerRoot'), cfCaption("explorerComputerRoot"));
					// Special : 'path' data is commonly used as base directory for explorer resources - show help button
					if($dataName=='path') $output.='&nbsp;&nbsp;&nbsp;'.outHelpButton('helpPath');
				}
				else{
					// File selection button (application environment only)
					if(cfIsInApp())
						$output.=outButton(cfCaption('genBrowse'),"javascript:wl.UICommand('openFile','".$dataName.";".$constraints."')",outIcon('explorer'),false,false,'onclick="asUpdt(\''.$dataName.'\',this.value)"');
				}
			break;

			// IMAGE
			// Control definition:
			// left-hand caption | Array(left-hand caption, extraStyle)
			// Use constraint as path to images
			case dataImage:
				$path=((is_array($constraints))?$constraints[0]:$constraints);
				if(substr($path,0,strlen(cfAppDocRoot()))==cfAppDocRoot()) $path=substr($path,strlen(cfAppDocRoot()));
				// Caption
				$output.=((isset($controlDefinition[0]))?$controlDefinition[0]:$controlDefinition).'&nbsp;&nbsp;';
				// Image
				if(strpos(@$controlDefinition[1],'width')!==false && strpos(@$controlDefinition[1],'height')!==false) // if width & height are passed force image size
					$output.='<a class="imageSelector"><img src="'.$path.'/'.$outCD[$dataName].'" onclick="asUpdtImg(\''.$dataName.'\')" name="asUpdt'.$dataName.'" style="'.$controlDefinition[1].'"></a>';
				else
					$output.=outImage($path.'/'.$outCD[$dataName],'javascript:asUpdtImg(\''.$dataName.'\')','name="asUpdt'.$dataName.'" class="imageSelector"',((isset($controlDefinition[1]))?$controlDefinition[1]:false));
				// Image selection popup
				$output.='<div class="popup" style="text-align:center;display:none;width:400px;height:400px;" id="asUpdtImg'.$dataName.'" onkeypress="asUpdtImgKP(\''.$dataName.'\',event)">';
				$output.=outFrameHeaderTable('popupHeader',
					((@count($controlDefinition)==1)?$controlDefinition[0]:''),
					outButton(cfCaption('genCancel'),'javascript:asUpdtImgH(\''.$dataName.'\')',outIcon('cancel')));
				$output.='<div style="text-align:left; height:370px;overflow:auto">';
				foreach (glob(((is_array($constraints))?$constraints[0]:$constraints).'/'.((is_array($constraints) && isset($constraints[1]))?'*.{'.implode(',',$constraints[1]).'}':'*.*'),GLOB_BRACE) as $file){
					$output.=outImage($path.'/'.basename($file),'javascript:asUpdtImgS(\''.$dataName.'\',\''.basename($file).'\')','class="imageSelectorImg"');
				}
				$output.='</div></div>';
			break;
		}
	}
	return $output;
}

/**
 * @desc Return async control modified data name
 *
 * @return mixed value, null if no command passed
 */
function outControlCommandName(){
	if(!isset($_POST['asUpdtName'])) return null;
	return $_POST['asUpdtName'];
}

/**
 * @desc Return async control modified data value
 *
 * @return mixed value, null if no command passed
 */
function outControlCommandValue(){
	if(!isset($_POST['asUpdtValue'])) return null;
	return $_POST['asUpdtValue'];
}

/**
 * @desc Return code for thumnails size buttons
 *
 * @param bool $resourceHasThumbnails: true if...
 * @return string
 */
function outThumbnailsSizeButtons($resourceHasThumbnails=true){
	// If locked, do not display control
	if(cfRGetVar('thumbnailsSizeFactorLocked') || ((!$resourceHasThumbnails) && efFilesListIsResource())) return '';

	$ts=cfRGetVar('thumbnailsSizeFactor');
	$dis=(efFilesListType()=='list');
	$out='';
	for($i=0;$i<3;$i++)
		$out.=outButton(false,'javascript:efTNSize('.$i.')',outIcon('efTNSize'.$i.''),cfCaption('genSize').' - '.(1+$i).'/3','efTNSize'.$i,false,false,$dis||($ts==$i));
	return $out;
}

/**
 * @desc Return code for a selectable/hoverable frameset item
 *
 * @param string $content: HTML
 * @param string $extraHTML: extra HTML embedded in <fieldset>
 * @param string $extraStyle: extra style of <fieldset> element
 * @param string $options: onclick=>onclick javascript function(), type=>'checkbox' or 'radio', 'extraClass': class added to fieldset;
 * @param string $state: 'sel', 'unsel', 'disabled'
 */
function outBlockItem($groupName,$content,$extraHTML='',$extraStyle='',$options=false,$state='unsel'){
	$out='<fieldset name="'.$groupName.'" ';
	$out.='class="blockItem'.(($state=='unsel')?'':(($state=='sel')?' biSel':' biDisabled')).((isset($options['extraClass']))?' '.$options['extraClass']:'').'" ';
	$out.='onmouseover="wl.blockItem(this).over();'.((isset($options['onmouseover']))?$options['onmouseover']:'').'" ';
	$out.='onmouseout="wl.blockItem(this).out();'.((isset($options['onmouseout']))?$options['onmouseout']:'').'" ';
	$out.='onclick="wl.blockItem(this).select(1,'.((int)(@$options['type']=='checkbox')).');'.((isset($options['onclick']))?$options['onclick']:'').'" ';
	$out.='style="'.$extraStyle.'"';
	$out.=' '.$extraHTML;
	return $out.'>'.$content.'</fieldset>';
}
?>