<?php
/**
 * WebDAV access script (SabreDAV based: http://code.google.com/p/sabredav/)
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2012 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 4.0.0
 */

// Includes
require_once(INCLUDE_DIR.'explorerFunctions.php');
require_once(INCLUDE_DIR.'outputFunctions.php');

$_ENV['noDebug']=1; if(0) cfDbg();


// Include sabreDAV
list($major,$minor,$release)=@explode('.',phpversion());
if($major>=5 && $minor>=3)
	set_include_path(INCLUDE_DIR.'SabreDAV/lib/' . PATH_SEPARATOR . get_include_path());
else
	set_include_path(INCLUDE_DIR.'SabreDAV5.8/lib/' . PATH_SEPARATOR . get_include_path());

require_once 'Sabre/autoload.php';


/**
 * @Desc Base directory management class
 *
 */
class ResourceDirectory extends Sabre_DAV_Collection {

	private $myPath;

	/**
	 * Constructor
	 *
	 * @param string $myPath: directory or *resourceBasePath*
	 */
	function __construct($myPath) {
		$this->myPath = $myPath;
		$this->fileRights=cfFileRights($myPath);
	}

	/**
	 * Get directory's files
	 *
	 * @return array
	 */
	function getChildren() {
		$children = array();

		// Special: if client goes on upper dirs, just return /dav/xyz tree
		if($_ENV['baseURI']=='/'){$children[] = new ResourceDirectory('/dav'); return;}
		if($_ENV['baseURI']=='/dav'){$children[] = new ResourceDirectory(cfRGetVar('webDAVbaseUri')); return;}

		cfRSetVar('efCurrentDirectory',$this->myPath);

		efDirUpdate(false);


		// Computer root (Windows)
		if($this->myPath=='*resourceBasePath*' && cfRGetVar('path')=='computerRoot'){
			foreach($_ENV['weezoResEfFiles'] as $key=>$file) if(is_numeric($key)){
				//if(cfFileRights($file['name'],'read')) $children[] = $this->getChild(str_replace(':/','',$file['name']));
				if(cfFileRights($file['name'],'read')) $children[] = $this->getChild($file['name']);
			}
			return $children;
		}

		// Loop through the directory, and create objects for each directory
		foreach($_ENV['weezoResEfFiles'] as $key=>$file) if(is_numeric($key) && is_dir(cfFilenameResolve($this->myPath.'/'.$file['name']))) {
			if(cfFileRights(cfFilenameResolve($this->myPath.'/'.$file['name']),'read')) $children[] = $this->getChild($file['name']);
		}
		// Loop through the directory, and create objects for each file
		foreach($_ENV['weezoResEfFiles'] as $key=>$file) if(is_numeric($key) && is_file(cfFilenameResolve($this->myPath.'/'.$file['name']))) {
			if(cfFileRights(cfFilenameResolve($this->myPath.'/'.$file['name']),'read')) $children[] = $this->getChild($file['name']);
		}
cfDbg('get children for '.$this->myPath.', '.(count($children)).' children returned','');
		return $children;
	}

	function getChild($name) {
		if(cfRGetVar('path')=='computerRoot' && $this->myPath=='*resourceBasePath*') $path=$name;
		else $path = cfFilenameResolve($this->myPath . '/' . $name);

		// We have to throw a NotFound exception if the file didn't exist
//if (!file_exists($path)) cfDbg('The file with name: ' . $name . ' could not be found');
		if (!file_exists($path)) throw new Sabre_DAV_Exception_NotFound('The file with name: ' . $name . ' could not be found');

		// Or an access denied
		if (!cfFileRights($path,'read'))  throw new Sabre_DAV_Exception_NotFound('Access denied');

		if (is_dir($path)) return new ResourceDirectory($path);
		return new ResourceFile($path);

	}

	function childExists($name) {
		$cfn=cfFilenameResolve($this->myPath.'/'.$name);
		return file_exists($cfn) && cfFileRights($cfn,'read');

	}

	/**
	 * @desc Return last-modified date
	 *
	 * @return integer
	 */
	function getLastModified(){
		return @filemtime(cfFilenameResolve($this->myPath));
	}

	/**
	 * @desc Return directory name
	 *
	 * @return string
	 */
	function getName() {
		$r=new Sabre_HTTP_Request();
		if($r->getMethod()=='GET') return (basename($this->myPath));

		return utf8_encode(basename($this->myPath));
	}

	/**
	 * @desc rename directory
	 *
	 * @param string $newName
	 */
	function setName($newName){
		if(cfCmpLeft($newName,'..') || cfCmpLeft($newName,'/')) return;

		$newName=dirname($this->myPath).'/'.$newName;

cfDbg('rename '.$this->myPath.' into '.$newName);

		if($this->fileRights['modify'] && cfFileRights(dirname($this->myPath),'write')) {
			rename($this->myPath,$newName);
		}
		else{
cfDbg('directory rename error, no access rights on '.$this->myPath,'');
		}
	}

	/**
	 * Delete directory
	 */
	function delete(){
		if($this->fileRights['modify']) {
cfDbg('delete directory '.$this->myPath.' ok','');
			cfUnlinkDir($this->myPath);
		}
		else {
			throw new Sabre_DAV_Exception_Forbidden('Permission denied to delete file');
			return false;
cfDbg('delete error, directory protected '.$this->myPath,'');
		}
	}

    /**
     * Creates a new file in the directory
     *
     * @param string $name Name of the file
     * @param resource $data Initial payload, passed as a readable stream resource.
     * @throws Sabre_DAV_Exception_Forbidden
     * @return void
     */
	public function createFile($name, $data = null) {

cfDbg('Create file '.$this->myPath.'/'.$name,'');
		if(cfCmpLeft($name,'/') || cfCmpLeft($name,'..')) {
			throw new Sabre_DAV_Exception_NotFound('Invalid file name');
			return;
		}

		// Directory write rights
		if(!cfFileRights(cfFilenameResolve($this->myPath),'write')){
cfDbg('no write in '.cfFilenameResolve($this->myPath),'');
			throw new Sabre_DAV_Exception_NotFound('Directory write access denied');
			return;
		}
		$cfn=cfFilenameResolve($this->myPath.'/'.$name,false);


		// Check file modify rights if file exists
		if(file_exists($cfn) && !cfFileRights($cfn,'modify')){
cfDbg('no write modify in '.cfFilenameResolve($this->myPath),'');
			throw new Sabre_DAV_Exception_NotFound('File modify access denied');
			return;
		}

		// Check file type
		if($filter=cfRGetVar('fileTypeFilter')){
			$mime=efFileType($cfn);
			if(!in_array($mime,$filter)){
cfDbg('forbidden file type '.$mime.' / '.$cfn,'');
				throw new Sabre_DAV_Exception_NotFound('File type creation denied');
				return;
			}
		}
		// Write data
cfDbg('Write data into '.$cfn,'');
		$written=file_put_contents($cfn,$data);
		return md5($cfn.@filesize($cfn));
		/*
		if(0 && $data && $out=fopen($cfn,'wb')){
			$written=0;
			while(!feof($data)) {
//				$d=fread($data,16384);
				$d=stream_get_contents($data);
				fwrite($out,$d);
				$written+=strlen($d);
			}
			fclose($out);
cfDbg('create: '.$written.' bytes written','');
			return true;
		}
		*/
	}

    /**
     * Creates a new directory in the directory
     *
     * @param string $name Name of the file
     * @param resource $data Initial payload, passed as a readable stream resource.
     * @throws Sabre_DAV_Exception_Forbidden
     * @return void
     */
	public function createDirectory($name) {

cfDbg('Create directory '.$this->myPath.'/'.$name,'');

	if(cfCmpLeft($name,'/') || cfCmpLeft($name,'..')) {
			throw new Sabre_DAV_Exception_NotFound('Invalid file name');
			return;
		}

		// Directory write rights
		if(!cfFileRights(cfFilenameResolve($this->myPath),'write')){
cfDbg('no write in '.cfFilenameResolve($this->myPath),'');
			throw new Sabre_DAV_Exception_NotFound('Directory write access denied');
			return;
		}

		$cfn=cfFilenameResolve($this->myPath.'/'.$name,false);
cfDbg('Directory created '.$cfn,'');
		// Create directory
		mkdir($cfn);
	}
}

/**
 * @Desc Base file management class
 *
 */
class ResourceFile extends Sabre_DAV_File {
	private $myPath; // complete file name
	private $fileRights; // Array of cfFileRights

	function __construct($myPath) {
		$this->fileRights=cfFileRights($myPath);
		if(file_exists($myPath) && $this->fileRights['download']) $this->myPath = $myPath;
	}

	/**
	 * @desc Return file name
	 *
	 * @return string
	 */
	function getName() {
cfDbg('get file name for '.$this->myPath,'');
		$r=new Sabre_HTTP_Request();
		if($r->getMethod()=='GET') return (basename($this->myPath));
		return utf8_encode(basename($this->myPath));
	}

	/**
	 * @desc Return file's data
	 *
	 * @return stream
	 */
	function get() {
cfDbg('Start reading '.$this->myPath,'');
		return fopen($this->myPath,'r');
	}

	/**
	 * @desc Return file size in bytes
	 *
	 * @return integer
	 */
	function getSize() {
cfDbg('get file size for '.$this->myPath,'');
		return cfFileSize($this->myPath);
	}

	/**
	 * @desc Return file's eTag (md5 of path+file size)
	 *
	 * @return string
	 */
	function getETag() {
cfDbg('get file eTag for '.$this->myPath,'');
		return '"' . md5($this->myPath.@filesize($this->myPath)) . '"';
	}

	/**
	 * @desc Return last modified timestamp
	 *
	 * @return integer
	 */
	function getLastModified(){
cfDbg('get last modified date for '.$this->myPath,'');
		return @filemtime($this->myPath);
	}

	/**
	 * @desc Write data into file
	 *
	 * @param stream $data
	 */
	function put($data){
cfDbg('put '.$data.' in '.$this->myPath,'');
		if($this->fileRights['modify'] || !@filesize($this->myPath)) {
			if($data && is_string($data)){
				$written=file_put_contents($this->myPath,$data);
cfDbg($written.' bytes written','');
			}
			elseif($data){
				if($out=fopen($this->myPath,'wb')){
					$written=0;
					while($d=fread($data,16384)){
						fwrite($out,$d);
						$written+=strlen($d);
					}
					fclose($out);
cfDbg($written.' bytes written','');
				}
			}

		}
		else throw new Sabre_DAV_Exception_Forbidden('Permission denied to change data');
	}

	/**
	 * Delete file
	 */
	function delete(){
		if($this->fileRights['modify']) {
cfDbg('delete '.$this->myPath.' ok','');
			unlink($this->myPath);
		}
		else {
			throw new Sabre_DAV_Exception_Forbidden('Permission denied to delete file');
cfDbg('delete error, file protected '.$this->myPath,'');
		}
	}

	/**
	 * @desc rename file
	 *
	 * @param string $newName
	 */
	function setName($newName){
cfDbg('Update filename for '.$this->myPath.' to '.$newName,'');
		if(cfCmpLeft($newName,'..') || cfCmpLeft($newName,'/')) return;

		$newName=dirname($this->myPath).'/'.$newName;

		//cfDbg('rename '.$this->myPath.' into '.$newName);

		if($this->fileRights['modify'] && cfFileRights(dirname($this->myPath),'write')) {
			rename($this->myPath,cfUTF8Decode($newName));
		}
		else{
cfDbg('file rename error, no access rights on '.$this->myPath,'');
		}
	}

	/**
	 * @desc Get mime type
	 *
	 * @return string
	 */
	public function getContentType() {
cfDbg('get content (mime) type for '.$this->myPath,'');
		if(!$this->myPath) throw new Sabre_DAV_Exception_Forbidden('Permission denied to read file type');
		else{
			require_once(INCLUDE_DIR.'mime_type.php');
			return mimeType(cfFileExtension($this->myPath));
		}
	}
}

/**
 * Auth class, fixing a Windows XP client bug
 *
 */
class ResourceDigestAuth extends Sabre_HTTP_DigestAuth{
	/**
	 * Fix user name (Windows XP client issues)
	 *
	 */
	function fixUsername(){
		if(!$this->digestParts['username']) return;

		// Replace \\ by \ in user name in order to fix a windows XP issue (domain name is prepended to user name with a double \ (escaped \ ?), but digest is based on a single \)
		$this->digestParts['username']=str_replace('\\\\','\\',$this->digestParts['username']);
	}

	/**
	 * Return username with removed domain name (if appended by windows)
	 *
	 */
	function getDomainLessUsername(){
		return (strpos($this->digestParts['username'],'\\'))?substr($this->digestParts['username'],strpos($this->digestParts['username'],'\\')+1):$this->digestParts['username'];
	}
}

/**
 * @desc Authenticate remote client and determine which resource is being accessed
 *
 */
function webDAVAuthenticate(){
	if(!cfConstant('WEBDAV_ENABLED')) die("WebDAV access not enabled\n");
	if(cfIPBanned()) die("Authentication required\n");

	$auth = new ResourceDigestAuth();
	$auth->init();
	$auth->fixUsername();

	$r=new Sabre_HTTP_Request();
	$passedToken=substr($r->getUri(),5);
	if(strpos($passedToken,'/')) $passedToken=substr($passedToken,0,strpos($passedToken,'/'));

	// TEST
	if(0 && $passedToken=='fre') foreach (WEnv::resConfigs() as $resId=>$resConfig){
		// All OK, proceed (set stuff so cfRGetVar works)
		if(cfCmpLeft(strtolower($resConfig->getVar('directResourceAccessToken')),$passedToken)) return $resConfig;
	}


	// Browse all resources and check if passed user & password match with this resource's
	foreach (WEnv::resConfigs() as $resId=>$resConfig){
		if(0) $resConfig=new WResConfig(); // Zend IDE WA

		// Check if webDAV is enabled for this resource
		if(!$resConfig->getVar('webDAVEnabled')) continue;

		// Retrieve username and password
		$username=$resId;
		$password=($resConfig->getVar('directResourceAccessPasswordHas'))?cfStoredPasswordDecrypt($resConfig->getVar('directResourceAccessPassword')):'';
		// Windows WA: user name may be prepended by remote domain name plus \\
		if(strpos($auth->getUsername(),'\\\\')) {
			$username=substr($auth->getUsername(),0,strpos($auth->getUsername(),'\\\\')+2).$username;
			//cfDbg('modified user: '.$auth->getUsername().' => compare to '.$username);
		}

		// Check if user name is resource id and password is correct
		if ($auth->getDomainLessUsername() == $username && $auth->validatePassword($password)) {
			// Ensure it matches with passed directResourceAccessToken (if url actually starts
			$r=new Sabre_HTTP_Request();

			if(cfCmpLeft($r->getUri(),'/dav/')){
				$passedToken=substr($r->getUri(),5);
				if(strpos($passedToken,'/')) $passedToken=substr($passedToken,0,strpos($passedToken,'/'));

				// All OK, proceed (set stuff so cfRGetVar works)
				if(cfCmpLeft(strtolower($resConfig->getVar('directResourceAccessToken')),$passedToken)) {
//cfDbg('Authentication ok for '.$username,'');
					return $resConfig;
				}
			}
			// All OK, proceed (set stuff so cfRGetVar works)
			else {
cfDbg('Authentication ok for '.$username.' on '.$_SERVER['REQUEST_URI'],'');
				return $resConfig;
			}
		}
	}

	// Limit velocity
	if(0){
		define('API_LOADED',true);
		require(cfAppDocRoot().'/login.php');
		setWrongPasswordTyped();
	}

	// Exit;
    $auth->requireLogin();
    die("Authentication required\n");
}

/**
 * @desc Init stuff that should have been stored in session
 *
 * @param $resConfig
 */
function webDAVInitResource($resConfig){
	cfRCreateFakeResource($resConfig->getVar(),$resConfig->getVar('id'));

	// LAN restricted access: check
	if(cfRGetVar('directResourceAccessLanOnly') && !cfIsOnLAN()){
		throw new Sabre_DAV_Exception_NotFound('Access denied');
		die();
	}


	// Set base uri
	if(cfGGetVar('directLinkLength')>2)
		cfRSetVar('webDAVbaseUri','/dav/'.strtolower(substr($resConfig->getVar('directResourceAccessToken'),0,cfGGetVar('directLinkLength'))));
	else
		cfRSetVar('webDAVbaseUri','/dav/'.$resConfig->getVar('directResourceAccessToken'));

	// Init explorer stuff
	efInit();

	// Set file type filter (audio / image / video)
	switch(strtolower($resConfig->getVar('subType'))){
		case 'photo1':
		case 'photo2':
		case 'photo3':
			cfRSetVar('fileTypeFilter',array('image'));
			if(cfRGetVar('uploadAllowed')) cfRSetVar('writeAllowed',true); // Allow writting if upload is allowed

			break;
		case 'music1':
		case 'music2':
		case 'music3':
		case 'musicWinamp':
			cfRSetVar('fileTypeFilter',array('audio'));
			if(cfRGetVar('uploadAllowed')) cfRSetVar('writeAllowed',true); // Allow writting if upload is allowed
			break;
		case 'video':
			cfRSetVar('fileTypeFilter',array('video'));
			if(cfRGetVar('uploadAllowed')) cfRSetVar('writeAllowed',true); // Allow writting if upload is allowed
			break;
		case 'multimedia':
			cfRSetVar('fileTypeFilter',array('video','audio','image'));
			if(cfRGetVar('uploadAllowed')) cfRSetVar('writeAllowed'); // Allow writting if upload is allowed
			break;
		default:
	}
}

/**
 * Test function
 *
 */
function webDAVRedirectTest(){
/*
if($_SERVER['HTTP_HOST']=='localhost.'){
	$url='http://127.0.0.1'.$_SERVER['REQUEST_URI'];
//cfDbg('redir: '.$url);
	//header("HTTP/1.1 302 FOUND");
	header("HTTP/1.1 301 Moved Permanently");
	header("Location: ".$url);
	header("Content-Length: 0");
	header("translate: f");
	header("Depth: 0");
?>
<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html><head>
<title>302 Found</title>
</head>
<body>
<h1>Found</h1>
<p>The document has moved <a href="<?php echo $url;?>">here</a>.</p>
</body>
</html>
<?php
	die();
}
*/
}

/**
 * tmp debug functions
 *
 */
function webDAVDebugRequest(){
	cfDbg($_SERVER['REQUEST_URI']);
	return;
	cfDbg(getallheaders());
	cfDbg($_SERVER['REQUEST_URI']);
	cfDbg($_GET);
	cfDbg($_POST);
	cfDbg(file_get_contents('php://input'));
	cfDbg($HTTP_RAW_POST_DATA);
}

// Authenticate remote client and determine which resource is being accessed
$resConfig=webDAVAuthenticate();

//Init stuff that should have been stored in session
webDAVInitResource($resConfig);

// Top level directory
$topLevelDir = new ResourceDirectory('*resourceBasePath*');

// The object tree needs in turn to be passed to the server class
$server = new Sabre_DAV_Server($topLevelDir);

// Set base URI (theorically /dav/xxx, but in some case, requests are moved to doc root
if(cfCmpLeft($_SERVER['REQUEST_URI'],cfRGetVar('webDAVbaseUri'))) $_ENV['baseURI']=cfRGetVar('webDAVbaseUri');
else if(cfCmpLeft($_SERVER['REQUEST_URI'],'/dav')) $_ENV['baseURI']='/dav';
else $_ENV['baseURI']='/';
$server->setBaseUri($_ENV['baseURI']);

// Support for LOCK and UNLOCK
Sabre_DAV_Property_LockDiscovery::$hideLockRoot = true; // MS Office compatibility
$lockBackend = new Sabre_DAV_Locks_Backend_File(cfAppTempDir() . '/locksdb');
$lockPlugin = new Sabre_DAV_Locks_Plugin($lockBackend);
$server->addPlugin($lockPlugin);

// Support for html frontend
$browser = new Sabre_DAV_Browser_Plugin();
$server->addPlugin($browser);

// Automatically guess (some) contenttypes, based on extesion
$server->addPlugin(new Sabre_DAV_Browser_GuessContentType());

// Temporary file filter
$tempFF = new Sabre_DAV_TemporaryFileFilterPlugin(cfAppTempDir());
$server->addPlugin($tempFF);

// And off we go!
$server->exec();
exit;