/*
Node structure: (created by treeEmptyNode function)
<div id="treeName_path" [name="treeName"] class="treeFolded|treeUnfolded|treeEmpty">
	<img src="/gfx/[plus|minus|vs].gif" onclick="" class="treeHandle">
	<a href="javascript:treeSelect(this.parentNode.parentNode)" [class="selected"]>
		<img src="/gfx/fi/folder.png">
		<span>Folder name</span>
	</a>
	[<div name="sub" class="subTree">...</div>]
</div>

JS Tree structure:
{"folder basename":{
		(optional) "ico": icon's name or "" for standard folder icon,
		"sub": subfolders tree, false if not processed, {} if no subfolders
		},
...
}
*/

var treeFoldAnimSteps=10;

var treeViews={}; // Array of all treeViews
var treeSelectedNode={};
var treeTmpSelectedNode=0; // while updating node and subnodes, detect node set as "selected" in JS array and store for later setSelected
var treeTmpBottomNode=0; // while updating node and subnodes, detect node set as "bottom" in JS array and store for later unfold parent
var treeProcessing=false;
var blockIfTreeProcessing=false;
var TVDebug=0,TVDebugCount=0;
var IE6SyncWA=0; // IE 6 synchronous form submition workaround

/**
 * treeView item
 */
function treeView(treeName, tree){
	this.name=treeName;
	this.tree=tree;
	this.selectedNode=false;
	this.draggableNodes=0;
	this.receiverOnlyNodes=0;
	this.selectFunction=0;
	this.dblClickFunction=0;
	this.noATag=0;
	this.initialized=0;
	this.selectOnUpdate=0;
	this.offsetHeight=0;
	this.outerDiv;
	this.div;
	this.sync='auto';
}
// Generate treeView from tree JS assoc. array
treeView.prototype.init=function (){
	var treeBaseNode=treeEmptyNode(this.name,this.tree['path'],1); // Tree DOM base node
	treeBaseNode.setAttribute('name',this.name)
	var baseNode=new treeNode(treeBaseNode);

	baseNode.update(this.tree);
	this.outerDiv=dgi(this.name+'_outer');
	this.div=baseNode.node;

	treeBaseNode.setAttribute('class','treeView');
	treeBaseNode.setAttribute('className','treeView');
	dgi(this.name).appendChild(treeBaseNode);
	if(treeTmpSelectedNode) treeTmpSelectedNode.setSelected();

	treeTmpSelectedNode=0;treeTmpBottomNode=0;
	this.initialized=1;
}
// Destroy tree and regenerate from given tree
treeView.prototype.reset=function (tree){
	this.tree=tree;
	var treeBaseNode=new treeNode(this.name+'_'+this.tree['path']);
	treeBaseNode.node.parentNode.removeChild(treeBaseNode.node);
	this.init();
}

// Return tree's first node
treeView.prototype.firstChild=function (){return this.baseNode().firstChild()}

// Return tree's base node
treeView.prototype.baseNode=function (){return new treeNode(this.div)}

/**
 * Update treeView
 * @param tree: JS Tree structure tree: top updated node information
 * @param setSelected: boolean: true to select node tagged with "selected" value
 */
treeView.prototype.update=function (tree, setSelected){
	if(TVDebug && !dgi(this.name+'_'+tree['path'])) alert('treeView update error : inexisting id '+this.name+'_'+tree['path']);
	var node=new treeNode(this.name+'_'+tree['path']);

	if(TVDebug) alert('UPDATE --- \n'+ ((node)?'\n':'NULL node !\n') + treeToString(tree,0,tree['path']));

	// Update node and subnodes
	node.update(tree);

	if(treeTmpBottomNode && treeTmpBottomNode.parentNode()) treeTmpBottomNode.parentNode().unfold(); // Unfold all nodes to bottom node
	if(tree['selected']==undefined) node.unfold(); // If top updated node is not selected, unfold
	if(setSelected && treeTmpSelectedNode) treeTmpSelectedNode.setSelected();
	if(this.selectOnUpdate) this.selectOnUpdate.select(); this.selectOnUpdate=0;
	treeTmpSelectedNode=0;treeTmpBottomNode=0;
}
// Selected node
treeView.prototype.setSelectedNode=function (node){this.selectedNode=node}
treeView.prototype.selectedNode=function (){return this.selectedNode}
treeView.prototype.setSelectOnUpdate=function (treeNode){this.selectOnUpdate=treeNode}

// Set select function
treeView.prototype.setSelectFunction=function (func){
	this.selectFunction=func
	if(func==treeSyncSelectPath) IE6SyncWA=1;
}
// Set double click function
treeView.prototype.setDblClickFunction=function (func){this.dblClickFunction=func}

// Set right click function
treeView.prototype.setRightClickFunction=function (func){this.rightClickFunction=func}

// Set nodes toggle fold function
treeView.prototype.setToggleFoldFunction=function (func){this.toggleFoldFunction=func}

// Set nodes toggle fold function
treeView.prototype.setNoATag=function (func){this.noATag=1}

// Update inner frame
treeView.prototype.adjustInnerDiv=function (){alignBottom(dgi(this.name),dgi(this.name + '_outer'),2);}

// Find a node in tree
treeView.prototype.getNodeById=function (id){return this.firstChild().recurseFindNode(id,'id')}



/**
 * treeNode item
 * @param node: DOM node or id
 */
function treeNode(node,dbg){
	if(!node) alert("Empty treeNode");
	else if(typeof(node)=='string' && !dgi(node)) alert('inexisting node '+node);

	if(typeof(node)=='string') node=dgi(node);
	this.node=node;

	this.id=node.id;
	if(dbg) alert(this.id)
}

/**
 * Check if a node is a treeNode item
 * @param node: DOM node or id
 * @return treeNode or false
 */
function treeNodeChecked(node){
	if(typeof(node)=='string') node=dgi(node);
	if(!node || !node.id) return false;
	for(elem in treeViews) if(node.id.substr(0,elem.length+1)==elem+'_') return new treeNode(node);
	return false;
}
// Get treeNode's treeView
treeNode.prototype.treeView=function (){return treeViews[this.id.substr(0,this.id.indexOf('_'))]}

// Return true if treeNode is treeView's root node
treeNode.prototype.isRoot=function (){if(this.node && nameAttribute(this.node)) return 1; else return 0}

// Update node with tree
treeNode.prototype.update=function (tree, level){
	var subNode=false,DOMDir=false,newSubNode=false,nextNodeFound,sn,subPath,nbSub=false,tmpDOMDir=false;

	if(level===undefined) {
		var isTopNode=1;
		level=0;
	}
	var path=tree['path'];

	if(TVDebug) alert('START UPDATE OF '+path);
	// basename(path)
	var name=path.substr(path.lastIndexOf('/')+1);

	// External id
	if(tree['id']) this.setExtId(tree['id'])

	// Icon
	if(tree['ico'])	this.setIconSrc((tree['ico'].substr(0,1)=='/')?tree['ico']:'/gfx/fi/'+tree['ico']);
	else if(tree['ico']==false) this.hideIcon();

	// Label
	if(tree['label']==undefined) label=name; else label=tree['label'];

	// Workaround: don't update label of drives
	if(level!==0 || label.length!=2 || label.substr(1,1)!=':'){
		this.setLabel(label);
	}
	if(this.isRoot()) {
		if(!label) this.hideNode(1);
		this.label().setAttribute('class','treeRootLabel');
		this.label().setAttribute('className','treeRootLabel');
	}

	// Is selected
	if(tree['selected']!=undefined && !treeTmpSelectedNode) treeTmpSelectedNode=this;

	// Is faded
	if(tree['faded']) this.setFaded();

	// Has a tooldip
	if(tree['tooltip']) this.setTooltip(tree['tooltip']);

	// Sub directories
	if(tree['sub']===false) nbSub=false; else nbSub=count(tree['sub']);

	var subDirs=this.childrenNodesIds(); // name=>path array
	if(!nbSub && subDirs!==null) nbSub=Math.max(count(subDirs),nbSub);

	if(subDirs===null && (nbSub || path=='computerRoot' || path=='*resourceBasePath*')){
		// Create "sub" node
		subNode=D.createElement('div');
		subNode.setAttribute('name','sub');
		subNode.setAttribute('class','subTree tn'+level);
		subNode.setAttribute('className','subTree tn'+level);

		if(this.treeView().initialized) this.setMainClass('treeFolded');
		newSubNode=1;
	}
	else subNode=this.subNodeDiv();

	if(this.isRoot()) subNode.style.marginLeft='-12px';

	var nodeProcessed;

	if(TVDebug) alert("COMPARE  \n" + arrayToString(tree['sub']) + " AND\n" + arrayToString(subDirs));

	// Update existing subdirs
	for(arrayDir in tree['sub']){
		for(DOMDir in subDirs) {
			if(DOMDir==arrayDir){
				if(TVDebug && level==0) alert('DELETE ' + arrayDir + ' from ' + path);
				if(count(tree['sub'][arrayDir]['sub'])) {
					if(TVDebug) alert('go to update subnode ' + arrayDir + ' from ' + path);
					if(path=='computerRoot') tree['sub'][arrayDir]['path']=arrayDir; else tree['sub'][arrayDir]['path']=path+'/'+arrayDir;
					//alert('update '+treeToString(tree['sub'][arrayDir],1,arrayDir));
					tmpDOMDir=arrayDir;
					this.childNode(arrayDir).update(tree['sub'][arrayDir],level+1);
					arrayDir=tmpDOMDir;
				}
				else{
					if(TVDebug) alert('NO update of subnode ' + arrayDir + ' from ' + path);
				}
				delete(tree['sub'][arrayDir]);
				delete(subDirs[arrayDir]);
				if(TVDebug) alert('DELETE ' + arrayDir + ' from ' + path);
				break;
			}
		}
	}

	if(TVDebug) alert('Pre-remove subDirs ' + arrayToString(subDirs) + ' from ' + path);
	// Remove DOM directories that doesn't exist anymore
	for(DOMDir in subDirs) {
		if(TVDebug) alert('REMOVE subnode ' + subDirs[DOMDir] + ' from ' + path);
		subNode.removeChild(dgi(subDirs[DOMDir]));
	}

	// Insert new subdirs
	var nb=0,ts,opt,
		receiverOnlyNodes=this.treeView().receiverOnlyNodes,
		treeName=this.treeView().name,
		hasDblClick=((this.treeView().dblClickFunction)?1:0),
		hasRightClick=((this.treeView().rightClickFunction)?1:0),
		hasNoATag=((this.treeView().noATag)?1:0),
		scrollContainer=dgi(treeName+'_outer');

	for(arrayDir in tree['sub']){
		ts=tree['sub'][arrayDir];
		if(nb++==0) subDirs=this.childrenNodesIds();
		nextNodeFound=false;
		for(DOMDir in subDirs) {
			// Find 1st DOM node > to arrayDir
			if(DOMDir.toLowerCase()>arrayDir.toLowerCase()){
				nextNodeFound=DOMDir;
				break;
			}
		}
		if(path=='computerRoot') subPath=arrayDir; else subPath=path+'/'+arrayDir;

		// New node innerHTML
		opt={}
		if(ts.disabled) opt.disabled=1;
		if(ts.faded) opt.faded=1;
		if(ts.noClick) opt.noClick=1;
		if(ts.rcf) opt.rcf=ts.rcf;
		if(ts.action) opt.action=ts.action;
		if(ts.outerLabel) opt.outerLabel=ts.outerLabel;
		if(hasNoATag) opt.noATag=1;
		opt.level=level;
		sn=treeEmptyNode(treeName,subPath,hasDblClick,hasRightClick,ts.tag,opt);

		// Add to receiver-only objects
		if(receiverOnlyNodes) {
			drec.addElt(sn);
			drec.addTimeoutFunc(sn,'new treeNode("'+sn.id+'").unfold()',treeOpenTimeout);
			drec.setScrollContainer(sn,scrollContainer);
		}

		ts['path']=subPath;
		tmpNode=new treeNode(sn);
		if(TVDebug) alert('insert ' + subPath + ' into ' + path);
		tmpNode.update(ts,level+1);
		//insert new node
		if(nextNodeFound) subNode.insertBefore(sn,dgi(subDirs[DOMDir])); else subNode.appendChild(sn);

		//if(this.treeView().draggableNodes) tmpNode.setDraggable();
	}

	// Folded / Unfolded / Empty
	if(!this.isRoot()){
		if(nbSub===false) {
			this.setFolded();
			if(TVDebug) alert('update ' + path + ' folded');
		}
		else {
			if(nbSub) {
				if(tree['unfolded']){
					this.setUnfolded();
					if(TVDebug) alert('update ' + path + ' unfolded');
				}
			}
			else{
				this.setEmpty();
				if(TVDebug) alert('update ' + path + ' empty');
			}
		}
		if(tree['bottom']) treeTmpBottomNode=this;
	}
	else{
		this.setEmpty();
		if(TVDebug) alert('update ' + path + ' empty2');
	}

	if(newSubNode) {
		this.node.appendChild(subNode);
		// If updating empty node of an already-created treeView, anim
		//if(this.treeView().initialized) this.unfold();
	}
	treeProcessing=false;
	if(TVDebug) alert('END UPDATE OF '+path);
}

treeNode.prototype.parentNode=function (){
	if(this.isRoot() || !this.node.parentNode) return false;
	return new treeNode(this.node.parentNode.parentNode);
}
treeNode.prototype.childNode=function (path){
	return new treeNode(dgi(this.id + '/' + path));
}
treeNode.prototype.firstChild=function (){
	var sn=this.subNodeDiv();
	if(!sn) return false;
	sn=sn.childNodes;
	if(!sn || !sn[0]) return false;
	return new treeNode(sn[0].id);
}
treeNode.prototype.nextSibling=function (){
	var bn=this.parentNode().subNodeDiv().childNodes;
	if(!bn) return false;
	var f=0;
	for(var n in bn) {
		if(f && bn[n]['id']) return new treeNode(bn[n]['id']);
		if(bn[n]['id']==this.id) f=1;
	}
}
treeNode.prototype.previousSibling=function (){
	var i,bn=this.parentNode().subNodeDiv().childNodes;
	if(!bn) return false;
	for(i=1;i<bn.length;i++) {
		if(i>1 && bn[i]['id']==this.id) return new treeNode(bn[i-1]['id']);
	}
}

// Recursively browse nodes to find a node with matching criterias
treeNode.prototype.recurseFindNode=function (value,key){
	 if(key=='id'&&(this.id.substr(this.id.lastIndexOf('/')+1))==value) return this;

	 // Look in sub nodes
	 if(this.firstChild()){
	 	res=this.firstChild().recurseFindNode(value,key);
	 	if(res) return res;
	 }
	 // Go to next node
	 if(this.nextSibling()) return this.nextSibling().recurseFindNode(value,key);
	 return false;
}

// Get handle, icon, label and subnodes nodes
treeNode.prototype.handle=function () {return this.node.firstChild}
treeNode.prototype.icon=function (){if(!this.node.firstChild) return false;return this.node.firstChild.nextSibling.firstChild}
treeNode.prototype.label=function (){return this.node.firstChild.nextSibling.firstChild.nextSibling} // Return label node
treeNode.prototype.mainNode=function (){return this.node.firstChild.nextSibling} // Return icon + label node
treeNode.prototype.subNodeDiv=function (){
	var n=this.node.lastChild;
	if(n.tagName!="DIV"||n.getAttribute("name")!="sub") return null;
	return n
}

treeNode.prototype.setHandleSrc=function (src){this.handle().setAttribute('src',src)}
treeNode.prototype.getLabel=function (){return this.label().innerHTML} // Return label
treeNode.prototype.setLabel=function (label){this.label().innerHTML=label/*.replace(/&/g,'&amp;')*/}

treeNode.prototype.tag=function (){return this.node.getAttribute("alt")}
treeNode.prototype.setTag=function (val){this.node.setAttribute("alt",val)}

treeNode.prototype.setFaded=function (){setAlpha(this.node,0.5)}
treeNode.prototype.setTooltip=function (txt){this.node.onmouseover=function(){tooltip(this,txt,true)}}

treeNode.prototype.setIconSrc=function(src) {var n;if(!(n=this.icon())) return; if(src) n.setAttribute('src',src); else n.style.display='none'}
treeNode.prototype.hideIcon=function() {this.setIconSrc(0)}
treeNode.prototype.setExtId=function(i) {this.node.setAttribute('extId',i)}
treeNode.prototype.getExtId=function(i) {return this.node.getAttribute('extId')}
treeNode.prototype.treeName=function() {return this.id.substr(0,this.id.indexOf('_'))}
treeNode.prototype.getPath=function() {return this.id.substr(this.id.indexOf('_')+1)}
treeNode.prototype.hideNode=function (showSubNodes) {
	this.handle().style.display='none';this.mainNode().style.display='none';
	if(!showSubNodes && this.subNodeDiv()) this.subNodeDiv().style.display='none';
}

treeNode.prototype.showNode=function () {
	var sND;
	this.handle().style.display='';
	this.mainNode().style.display='';
	if(sND=this.subNodeDiv()) {sND.style.display='';$(sND).e.setAlpha(1)}
}
treeNode.prototype.setMainClass=function (className) {
	$(this.node).e.className().remove('treeFolded');
	$(this.node).e.className().remove('treeUnfolded');
	$(this.node).e.className().remove('treeEmpty');
	$(this.node).e.className().add(className);
}
treeNode.prototype.setLoading=function () {this.setHandleSrc(treeIconLoading);this.handle().onmouseover=function (){};this.handle().onmouseout=function (){}}
treeNode.prototype.isSelected=function (){return this.treeView().selectedNode.id==this.id};
treeNode.prototype.isFolded=function (){return this.node.className.indexOf('treeFolded')!=-1};
treeNode.prototype.setFolded=function () {
	this.setMainClass('treeFolded');
	this.setHandleSrc(treeIconPlus);
	if(treeIconPlusOver) {
		this.handle().onmouseover=function (){this.src=treeIconPlusOver};
		this.handle().onmouseout=function (){this.src=treeIconPlus};
	}
	if(this.treeView().initialized && this.treeView().toggleFoldFunction) this.treeView().toggleFoldFunction(this) // If tree has a ontogglefold callback, call it
}
treeNode.prototype.focusView=function (){
	var t=this.node.offsetTop,od=this.treeView().outerDiv;
	if(t<od.scrollTop || t>od.scrollTop+od.offsetHeight) od.scrollTop=t-(od.offsetHeight/8)
}
treeNode.prototype.setUnfolded=function (){
	var sND;
	if(this.isRoot()) return;
	if($(this.node).e.className().has('treeUnfolded')) return;
	iconSrc=this.icon().src.toLowerCase();
	iconSrc=iconSrc.substr(iconSrc.length-treeIconFolder.length);
	if(iconSrc==treeIconFolder.toLowerCase()) this.setIconSrc(treeIconFolderO);
	this.setMainClass('treeUnfolded');
	// Make sure subNodeDiv is shown
	if(sND=this.subNodeDiv()) {
		with(sND.style){display='';overflow='';height=''}
		$(sND).e.setAlpha(1);
	}
	this.setHandleSrc(treeIconMinus);
	if(treeIconMinusOver) {
		this.handle().onmouseover=function (){this.src=treeIconMinusOver};
		this.handle().onmouseout=function (){this.src=treeIconMinus};
	}

	if(this.treeView().initialized && this.treeView().toggleFoldFunction) this.treeView().toggleFoldFunction(this) // If tree has a ontogglefold callback, call it

}
treeNode.prototype.toggleFold=function (){if(this.isFolded()) this.setUnfolded(); else this.setFolded()};
treeNode.prototype.toggleFoldAnim=function (){if(this.isFolded())  this.unfold(); else this.fold()};
treeNode.prototype.setEmpty=function (){
	if(this.isRoot()){
		this.handle().style.display='none';
		return;
	}
	this.setMainClass('treeEmpty');
	this.setHandleSrc('/gfx/vl.gif');
	this.handle().onmouseover=null;
	this.handle().onmouseout=null;
}

// Return 1st level children nodes assoc. array name => id
treeNode.prototype.childrenNodesIds=function (){
	var subNode=this.subNodeDiv();
	if(!subNode) return null;

	var childNodes=subNode.childNodes;
	var children={}; var id;
	for(i=0; i<childNodes.length; i++){
		id=childNodes[i].getAttribute('id');
		if(id.lastIndexOf('/')==-1) children[id.substr(id.indexOf('_')+1)]=id; else	children[id.substr(id.lastIndexOf('/')+1)]=id;
	}
	return children;
}
treeNode.prototype.childrenNodes=function (){
	if(!this.subNodeDiv()) return null;
	return this.subNodeDiv().childNodes;
}
// Unfold node
treeNode.prototype.unfold=function (){
	// If node is unfolded, don't unfold
	if(this.isFolded()){
		// If node has never been unfolded, send content request to server
		children=this.childrenNodesIds();
		if(children===null) {
			if(treeProcessing && blockIfTreeProcessing) return;
			treeProcessing=true;
			this.setLoading();
			var prevD1=D.comForm.data1.value;
			D.comForm.data1.value=this.getPath();
			fAS('',"treeUnfoldTo",this.treeView().name,this.id,true);
			D.comForm.data1.value=prevD1;
		}
		// Else, unfold
		else{
			this.setUnfolded();
			var s=this.subNodeDiv().style;
			s.display=''; s.overflow='hidden'; s.height='0px';

			var h=(actualOffsetTop(this.subNodeDiv().lastChild)-actualOffsetTop(this.subNodeDiv().firstChild)+this.subNodeDiv().firstChild.offsetHeight);
			treeNodeFoldAnim(this.id,1,0,h);
		}
	}

	// Unfold parents chain
	if(this.parentNode() && this.parentNode().isFolded()) this.parentNode().unfold();
}
// Fold node
treeNode.prototype.fold=function (){
	if(!this.subNodeDiv()) return;
	var h=this.subNodeDiv().offsetHeight;
	this.subNodeDiv().style.overflow='hidden';
	treeNodeFoldAnim(this.id,-1,undefined,h)
}
/**
 * Fold/unfold
 * id: id of node
 * dir: Fold (dir=-1)/ unfold (dir=1)
 * baseH: unfolded node height
 */
function treeNodeFoldAnim(id,dir,step,baseH){
	var tn=new treeNode(id);
	// Unfold anim
	if(dir==1) wl.anim({'item':tn.subNodeDiv(),'duration':Math.min(700,3*baseH),'extraParams':{'baseH':baseH,'treeNode':tn},
		'function':function(p,anim){
			var f=wl.animFunctionUnfold2(p,anim);
			return {'height':Math.max(1,f*anim.extraParams.baseH),'opacity':(phpBrowser=='ie')?1:Math.min(1.2*f,1)}
		},
		'callbackComplete':function(anim){
			var tn=anim.extraParams.treeNode;
			anim.item.style.height=''; // there seems to be a bug with IE with height not set
			anim.item.className="subTree tn1"
			tn.setUnfolded();
			if(tn.treeView().receiverOnlyNodes) drec.computeRect();
		}
	});
	// Fold anim
	else wl.anim({'item':tn.subNodeDiv(),'duration':Math.min(700,3*baseH),'extraParams':{'baseH':baseH,'treeNode':tn},
		'function':function(p,anim){
			var f=wl.animFunctionFold2(p,anim);
			return {'height':Math.max(1,f*anim.extraParams.baseH),'opacity':(phpBrowser=='ie')?1:Math.min(1.5*f,1)}
		},
		'callbackComplete':function(anim){
			var tn=anim.extraParams.treeNode;
			tn.setFolded();
			if(tn.treeView().receiverOnlyNodes) drec.computeRect();
		}
	});
}

// Default node selection function (historicaly used by treeViewFolder)
treeNode.prototype.select=function (hasAction){
	if(!treeUnfoldOnSelect && this.isSelected() && !this.isRoot()){
		if(this.isFolded()) this.unfold(); else this.fold();
	}
	// Dbl click: ask to select node after update, so it gets unfolded
	if(treeProcessing){if(!blockIfTreeProcessing) this.treeView().setSelectOnUpdate(this);	return;}

	if(treeUnfoldOnSelect && !this.isFolded() && !this.isRoot() && this.subNodeDiv()) {this.fold();return}
	this.setSelected();
	var s=this.treeView().sync;
	if(hasAction) return;
	treeProcessing=1;
	goDirect(this.id.substr(this.id.indexOf('_')+1),(s=='auto')?false:s)
	//fAS(this.id.substr(this.id.indexOf('_')+1),"goDirect","","",(s=='auto')?false:s);
	if(treeUnfoldOnSelect && this.isFolded()) this.unfold();
}

// Unselect previously selected node and set node as selected
treeNode.prototype.setSelected=function (){

	// Unselect previously selected nodes
	var iconSrc,node2=this.treeView().selectedNode
	while(node2){
		$(node2.mainNode()).e.className().remove('selected');
		iconSrc=node2.icon().src.toLowerCase();
		iconSrc=iconSrc.substr(iconSrc.length-treeIconFolderO.length);
		if(iconSrc==treeIconFolderO.toLowerCase()) node2.setIconSrc(treeIconFolder);
		node2=node2.parentNode();
	}
	this.treeView().setSelectedNode(this);

	$(this.mainNode()).e.className().add('selected');

	// Unfold all parents
	var node=this;
	while(1){
		if(!(node=node.parentNode())) break;
		try{node.setUnfolded()} catch(e){}
	}
}

// Set node handle to error
treeNode.prototype.setError=function (){this.setHandleSrc(treeIconError);}

// Set draggable node
treeNode.prototype.setDraggable=function (){
	dragItem=this.id+NO_DRAG+CURSOR_HAND+RECEIVER;
	if(dd.ie) {try{ADD_DHTML(dragItem);} catch(err){};}	else ADD_DHTML(dragItem);
}


// Remove node from tree
treeNode.prototype.remove=function (){removeNode(this.node)}


/**
 * Common treeView functions
 */
function treeToggleFold(nodeDiv){
	var node=new treeNode(nodeDiv);
	if(node.isFolded()) node.unfold(); else node.fold();
}

// Synchronous tree frame folding request
function treeToggleFrame(){fAS(true,'toggleTreeFrame',0,false,'sync');}

// Select node
// hasAction: true if a js action is appened to treeSelect (so select function should just set node as selected, but perform no action)
function treeSelect(nodeDiv,hasAction){
	var node=new treeNode(nodeDiv);
	if(node.treeView().selectFunction) node.treeView().selectFunction(node);// If treeView has a specific select function call it
	else node.select(hasAction); // else use default async select function
}
// Double click
function treeDblClickF(nodeDiv){
	var node=new treeNode(nodeDiv);
	if(node.treeView().dblClickFunction) node.treeView().dblClickFunction(node);// If treeView has a specific select function call it
}

// Right click
function treeRightClickF(e,nodeDiv){
	var node=new treeNode(nodeDiv);
	node.treeView().rightClickFunction(node);
	if(e.stopPropagation) e.stopPropagation(); else	e.cancelBubble=true;
	return false;
}

// Synchronous path treeView select node
function treeSyncSelectPath(node){
	goDirect(this.id.substr(this.id.indexOf('_')+1),'sync');
	//fAS(node.id.substr(node.id.indexOf('_')+1),"goDirect",undefined,undefined,"sync");
}

// Empty DOM tree node
function treeEmptyNode(treeName, path, hasDblClick, hasRightClick,tag,opt){
	if(!opt) opt={}
	var en=(!opt.disabled),noClick=opt.noClick,n=D.createElement('div');// en is true if node is enabled, false if disabled
	n.setAttribute('id',treeName+'_'+path);
	n.setAttribute('class','treeFolded dl'+opt.level);
	n.setAttribute('className','treeFolded dl'+opt.level);
	if(tag) n.setAttribute('alt',tag);

 	ih ='<img src="'+treeIconPlus+'" class="treeHandle"'+ ((en)?' onclick="treeToggleFold(this.parentNode)"':'');
	if(en && treeIconPlusOver) ih+=' onmouseover="this.src=\''+treeIconPlusOver+'\'" onmouseout="this.src=\''+treeIconPlus+'\'"';

	ih+='>';
	if(noClick||!en) ih+='<i '+((!en)?'class="treeFaded"':'');
	else{
		if(opt.noATag) ih+='<fieldset onclick="treeSelect(dgi(\''+treeName+'_'+path.replace(new RegExp("'", 'g'),"\\'")/*.replace(/&amp/g,"&'+'amp")*/+'\')' +((opt['action'])?',1);'+opt['action']:')')+'"';
		ih+='<a href="javascript:treeSelect(dgi(\''+treeName+'_'+path.replace(new RegExp("'", 'g'),"\\'")/*.replace(/&amp/g,"&'+'amp")*/+'\')' +((opt['action'])?',1);'+opt['action']:')')+'"';
		if(hasDblClick) ih+='ondblclick="treeDblClickF(this.parentNode)"';
		if(opt.rcf) ih+='oncontextmenu="eval(this.href.replace(/javascript:/,\'\'));return '+opt.rcf+'(this.parentNode.id)"';
		else if(hasRightClick) ih+='oncontextmenu="return treeRightClickF(event,this.parentNode)"';
		if(opt.faded) ih+=' class="treeFaded"';
	}
	ih+= '>';
	ih+='<img src="'+treeIconFolder+'" class="treeIcon" '+((en&&!opt['faded'])?'':' style="filter:Alpha(opacity=20, style=0);opacity:0.2"')+'>';
	ih+='<span class="l'+opt.level+'"></span>\n';
	if(noClick||!en) ih+='</i>';
	else if(opt.noATag) ih+='</fieldset>';
	else ih+='</a>';

	if(opt.outerLabel) ih+=opt.outerLabel;
	n.innerHTML=ih;
	return n;
}
function treeMonitorHeight(){
	for(t in treeViews) with(treeViews[t]){

		if(treeViews[t].outerDiv&&Math.abs(offsetHeight-outerDiv.offsetHeight)>3){
			offsetHeight=outerDiv.offsetHeight;
			adjustInnerDiv();
		}
	}
}


// TVDebug
function treeToString(tree, level, name){
	if(level==undefined) level=0;
	if(name==undefined) label='???'; else label=name;
	if(tree['label']!=undefined) label=tree['label'];
	else{
		if(name==undefined) label='???'; else label=name;
	}
	var shift=' ';
	for(var i=0;i<level;i++) shift+="    ";

	output=shift+label+' - ';
	for(elem in tree){
		if(elem!='label' && elem!='sub') output+='#'+elem+':'+tree[elem];
	}
	output+="\n";
	// Display subtree
	if(tree['sub']!==false){
		for(sdir in tree['sub']) {
			output+=treeToString(tree['sub'][sdir],level+1,sdir);
		}
	}

	return output;
}


// Get tag name
function nameAttribute(node){
	if(node.name) return node.name;
	if(!node.hasAttribute || !node.hasAttribute('name')) return undefined; else return node.getAttribute('name');
}

function arrayToString(arr){
	var output="";
	for(e in arr) output += e + " => " + arr[e] + "\n";
	return output;
}

// treeViewFolder dblClick function
function treeFolderDblClickSelect(node){if(node.isFolded()) node.unfold();}

//setInterval('treeMonitorHeight()',100);
