<?php
/**
 * Application and server connection test
 *
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

/**
 * Build ports list
 */
function serverAccessPortsList(){
	$ports=array();

	// Main port
	$ports[]=array(
		'port'=>cfGGetVar('serverPort'),
		'ssl'=>(cfGGetVar('protocol')=='https')?1:0,
		'label'=>cfCaption('configServerPort'),
		'code'=>'main'
	);

	// 1st access test: check only main server port
	if(@$_GET['onlyMainPort']) return $ports;

	// Alternate server port
	if(cfGGetVar('alternateServerPort')) $ports[]=array(
		'port'=>cfGGetVar('alternateServerPort'),
		'ssl'=>(cfGGetVar('alternateServerPortSSL'))?1:0,
		'label'=>cfCaption('alternatePort'),
		'code'=>'alternate'
	);

	// Bittorrent port
	if(cfGGetVar('bittorrentEnabled')) $ports[]=array(
		'port'=>cfGGetVar('bittorrentPort'),
		'ssl'=>0,
		'label'=>cfCaption('bittorrent'),
		'code'=>'bittorrent'
	);

	return $ports;
}


// Initialize script data
require_once('localFunctions.php');
lfInit();

$steps=array(0=>array('icon'=>'ok'),1=>array('icon'=>'v'),2=>array('icon'=>'v'));

if(!isset($_GET['step']) && !isset($_POST['step'])) die('missing argument');
if(isset($_POST['step'])) $step=$_POST['step']; else $step=$_GET['step'];
if(isset($_POST['silent'])) $_GET['silent']=$_POST['silent'];

// APPLICATION OUT ERROR
if($step=='appOutError'){
	$steps[0]['icon']='alert';
	$steps[0]['message']=cfCaption('accessTestAppNok');
	$steps[0]['messageType']='error';

	$steps[1]['message']=cfCaption('accessNotTested');
	$steps[2]['message']=cfCaption('accessNotTested');

	$leftIcon='alertBig';
	$leftMessage=cfCaption('genError');

	$button1=outBt(cfCaption('genRetry'),outIcon('go'),'javascript:wl.UICommand(\'close:restart\')',false,false,false,'up',120);
	$button2=outBt(cfCaption('genAbandon'),outIcon('cancel'),'javascript:wl.UICommand(\'close:cancel\')',false,false,false,'up',120);

	$error=true;
}

// APPLICATION OUT OK
elseif($step=='appOutOk'){
	$steps[0]['icon']='ok16';
	$steps[0]['message']='<b>'.cfCaption('genOK').'</b>';
	$steps[0]['messageType']='ok';

	$steps[1]['icon']='go';
	$steps[1]['message']=cfCaption('accessTesting');
	$steps[1]['messageType']='progress';

	$steps[2]['message']=cfCaption('accessNotTested');

	$leftIcon='loadingBig';
	$leftMessage=cfCaption('accessTesting');
	$nextStep='serverOutTest';

	$button1=outBt(cfCaption('genAbandon'),outIcon('cancel'),'javascript:wl.UICommand(\'close\')',false,false,false,'up',120);
}
elseif($step=='serverOutTest'){
	// Check outgoing server connection by opening a socket and HTTP connection to weezo.net server
	cfAsyncHeader();
	if (cfSocketHTTPRequest(DNS_SITE.'/portCheck.php')){
		echo cfAsyncXMLJSaction('document.location="/local/serverAccess.php?step=serverOutOk&silent='.((@$_GET['silent'])?1:0).'&onlyMainPort='.((int)@$_GET['onlyMainPort']).'";');
	}
	else{
		echo cfAsyncXMLJSaction('document.location="/local/serverAccess.php?step=serverOutError&silent='.((@$_GET['silent'])?1:0).'&onlyMainPort='.((int)@$_GET['onlyMainPort']).'";');
	}
	echo cfAsyncFooter();
	exit;
}

// SERVER OUT OK
elseif($step=='serverOutOk'){
	$steps[0]['icon']='ok16';
	$steps[0]['message']='<b>'.cfCaption('genOK').'</b>';
	$steps[0]['messageType']='ok';

	$steps[1]['icon']='ok16';
	$steps[1]['message']='<b>'.cfCaption('genOK').'</b>';
	$steps[1]['messageType']='ok';

	if(cfGGetVar('serverPort')<80){
		$steps[2]['icon']='unknown';
		$steps[2]['message']=cfCaption('startPortsTested','<b>'.cfGGetVar('serverPort').'</b>');
		$steps[2]['messageType']='error';

		$leftIcon='unknownBig';
		$leftMessage=cfCaption('accessNotTested');

		$button1=outBt(cfCaption('genOK'),outIcon('ok16'),'javascript:wl.UICommand(\'close\')',false,false,false,'up',120);
	}
	else{
		$steps[2]['icon']='go';
		$steps[2]['message']=cfCaption('accessTesting');

		$leftIcon='loadingBig';
		$leftMessage=cfCaption('accessTesting');

		$nextStep='serverInTest';

		$button1=outBt(cfCaption('genAbandon'),outIcon('cancel'),'javascript:wl.UICommand(\'close\')',false,false,false,'up',120);
	}
}

// SERVER OUT ERROR
elseif($step=='serverOutError'){
	$steps[0]['icon']='ok16';
	$steps[0]['message']='<b>'.cfCaption('genOK').'</b>';
	$steps[0]['messageType']='ok';

	$steps[1]['icon']='alert';
	$steps[1]['message']=cfCaption('accessTestHttpdOutNok');
	$steps[1]['messageType']='error';

	$steps[2]['message']=cfCaption('accessNotTested');

	$leftIcon='alertBig';
	$leftMessage=cfCaption('genError');

	$button1=outBt(cfCaption('genRetry'),outIcon('go'),'javascript:reload(\'appOutOk\')',false,false,false,'up',120);
	$button2=outBt(cfCaption('genAbandon'),outIcon('cancel'),'javascript:wl.UICommand(\'close\')',false,false,false,'up',120);

	$error=true;
}

// SERVER IN
elseif($step=='serverInTest'){

	$portsString='';
	foreach (serverAccessPortsList() as $k=>$v) $portsString.='&ports['.$v['port'].']='.$v['ssl'];

	// Send request
	$data=cfSocketHTTPRequest(DNS_SITE.'/portCheck.php?lng=en&v='.cfGGetVar('appVersion').$portsString);
	$portCheckResult=substr($data,strpos($data,'PORTCHECK=')+10);

	cfAsyncHeader();

	// Main port is accessible
	if(strpos($data,'RESULT=OK')!==false) {
		// Check for wrong LAN IP found by weezo.net server (i.e. another weezo apache server on the LAN has answered the request)
		if(strpos($data,'LANIP=')!==false && strpos($data,'LANIP='.cfGGetVar('LANIPComputed'))===false)
			echo cfAsyncXMLJSaction('document.location="/local/serverAccess.php?step=serverInError&silent='.((@$_GET['silent'])?1:0).'&portCheckResult='.$portCheckResult.'&onlyMainPort='.((int)@$_GET['onlyMainPort']).'";');
		// All OK
		else
			echo cfAsyncXMLJSaction('document.location="/local/serverAccess.php?step=serverInOk&silent='.((@$_GET['silent'])?1:0).'&portCheckResult='.$portCheckResult.'&onlyMainPort='.((int)@$_GET['onlyMainPort']).'";');
	}
	// Main port couldn't be accessed
	else echo cfAsyncXMLJSaction('document.location="/local/serverAccess.php?step=serverInError&silent='.((@$_GET['silent'])?1:0).'&portCheckResult='.$portCheckResult.'&onlyMainPort='.((int)@$_GET['onlyMainPort']).'";');

	echo cfAsyncFooter();
	exit;
}

// SERVER IN OK
elseif($step=='serverInOk'){
	$steps[0]['icon']='ok';
	$steps[0]['message']='<b>'.cfCaption('genOK').'</b>';
	$steps[0]['messageType']='ok';

	$steps[1]['icon']='ok';
	$steps[1]['message']='<b>'.cfCaption('genOK').'</b>';
	$steps[1]['messageType']='ok';

	$steps[2]['icon']='ok';
	$steps[2]['message']='<b>'.cfCaption('genOK').'</b>';
	$steps[2]['messageType']='ok';

	$leftIcon='okBig';
	$leftMessage=cfCaption('genOK');

	$button2=outBt(cfCaption('genOK'),outIcon('ok16'),'javascript:wl.UICommand(\'close:ok\')',false,false,false,'up',120);
}

// SERVER IN ERROR
elseif($step=='serverInError'){
	$steps[0]['icon']='ok';
	$steps[0]['message']=cfCaption('genOK');
	$steps[0]['messageType']='ok';

	$steps[1]['icon']='ok';
	$steps[1]['message']=cfCaption('genOK');
	$steps[1]['messageType']='ok';

	$steps[2]['icon']='alert';
	$steps[2]['message']=cfCaption('accessTestHttpdInNok','<b>'.cfGGetVar('serverPort').'</b>');
	$steps[2]['messageType']='error';

	$leftIcon='alertBig';
	$leftMessage=cfCaption('genError');

	$button1=outBt(cfCaption('genRetry'),outIcon('go'),'javascript:reload(\'serverOutOk\')',false,false,false,'up',120);
	$button2=outBt(cfCaption('genAbandon'),outIcon('cancel'),'javascript:wl.UICommand(\'close\')',false,false,false,'up',120);

	$error=true;
}

/**
 * Display page
 */
cfInsertHEAD(true,array('size'=>'650x460','closable'=>'0','title'=>APPLICATION_NAME.' - '.cfCaption('startTestServerAccess')));
echo '<body title="Weezo" style="width:650px;height:460px;overflow:hidden;padding:0;margin:0" oncontextmenu="return false;" '.((isset($nextStep))?'onload="window.setTimeout(\'goSubmit()\',500)" style="cursor:wait;">':'>');
?>
<script type="text/javascript">
function goSubmit(){sendData("silent=<?php echo ((@$_GET['silent'])?1:0);?>&step="+document.forms.nextStep.step.value)}
function reload(nStep){
	W.location='<?php echo $_SERVER['PHP_SELF'];?>?step='+nStep+'&silent=<?php echo ((@$_GET['silent'])?1:0);?>&onlyMainPort=<?php echo ((int)@$_GET['onlyMainPort']);?>';
}
function retry(nStep){
	D.body.style.cursor='wait';
	D.forms.nextStep.step.value=nStep;
	goSubmit();
}

</script>
<table style="width:100%" class="frame1"><tr><td style="vertical-align:top; text-align:center; width:80px;padding-right:10px;height:100%">
<?php

// Left icon
echo '<div class="frame1Header">'.cfCaption('genStatus','').'</div><br/>';
echo '<div style="border:1px solid #CCC;padding:7px; width:auto !important; width:1px; margin-bottom:5px">';
if($leftIcon=='loadingBig') echo outImage(outIcon($leftIcon),false,'id="leftIcon"','margin:8px'); else echo outImage(outIcon($leftIcon),false,'id="leftIcon"');
echo '</div>';
if($leftMessage==cfCaption('genError')) echo '<div class="warning" style="font-weight:bold">'; else echo '<div>';
echo $leftMessage.'</div><br/><br/><br/>';

// UPnP information
echo '<div style="border:1px solid #CCC;padding:7px; width:auto !important; width:1px; margin-bottom:5px;">'.outImage(outIcon('UPnP2'),false,false).'</div>';
if(cfGGetVar('UPnPEnabled')){
	echo '<div style="color:#2A2;font-weight:bold">'.cfCaption('genActivated').'</div>';
	if(!cfGGetVar('UPnPMappingOK')) echo '<br/><span class="warning">'.outImage(outIcon('alert')).'&nbsp;'.cfCaption('UPnPFailed').'</span>';

}
else{
	echo '<span class="warning" style="font-weight:bold">'.cfCaption('genActivatedFalse').'</span>';
}
echo '</td><td>';


/**
 * Right part
 */
echo '<div class="frame1Header" style="visibility:hidden">'.cfCaption('genStep','').'</div><br/>';
for($i=0;$i<3;$i++){
	echo outDivFrame('frame2',false,'width:540px');
	// Step number
	echo '<div style="font-size:30px;float:left; margin-right:20px; margin-top:25px; padding-bottom:25px; margin-left:3px">'.($i+1).'. </div>';
	// Title
	echo '<div class="frame2Header" style="width:100%">'.outImage(outIcon($steps[$i]['icon']),false,false,'height:16px;width:16px;margin-right:7px;vertical-align:middle');
	switch ($i) {
		case 0:
			echo cfCaption('accessTestAppOut');
			echo outImage(outIcon('appOut'),false,false,'position:absolute;right:25px;');
			break;
		case 1:
			echo cfCaption('accessTestHttpdOut');
			echo outImage(outIcon('serverOut'),false,false,'position:absolute;right:25px;');
			break;
		case 2:
			echo cfCaption('accessTestHttpdIn');
			echo outImage(outIcon('serverIn'),false,false,'position:absolute;right:25px;');
			break;
	}
	echo '</div>';

	if(!isset($steps[$i]['messageType'])) echo '<div style="margin-left:79px;margin-top:3px">';
	elseif($steps[$i]['messageType']=='error') echo '<div class="warning" style="margin-left:79px;margin-top:3px">';
	elseif($steps[$i]['messageType']=='progress') echo '<div class="warning" style="color:#44F;margin-left:79px;margin-top:3px">';
	elseif($steps[$i]['messageType']=='ok') echo '<div class="warning" style="color:#2C2;margin-left:79px;margin-top:3px">';

	if(isset($steps[$i]['message'])) echo $steps[$i]['message']; else echo '&nbsp;<br>&nbsp;';

			// Display results for other ports
			if($i==2 && isset($_GET['portCheckResult'])){
				echo '<br><br>';
				$portCheckResult=cfExplode2Levels('/',':',$_GET['portCheckResult']);

				// Browse all opened ports (except main port)
				foreach (serverAccessPortsList() as $portDescription) if($portDescription['code']!=='main'){
					// find matching test result
					if(isset($portCheckResult[$portDescription['port']])){
						// Display OK/NOK icon
						if($portCheckResult[$portDescription['port']]) echo outImageIcon('ok').'<span>'; else echo outImageIcon('warning').'<span class="warning">';

						// Display port name & number
						echo $portDescription['label'].' ('.$portDescription['port'].')</span>';
						if($step=='serverInError') echo "&nbsp;&nbsp;"; else echo '<br>'; // Server in error: reduce height as error message is quite large
					}
				}
			}

	echo '</div>';
	if($step!='serverInError' || $i<2) echo '<br clear="all">';
	echo '</div><br/>';
}
echo '</td></tr></table>';

echo '<center>';
if(isset($button1)) echo $button1;
if(isset($button2)) echo '<span style="width:30px">&nbsp;</span>'.$button2;
echo '</center>';

echo '<form name="nextStep" action="'.$_SERVER['PHP_SELF'].'" style="display:none"><input type="text" name="step" value="'.((isset($nextStep))?$nextStep:'').'"><input type="text" name="silent" value="'.((@$_GET['silent'])?1:0).'"></form>';
exit;

// If error, force form show (might have been launched hidden after an update)
if(@$error){
	echo "<script type=\"text/javascript\">wl.UICommand('show')</script>";
}
if($step=='serverInOk' && @$_GET['silent']){
	echo "<script type=\"text/javascript\">wl.UICommand('close')</script>";
}
?>
</body>