<?php
/**
 * User Interface for embedded resources management
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 2.0.0
 */

/**
 * @desc Return embedded resource direct link
 *
 * @param array $res: resource definition
 * @return string
 */
function publishResourceLink($res){
	if(!isset($res['publishToken'])) return '';
	// Cache registration info
	static $reginfo=false;	if(!$reginfo) $reginfo=cfMGetVar('weezoRegInfo');
	// Generate link
	return DNS_SITE.'/embedded.php?userName='.$reginfo['regName'].((isset($res['publishTheme']))?'&theme='.$res['publishTheme']:'').'&publishToken='.$res['publishToken'].'&type=publishResource&v='.cfGGetVar('appVersion');
}


/**
 * @desc Return code for embedded resource
 *
 * @param array $res: resource definition
 * @return string
 */
function publishResourceCode($res){
	if(!isset($res['publishToken'])) return '';
	$out ='<div style="width:'.$res['publishWidth'].';height:'.$res['publishHeight'].';position:relative">';
	$out.='<iframe frameborder="0" scrolling="auto" style="width:100%;height:100%" src="'.publishResourceLink($res).'"></iframe>';
	$out.='<img style="position:absolute;top:0;right:0;width:16;height:16;cursor:pointer" src="'.APP_SITE.'/gfx/fscreen.gif" onclick="document.location=this.parentNode.firstChild.src">';
	$out.='</div>';
	return $out;
}


// Indicate an in-application display
$_ENV['configurationEnvironment']='application';


require_once(cfAppDocRoot().'/local/directLink.php');
require_once(INCLUDE_DIR.'outputFunctions.php');

// Memorize panel selection
cfGUpdateVar('uiResourcePublishPanel',cfFileWithoutExtension(basename($_SERVER['SCRIPT_NAME'])));

// Verify that external authentication is disabled into profile, else display error message
dlCheckExternalAuth('publishResource');


/*
 ***************************************************************************************************************************
 * Process POST
 ***************************************************************************************************************************
 */
if(cfIsAsync()){
	// Insert async response header
	cfAsyncHeader();
	$id=@$_POST['activate'];
	if(!$id) $id=@$_POST['update'];

	// Activate / update resource
	if($id && ($res=cfMGetVar('weezoResData'.$id))){
		if(isset($_POST['activate'])) $res['publishToken']=cfGenerateID(30);
		if(!isset($res['publishWidth'])) {
			$res['publishWidth']=640;
			$res['publishHeight']=500;
		}

		if(isset($_POST['w']) && is_numeric($_POST['w']) && $_POST['w']>0) $res['publishWidth']=$_POST['w'];
		if(isset($_POST['h']) && is_numeric($_POST['h']) && $_POST['h']>0) $res['publishHeight']=$_POST['h'];
		if(isset($_POST['theme']) && is_dir(cfJoinPathFile(cfAppDocRoot().'/themes/',$_POST['theme']))) $res['publishTheme']=basename($_POST['theme']);
		require_once(INCLUDE_DIR.'resourceConfigFunctions.php');
		rcWriteResourceFile($res);
		echo cfAsyncXMLJSaction('updateCode("'.$id.'","'.str_replace('"','\\"',publishResourceCode($res)).'");updateLink("'.$id.'","'.str_replace('"','\\"',publishResourceLink($res)).'");');
	}

	// Deactivate resource
	elseif(isset($_POST['deactivate']) && ($res=cfMGetVar('weezoResData'.$_POST['deactivate']))){
		unset($res['publishToken']);
		require_once(INCLUDE_DIR.'resourceConfigFunctions.php');
		rcWriteResourceFile($res);
	}

	// Insert async response footer
	die(cfAsyncFooter());
}



/*
 ***************************************************************************************************************************
 * Display page
 ***************************************************************************************************************************
 */
cfInsertHEAD(false);
echo cfScriptLink('winClient.js');
?>
</head>
<body onload="init()" style="overflow:hidden">
<?php echo cfScriptLink('wz_dragdrop.js'); ?>
<script type="text/javascript">
function init(){
	alignBottom(dgi('scrollDiv'),winMe,5)
}
function backKD(kc){
	if(kc==27) parent.slideFrame('publish','/local/uiResources/publish.php','up');
}
wl.setKeycodeListener(D,backKD)
function activate(id){
	setAlpha(dgi('img'+id),1);
	dgi('state'+id).innerHTML="<?php echo cfCaption('genActivated');?>";
	dgi('activateBt'+id).style.display="none";
	dgi('deactivateBt'+id).style.display="";
	dgi('linkTD'+id).style.display="";
	var w=dgi('w'+id).value,h=dgi('h'+id).value,t=dgi('theme'+id).value;
	sendData('activate='+id+'&w='+w+'&h='+h+'&theme='+t);
}
function deactivate(id){
	setAlpha(dgi('img'+id),0.2);
	dgi('state'+id).innerHTML="<?php echo cfCaption('genActivatedFalse');?>";
	dgi('activateBt'+id).style.display="";
	dgi('deactivateBt'+id).style.display="none";
	dgi('linkTD'+id).style.display="none";
	sendData('deactivate='+id);
}
function themeSelectionControlChanged(v, selectNode){
	updateRes(selectNode.id.substr(5));
}

function updateCode(id,v){dgi('code'+id).value=v}
function updateLink(id,v){dgi('link'+id).value=v}

function updateRes(id){
	var w=Math.floor(dgi('w'+id).value),h=Math.floor(dgi('h'+id).value),t=dgi('theme'+id).value;
	if(isNaN(w) || isNaN(h)) return false;
	//alert('update='+id+'&w='+w+'&h='+h+'&theme='+t)
	sendData('update='+id+'&w='+w+'&h='+h+'&theme='+t);
}
</script>
<?php
echo outDivFrame('frame1');
echo '<div style="float:right">'.outButton(cfCaption('genBack'),"javascript:parent.slideFrame('publish','/local/uiResources/publish.php','up')",outIcon('backRed')).'</div>';
echo '<div class="uiTabTitle">'.cfCaption('publishResourceLabel').'</div><br>';

echo outDivFrame('','id="scrollDiv"','height:1;overflow:auto');

// Display all created resources, grouped by type
$nb=0;
foreach (array('explorer','bookmarks','webcam','blog','tv','website','misc') as $type){
	foreach (WEnv::resConfigs() as $id=>$res){
		if(0) $res=new WResConfig();// Zend IDE WA
				
		if($res->definition('type')!=$type) continue;
		
		if($nb>0) echo outConfigSeparator();
		$nb++;
		
		$activated=$res->getVar('publishToken');

		// Resource icon and state
		echo "<div style=\"width:10%;overflow:hidden;vertical-align:top;float:left;text-align:center\">\n";
		echo '<img id="img'.$id.'" src="'.$res->icon32().'" style="width:32;height:32;margin:0.5em'.(($activated)?'':';'.outCssOpacity(20)).'">';
		// Link state
		echo '<div id="state'.$id.'">'.(($activated)?cfCaption('genActivated'):cfCaption('genActivatedFalse')).'</div>';
		echo '</div>';
		
		// Activate / deactivate buttons
		echo '<div style="width:15%;text-align:right;vertical-align:top;float:right">';
		echo outBt(cfCaption('genActivateTrue'),outIcon('fi/icoXml'),'javascript:activate(\''.$id.'\')','id="activateBt'.$id,(($activated)?'" style="display:none"':''));
		echo outBt(cfCaption('genActivateFalse'),outIcon('icoXmlDisabled'),'javascript:deactivate(\''.$id.'\')',false,'id="deactivateBt'.$id,(($activated)?'':'" style="display:none"'));
		echo '&nbsp;</div>';

		// Center part
		echo '<fieldset style="width:75%">';
		
		
		// Resource name
		echo '<div class="frame2Header" style="white-space:nowrap">'.cfUTF8Encode($res->getVar('name')).'</div>';
		
		
		// Link part
		echo '<div style="'.(($activated)?'':';display:none').'" id="linkTD'.$id.'">';

		//  Width
		echo cfCaption('genWidth').cfCaption('genSeparator').'<input class="textInput" style="width:3em;margin-right:2em" value="'.(($res->getVar('publishWidth'))?$res->getVar('publishWidth'):640).'" onkeyup="if(isNaN(this.value)) this.value=this.value.replace(/[^0-9]/g,\'\'); else updateRes(\''.$id.'\')" onchange="updateRes(\''.$id.'\')" id="w'.$id.'">';

		// height
		echo cfCaption('genHeight').cfCaption('genSeparator').'<input class="textInput" style="width:3em" value="'.($res->getVar('publishHeight')?$res->getVar('publishHeight'):500).'" onkeyup="if(isNaN(this.value)) this.value=this.value.replace(/[^0-9]/g,\'\'); else updateRes(\''.$id.'\')" onchange="updateRes(\''.$id.'\')" id="h'.$id.'">';

		// Theme
		echo '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'.outThemeSelectionControl('standalone',false,'','',true,true,$res->getVar('publishTheme'),'theme'.$id).'<br>';

		// Code snippet
		echo '<b>'.cfCaption('linkCode').'</b><br>';
		echo '<input class="textInput" value="'.str_replace('"','&quot;',publishResourceCode($res->getVar(false))).'" style="width:100%" id="code'.$id.'" onclick="this.select()" readonly="readonly">';

		// Direct link
		echo '<b>'.cfCaption('genLink').'</b><br>';
		echo '<span style="float:right">'.outSocialURLShare(publishResourceLink($res->getVar(false)),false,$res->getVar('name')).'</span>';
		echo '<input class="textInput" value="'.publishResourceLink($res->getVar(false)).'" style="width:60%" id="link'.$id.'" onclick="this.select()" readonly="readonly">';
		
		echo '</div>';
		echo '</fieldset><br clear="all">';
	}
}

echo "</div>\n"; // end of scrollDiv
echo cfDragRegisterItems();