<?php
/**
 * Photo album 1 configuration file
 *
 * Configuration of standard photo album resource
 * this script is included by /res/administration/std/resourceConfig.php
 * this script should never be directly called.
 *
 * resourceConfig.php sets 2 variables :
 * $resourceData (array) which contains resource parameters
 * $_ENV['configurationEnvironment'] which indicates if resourceConfig.php has been called from browser ('browser')
 * or from application's embeded browser ('application')
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

require_once(INCLUDE_DIR.'resourceConfigFunctions.php');
require_once(INCLUDE_DIR.'outputFunctions.php');

$defaultDir=(cfGGetVar('specialFolderMyPictures'))?cfGGetVar('specialFolderMyPictures'):cfAppSharedDir();

/*
 ***************************************************************************************************************************
 * Verify config rights
 ***************************************************************************************************************************
 */
rcConfigRights(false);

/*
 ***************************************************************************************************************************
 * Display configuration form
 ***************************************************************************************************************************
 */

// Insert Javascript function needed for save button activation, and <form> with hidden fields.
rcInsertScriptAndForm();

// Resource icon & name
rcInsertIconName($resourceData,true,array('rcSharedFilesBox'=>array('image',$defaultDir,array('noComputerRoot'=>1))));


/*
 * Common explorers parameters
 */

/**
 * Shared folder or items
 */
//rcSharedFilesBox('image',$defaultDir);

/**
 * Combined layout
 */
$layouts=array(
	'20'=>outIcon('photoLayout20'),
	'21'=>outIcon('photoLayout21'),
	'10'=>outIcon('photoLayout10'),
	'30'=>outIcon('photoLayout30'),
	'31'=>outIcon('photoLayout31'),
	'32'=>outIcon('photoLayout32'),
	'40'=>outIcon('photoLayout40')
);

// Backward compatibility: set combinedLayout from resource subtype and layout properties
if(!isset($resourceData['combinedLayout'])){
	switch (($resourceData['subType'])){
		case 'Photo1': $resourceData['combinedLayout']='10'; break;
		case 'Photo2': $resourceData['combinedLayout']='21'; break;
		case 'Photo3':
			if(@$resourceData['layout']==0) $resourceData['combinedLayout']='32';
			elseif(@$resourceData['layout']==0) $resourceData['combinedLayout']='31';
			else $resourceData['combinedLayout']='30';
	}
};

// Insert hidden control for combined layout, for POST processing
$controlDefinition=array(); foreach ($layouts as $k=>$v) $controlDefinition[$k]=array(' ',$k,' '.$v);
echo '<div style="display:none">';
rcProcessData('combinedLayout',dataList,'21',array_keys($layouts),$controlDefinition);
echo '</div>';
echo '<div style="line-height:0.5em">&nbsp;</div>';


// Combined layout controls

echo '<h2>';
echo '<span style="float:right">';
rcProcessData('filesLayoutLocked',dataBoolean,false,false,array(outIcon('lock'),outIcon('lockOpened')),false); // Lock layout
echo '</span>';

echo cfCaption('defaultLayout');
echo '</h2>';

foreach ($layouts as $key=>$icon){
	echo outBlockItem('combinedLayoutSwitch','<img src="'.$icon.'" alt="">','id="combinedLayoutSwitch'.$key.'"','margin-right:7px;margin-left:0px;padding:5px;',array('onclick'=>'combinedLayoutSet(\''.$key.'\')'),(($key==$resourceData['combinedLayout'])?'sel':'unsel'));
}
echo '<br>';

rcProcessData('CLSShow',dataBoolean,false,false,false); 			// Default show combined layout selector to client


/*  Advanced config section
    vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv*/
rcAdvancedConfigStart();
echo '<br>';

// Upload
rcProcessData('uploadAllowed',dataUserRights,'none',false,cfCaption('explorerUploadAllowedImages').'&nbsp;&nbsp;&nbsp;'.outHelpButton('helpImgUL'),false);
$resourceData['userRights_writeAllowed']=$resourceData['userRights_uploadAllowed'];

// Download shortcut
rcProcessData('downloadShortcut',dataUserRights,'all',false,cfCaption('explorerDownloadShortcut').'&nbsp;&nbsp;&nbsp;'.outHelpButton('helpSDL'),false);

// Multiple download
rcProcessData('multipleDownloadAllowed',dataUserRights,'all',false,cfCaption('configAllowMultipleDownload').'&nbsp;&nbsp;&nbsp;'.outHelpButton('helpMD'),false);

// Allow/Forbid remote creation of direct link
rcProcessData('shareAllowed',dataUserRights,'none',false,cfCaption('shareAllowed').'&nbsp;&nbsp;&nbsp;'.outHelpButton('shareHelp','doShare'),false,false);

// Allow/Forbid emailing files
rcProcessData('mailoutAllowed',dataUserRights,'admin',false,cfCaption('sendByMail').'&nbsp;&nbsp;&nbsp;'.rcMailoutConfigButton(),false,false,array('faded'=>!Mailout::isMainConfigurationSet()));

// File view (image, audio,...)
rcProcessData('fileViewAllowed',dataUserRights,'all',false,cfCaption('explorerAllowFullSizeImage'),false); // Allow user to see full-size image

// File access parameters (apply to files and folders located in "path")
rcProcessData('downloadAllowed',dataBoolean,true,false,false); // Allow / Forbid read access to files and folders
rcProcessData('modifyAllowed',dataBoolean,false,false,false); // Allow / Forbid modifcation access to files and folders (need writeAllowed to true)
rcProcessData('executeAllowed',dataBoolean,false,false,false); // Allow / Forbid execute access to files
rcProcessData('remoteDownloadAllowed',dataBoolean,false,false,false); // Allow / Forbid remote download of files
rcProcessData('sendAllowed',dataBoolean,false,false,false); // Allow / Forbid send files

rcProcessData('userTempDirAccessAllowed',dataBoolean,false,false,false); // Allow / Forbid access to user temp directory (data/sessiondata/(session_id)/)
rcProcessData('resourceDataDirAccessAllowed',dataBoolean,true,false,false); // Allow / Forbid access to resource directory (data/res/(res type)/(res subType)/(res file name)/)
rcProcessData('reducedPreview',dataBoolean,true,false,false); // Preload small preview of images in photo view window
rcProcessData('bittorrentEnabled',dataBoolean,true,false,false); // Allow / Forbid bittorrent download


// Display jpg/tiff EXIF data
rcProcessData('showExifData',dataUserRights,'all',false,cfCaption('photoCfgExif'));

// Allow users to comment files (images, audio files...)
rcProcessData('commentsAllowed',dataUserRights,'none',false,cfCaption('genCommentsAllowed').'&nbsp;&nbsp;&nbsp;'.outHelpButton('helpAllowComments'),false);

// Allow/Forbid file rating
if(cfGGetVar('ratingAllowed')) rcProcessData('ratingAllowed',dataUserRights,'all',false,cfCaption('ratingAllowed'),false,false);
echo '<br>';


// Slideshow interval, in seconds
rcProcessData('slideShowInterval',dataNumeric,4,array(1,1000),array(cfCaption('photoCfgSlideShowInterval'),3,cfCaption('genSeconds')),false);

/**
 * Preview
 */
rcProcessData('previewAutoSize',dataBoolean,true,false,false); // Automatic preview size
rcProcessData('stretchSmallImages',dataBoolean,false,false,false); // Strech small images when displayed (preview and full view)


/**
 * Thumbnails
 */
echo '<div class="configSubFrame">';
echo '<div class="configSubFrameHeader" style="margin-bottom:0;padding-bottom:0">';
echo outButton(cfCaption('genClear'),'javascript:clearThumbnails()',outIcon('eraser'),false,false,'style="position:absolute;top:2px;right:5px"');
echo cfCaption('genThumbnails').'</div>';

// Indicate that thumbnails must be generated
rcProcessData('photoThumbnailsNeeded',dataBoolean,true,false,false); // Generate images thumbnails when browsing a dir

rcProcessData('itemsPerPage',dataList,'50',array('10','20','50','100','0'),false);	// nb of thumbnails per page (used by layout 21)


rcProcessData('thumbnailsSizeFactorLocked',dataBoolean,false,false,false); 			// If set to true, user may not modify size factor */

// Default sort by
rcProcessData('efSortBy',dataList,'',array('','date','size','name'),array(
	array(cfCaption('genSortBy')),array(
	array('','-'),
	array('date',cfCaption('explorerDate')),
	array('size',cfCaption('genSize')),
	array('name',cfCaption('explorerFilename'))),false),false);
	echo ' &nbsp; &nbsp;';
	// Default sort order
	rcProcessData('efSortOrder',dataList,'asc',array('asc','desc'),array(
	array('','asc',outIcon('up')),
	array('','desc',outIcon('down'))
	),true);

// Default thumbnails size
rcProcessData('thumbnailsSize',dataNumeric,240,false,false); 						// Biggest thumbnails size */
//rcProcessData('thumbnailsSizeFactor',dataNumeric,1,false,false); 					// Thumbnail size factor (0=1/4,1=1/2,2=1/1) */



// Thumbnails size controls
$thumbnailsSizeFactor=array(
	'0'=>outIcon('efTNSize480'),
	'1'=>outIcon('efTNSize481'),
	'2'=>outIcon('efTNSize482'),
);
echo '<div style="display:none">';
$controlDefinition=array(); foreach ($thumbnailsSizeFactor as $k=>$v) $controlDefinition[$k]=array(' ',$k,' '.$v);
rcProcessData('thumbnailsSizeFactor',dataList,'1',array(0,1,2),$controlDefinition);
echo '</div>';

echo '<span>'.cfCaption('genSize').'</span><br>';
foreach ($thumbnailsSizeFactor as $key=>$icon){
	echo outBlockItem('thumbnailsSizeFactorSwitch','<img src="'.$icon.'" alt="">','id="thumbnailsSizeFactorSwitch'.$key.'"','margin-right:7px;margin-left:0px;padding:5px;',array('onclick'=>'thumbnailsSizeFactorSet(\''.$key.'\')'),(($key==$resourceData['thumbnailsSizeFactor'])?'sel':'unsel'));
}
echo '<br>';



// Monitor thumbnails generation
rcProcessData('monitorThumbnailGen',dataBoolean,true,false,false);


// Preview
// No preview for this resource
rcProcessData('stretchSmallImages',dataBoolean,false,false,false); // Strech small images when displayed (preview and full view)
echo '</div>';


// Layout
rcProcessData('filesLayout',dataText,'resource',false,false/*do not display control*/); // Default files layout: resource specific */

// End of advanced config panel
rcAdvancedConfigEnd();

// Comments panel
echo rcCommentsPanelControl();

// Photo album theme
echo rcThemeSelectionControl();

// Save / cancel buttons
echo '<br/>'.rcButtonSaveCancel();

// If resource need access to resource data dir, create it
rcCreateResourceDataDir();

// Indicate that resource has been initialized
rcProcessData('resourceConfigured',dataBoolean,true,false,false);

// Save parameters
rcWriteResourceFile($resourceData, $fileName);
?>
<script type="text/javascript">
// Clear thumbnails
function clearThumbnails(browser){wl.mask.show(0,0); sendData("clearThumbnails=1",undefined,true,3);}
function combinedLayoutSet(l){
	var i,items=D.getElementsByName("combinedLayout"),id='combinedLayout'+l;
	for(i=0;i<(items.length);i++){
		if(items[i].value==l) D.getElementsByName("combinedLayout")[i].checked="checked"
	}
	eSaveBt('combinedLayout',l);
}
// Change thumbnails Size Factor
function thumbnailsSizeFactorSet(l){
	var i,items=D.getElementsByName("thumbnailsSizeFactor"),id='thumbnailsSizeFactor'+l;
	for(i=0;i<(items.length);i++){
		if(items[i].value==l) D.getElementsByName("thumbnailsSizeFactor")[i].checked="checked"
	}
	eSaveBt('thumbnailsSizeFactor',l);
}



// Clear thumbnails
function clearThumbnails(browser){
	wl.mask.show(0,0);
	sendData("clearThumbnails=1",undefined,true,3);
}
// Any parameter modified
function dataChanged(dataName,dataValue){}
//wl.event.emit("resource",{"create":1,"filename":"'.$filename.'","id":"fake"},"up")
</script>