<?php
/**
 * Chat main file
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
*
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.4
 */

/*
 * File map :
 * functions	getMinMaxDate
 *		generateID
 *		getTimeStamp
 *		getCommentsList
 *		getMessagesList
 *
 * INIT BLOG DATA (once)
 * Load messages XML file (or create it)
 * PROCESS POST PARAMETERS
 * Insert HEAD, FORMS and JAVASCRIPT functions
 * DISPLAY BLOG
 *	Messages and comments
 *	Profile section
 *	History
 *
 */

define('MAX_MESSAGES',100);
define('CHAT_DISCONNEC_TIMEOUT',10);

require_once(INCLUDE_DIR.'outputFunctions.php');
require_once('smileys.php');
$colors=array('#999', '#44B', '#6FAC20', 'red', 'F2007D', '#E78F28' ,'#004707', '#006A76');
$webcamSizes=array(0=>array('160','100'),1=>array('320','200'),2=>array('480','300'));

/**
 * INIT RESOURCE DATA
 */
function resWebcamInitResource(){

	// Define resource-vars that will be saved if user is singleUser
	cfSetSavedRVar('browserCnxSound','browserMsgSound');

	if(cfRGetVar('resourceParametersInitialized')) return;
	if(!isset($_ENV['configurationEnvironment'])) $_ENV['configurationEnvironment']='browser';

	$pseudo=((cfUGetVar('pseudo'))?cfUGetVar('pseudo'):cfUGetVar('name'));
	if(WEnv::user()->getVar('accountType')=='list') $pseudo=WEnv::user()->subUser()->name();

	if(substr($pseudo,0,4)=='FBID') $pseudo=substr($pseudo,strpos($pseudo,':')+1);
	cfRSetVar('userPseudo',$pseudo);

	cfRSetVar('lastDisplayedMessage',-1);
	cfRInitVar('browserCnxSound','true');
	cfRInitVar('browserMsgSound','false');
	cfRInitVar('refreshMs',1500);
	cfRInitVar('webcamFPS',10);

	// Webcam
	cfRInitVar('webcamSize',1); // 0 (160x100),1 (320x200),2 (480x300)
	cfRInitVar('webcamWidth',320);
	cfRInitVar('webcamHeight',200);

	// 1st time (from server start-up) resource is called, clear all remaining webcam thumbnails
	if(!cfMIssetVar('weezoChatRes'.cfRGetVar('id').'Messages')){
		foreach (glob(cfAppResourceDir().'/tmp_*.*') as $cfn) @unlink($cfn);
	}

	// iPhone specific
	if(cfBGetVar('name')=='iPhone') {
		cfRSetVar('meta','<meta name="viewport" content="width=device-width, initial-scale=1.0, user-scalable=no">');
		cfRSetVar('textInputValidation','onchange');
	}

	if(cfBGetVar('screenSize')=='monitor') cfRSetVar('displayDate',1);

	// Disable webcam for non-flash enabled browsers && Wii
	if(!cfBGetVar('flash') || cfBGetVar('name')=='Wii') cfRSetVar('webcam',false);

	cfRSetVar('resourceParametersInitialized',true);
}

/**
 * @desc Return latest webcam image's filename of given user
 *
 * @param sring $userId: id of user, empty for current user
 * @return string: filename
 */
function resWebcamTmpImage($userId=false){
	return cfAppResourceDir().'/tmp_webcam_'.(($userId)?$userId:cfRGetVar('chatUserId')).'.jpg';
}

/**
 * @desc Process POST
 *
 */
function resWebcamProcessUploadedImageWebcamProcessPOST(){
	if(substr($GLOBALS["HTTP_RAW_POST_DATA"],0,4)!='') die('Invalid data');
	// Update image index, and if OK (existing user), save image.
	if(resWebcamReadAndUpdateUsers(-1,false,true,false))	{
		file_put_contents(resWebcamTmpImage(cfRGetVar('chatUserId')),$GLOBALS["HTTP_RAW_POST_DATA"]);
	}

}

/**
 * @desc insert smileys images into a message
 *
 * @param string $message : message to process
 * @return string
 */
function resWebcamInsertSmileys($message){
	global $smileys;
	$output='';
	while(strlen($message)){
		$first=9999999;
		$smileyKey=false;
		foreach ($smileys as $key=>$value){
			$pos=strpos($message,$key);
			if($pos!==false && $pos<$first){
				$smileyKey=$key;
				if(($first=$pos)==0) break;
			}
		}
		if($smileyKey){
			$block=substr($message,0,$first+strlen($smileyKey));
			$output.= str_replace($smileyKey,outImage(outIcon('smileys/'.$smileys[$smileyKey]),false,false,'vertical-align:middle;'),$block);
			$message=substr($message,strlen($block));
		}
		else{
			$output.=$message;
			$message='';
		}
	}
	return $output;
}

/**
 * @desc post (re)connection processings
 *
 * @param boolean $reconnection : set to true if user is reconnecting, false if first-time connection
 */
function resWebcamSetUserConnected($reconnection=false){
	cfRSetVar('connectionTime',time());

	// Insert (re)connection message in messages list
	resWebcamReadAndUpdateMessages(cfUTF8Decode(cfCaption((($reconnection)?'chatReconnected':'chatConnected'),cfUTF8Encode(cfRGetVar('userPseudo'))),true,true,false),true);

	// Send visual notification on weezo computer
	if(cfRGetVar('connectionNotification') && cfUGetVar('id')!='applicationUser') cfServerSendCommand('chat newConnection="true" pseudo="'.cfRGetVar('userPseudo').'" icon="'.cfUGetVar('icon').'" resourceConfigFilename="'.cfRGetVar('resourceName').'.res" audioProfile="'.cfUGetVar('audioProfile').'"');

	// Force open chat window on other users if current user is admin
	if(!cfIsInApp() && cfUGetVar('administrator') && cfRGetVar('forceOpen')){
		$sessions=glob(cfAppDataDir().'/sessionData/sess_*');
		foreach ($sessions as $completeFilename){
			$sessionId=substr(basename($completeFilename),5);
			if($sessionId!=wSession_id()) cfMSetVar('weezoChatForceOpen'.$sessionId,cfRGetVar('resourceName').'.res');
		}
	}

}

/**
 * @desc return connected users
 * @return array: connected users :
 * 		- key is user sessionid
 * 		- name: user name / pseudo
 * 		- icon: user icon
 * 		- typing: true if user is typing
 * 		- color: color index
 * 		- connected: true if still connected, 'timeout' if timeout disconnected, false if gently disconnected
 * 		- lastPing: timestamp of last ping
 * 		- webcamImage: index of webcam image, 0 if no webcam started
 * @param boolean $isTyping : true if user is currently typing a message
 * @param boolean $disconnecting : true if user window is being closed
 * @param mixed $newWebcamImage: false, no update, true: new image received, 0: webcam disconnected
 * @param boolean $createUserIfUnknown: if false and inexisting user, return false
 *
 */
function resWebcamReadAndUpdateUsers($isTyping=-1, $disconnecting=false, $newWebcamImage=false, $createUserIfUnknown=true){
	global $colors;
	// Get connected users array
	if(cfMIssetVar('weezoChatRes'.cfRGetVar('id').'Users'))	$users=cfMGetVar('weezoChatRes'.cfRGetVar('id').'Users');
	// First-time : initialize users array with 'system' user
	else $users=array('system'=>array('name'=>false, 'typing'=>false, 'icon'=>false, 'color'=>0, 'connected'=>false, 'lastPing'=>0, 'ip'=>0, 'groupId'=>0, 'webcam'=>0));

	// If user doesn't have an id yet, try to see if a previously connected user matches, else generate ID
	if(!cfRGetVar('chatUserId')){
		// Browse connected users
		foreach ($users as $userId=>$user){
			// Found matching user -> reconnect
			if($user['name']==cfRGetVar('userPseudo') && $user['ip']==$_SERVER['REMOTE_ADDR'] && $user['groupId']==cfUGetVar('id') && $user['HTTP_USER_AGENT']==$_SERVER['HTTP_USER_AGENT']){
				cfRSetVar('chatUserId',$userId);
				cfRSetVar('color',$user['color']);
				$users[$userId]['connected']=true;
				resWebcamSetUserConnected();
				break;
			}
		}
		// If user not found in users list, create a new id
		if(!cfRGetVar('chatUserId')){
			if(!cfRGetVar('userPseudo') && isset($_GET['userPseudo']) && cfIsInApp()) cfRSetVar('userPseudo',$_GET['userPseudo']);

			cfRSetVar('chatUserId',rand(0,9999999999));
			cfRSetVar('color',(1 + (count($users) % (count($colors)-1))));
			$users[cfRGetVar('chatUserId')]=array('name'=>cfRGetVar('userPseudo'),'typing'=>max($isTyping,0), 'icon'=> cfUGetVar('icon'), 'color'=> (1 + (count($users) % (count($colors)-1))), 'connected'=>true, 'lastPing'=>time(), 'ip'=>$_SERVER['REMOTE_ADDR'], 'HTTP_USER_AGENT'=>$_SERVER['HTTP_USER_AGENT'],'groupId'=>cfUGetVar('id'),'webcam'=>0);
			resWebcamSetUserConnected(true);
		}
	}
	// If current user (has been assigned an Id but) is not in user list, add it
	elseif(cfRGetVar('chatUserId') && !isset($users[cfRGetVar('chatUserId')])){
		cfRSetVar('color',(1 + (count($users) % (count($colors)-1))));
		$users[cfRGetVar('chatUserId')]=array('name'=>cfRGetVar('userPseudo'),'typing'=>max($isTyping,0), 'icon'=> cfUGetVar('icon'), 'color'=> (1 + (count($users) % (count($colors)-1))), 'connected'=>true, 'lastPing'=>time(), 'ip'=>$_SERVER['REMOTE_ADDR'], 'HTTP_USER_AGENT'=>$_SERVER['HTTP_USER_AGENT'],'groupId'=>cfUGetVar('id'),'webcam'=>0);
		cfRSetVar('connectionTime',time());
	}
	// Update lastPing, typing and set timeout users
	foreach ($users as $userId=>$user){
		if(cfRGetVar('chatUserId') && $userId==cfRGetVar('chatUserId')) {
			// Set last ping
			$users[$userId]['lastPing']=time();
			// Set isTyping
			if($isTyping!=-1) $users[$userId]['typing']=$isTyping;
			// If reconnecting, send system message
			if(!$disconnecting && $user['connected']!==true){
				$users[$userId]['connected']=true;
				resWebcamSetUserConnected();
			}
			// User disconnected
			elseif($disconnecting) {
				resWebcamReadAndUpdateMessages(cfUTF8Decode(cfCaption('chatDisconnected',cfUTF8Encode($users[$userId]['name'])),true,true,false),true);
				$users[$userId]['connected']=false;
				$users[$userId]['webcam']=0;
				@unlink(resWebcamTmpImage($userId));
			}
			// new webcam image received
			if($newWebcamImage===true) {
				cfMSetVar('weezoChatWebcamIndex',(int)cfMGetVar('weezoChatWebcamIndex')+1);
				$users[$userId]['webcam']=cfMGetVar('weezoChatWebcamIndex');
			}
			elseif($newWebcamImage===0) $users[$userId][$webcam]=0;
		}
		// Timeout disconnect
		elseif ($user['connected']===true && (time() - $user['lastPing']) > CHAT_DISCONNEC_TIMEOUT && $user['lastPing']) {
			if($user['connected']===true) resWebcamReadAndUpdateMessages(cfUTF8Decode(cfCaption('chatDisconnected',cfUTF8Encode($user['name'])),true,true,false),true);
			$users[$userId]['connected']='timeout';
			$users[$userId]['webcam']=0;
			@unlink(resWebcamTmpImage($userId));
$discouserId=$userId; // Debug
		}
	}
	cfMSetVar('weezoChatRes'.cfRGetVar('id').'Users',$users);
	return $users;
}

/**
 * @desc update and return messages array
 *
 * @param string $newMessage : new message typed by current user (false if no message)
 * @param boolean $systemMessage : set to true to indicate that message has not been typed by an user but comes from an event.
 * @return array messages :
 * 		- key : sequential message id
 * 		- value : array('message'=>utf-8 decoded message, 'user'=>userId, 'time'=>timestamp)
 *		 extra keys / values :
 * 		- 'min' : min message Id
 * 		- 'max' : max message Id
 */
function resWebcamReadAndUpdateMessages($newMessage=false,$systemMessage=false){
	$messagesUpdated=false;
	if(cfMIssetVar('weezoChatRes'.cfRGetVar('id').'Messages')) $messages=cfMGetVar('weezoChatRes'.cfRGetVar('id').'Messages');
	else {
		$messages=array('min'=>0, 'max'=>0);
		$messagesUpdated=true;
		if(!$newMessage) $newMessage=' ';
	}
	// If messages array is too long, trash oldest messages
	while (count($messages) > MAX_MESSAGES){
		unset($messages[$messages['min']]);
		$messages['min']++;
		$messagesUpdated=true;
	}
	// If new message is posted by current user, add it
	if($newMessage){
		$messages[$messages['max']]=array('message'=>$newMessage, 'user'=>(($systemMessage)?'system':cfRGetVar('chatUserId')), 'time'=>time());
		$messages['max']++;
		$messagesUpdated=true;
		if($newMessage!="" && cfRGetVar('dumpToFile')) cfAppendTextToFile(date(cfCaption('_DATE_FORMAT')).' '.$newMessage,cfAppResourceDir().'/chat.txt');

		if(cfRGetVar('messageNotification') && cfUGetVar('id')!='applicationUser') cfServerSendCommand('chat newMessage="true" pseudo="'.cfRGetVar('userPseudo').'" icon="'.cfUGetVar('icon').'" resourceConfigFilename="'.cfRGetVar('resourceName').'.res" audioProfile="'.cfUGetVar('audioProfile').'"');
	}
	// Update messages
	if($messagesUpdated) cfMSetVar('weezoChatRes'.cfRGetVar('id').'Messages',$messages);

	return $messages;
}

/**
 * @desc Display connected user icon + pseudo + writing state
 *
 * @param string $userId : user id
 * @return string : HTML code;
 */
function resWebcamDisplayUser($userId){
	global $users;
	global $colors;

	if($userId=='system') return;

	$output='<div id="user'.$userId.'" class="frame3" style="overflow:hidden;display:'.(($users[$userId]['connected']===true)?'block':'none').((cfGetBrowser()=='ie')?';width:100%;':'').'"><div style="position:relative;font-weight:bold;color:'.$colors[$users[$userId]['color']].'" nowrap="nowrap">';


	// Webcam button
	if(cfRGetVar('webcam')){
		$output.='<div style="position:absolute;right:0.2em">';
		// Enable toggle webcam button
		if($userId==cfRGetVar('chatUserId')||$users[$userId]['webcam']>0)
			$output.=outButton(false,'javascript:toggleWebcam(\''.$userId.'\')',outIcon('webcam'),cfCaption('resWebcamStd'),'toggleWebcamBt_'.$userId);
		else
			$output.=outButtonDisabled(false,'javascript:toggleWebcam(\''.$userId.'\')',outIcon('webcamNo'),cfCaption('resWebcamStd'),'toggleWebcamBt_'.$userId);
		$output.='</div>';
	}

	// User icon
	$output.='<img src="'.cfExtGFX('/gfx/icons/'.$users[$userId]['icon'],24,24).'" style="vertical-align:middle; margin-right:0.5em">';
	// User name
	$output.=cfUTF8Encode($users[$userId]['name']);
	// Editing icon
	$output.=outImage(outIcon('edit'),false,'id="userT'.$userId.'"','position:absolute; right:0.5em;top:2px;display:'.(($users[$userId]['typing'] && $userId!=cfRGetVar('chatUserId'))?'block':'none'));
	$output.="</div>";
	// User webcam
	if(cfRGetVar('webcam')){
		// Folded webcam (if not activated so far)
		if($users[$userId]['webcam']==0)
			$output.=outFlashPlayer('/js/videoChat.swf','webcamFlash_'.$userId,1,1,false,false);
		else
			$output.=outFlashPlayer('/js/videoChat.swf','webcamFlash_'.$userId,cfRGetVar('webcamWidth'),cfRGetVar('webcamHeight'),false,array('initOnload'=>$userId));
	}

	$output.="</div>";
	return $output;
}

/**
 * @desc replace email, http, https, ftp adresses by HTML links
 * @author : Sune Rievers @ php manual
 *
 * @param string $text : text to parse
 * @return string
 */
function resWebcamMakeLinksBlank($text){
  return  preg_replace(
     array(
       '/(?(?=<a[^>]*>.+<\/a>)
             (?:<a[^>]*>.+<\/a>)
             |
             ([^="\']?)((?:https?|ftp|bf2|):\/\/[^<> \n\r]+)
         )/iex',
       '/<a([^>]*)target="?[^"\']+"?/i',
       '/<a([^>]+)>/i',
       '/(^|\s)(www.[^<> \n\r]+)/iex',
       '/(([_A-Za-z0-9-]+)(\\.[_A-Za-z0-9-]+)*@([A-Za-z0-9-]+)
       (\\.[A-Za-z0-9-]+)*)/iex'
       ),
     array(
       "stripslashes((strlen('\\2')>0?'\\1<a target=\"_blank\" href=\"\\2\">\\2</a>\\3':'\\0'))",
       '<a\\1',
       '<a\\1 target="_blank">',
       "stripslashes((strlen('\\2')>0?'\\1<a target=\"_blank\" href=\"http://\\2\">\\2</a>\\3':'\\0'))",
       "stripslashes((strlen('\\2')>0?'<a href=\"mailto:\\0\">\\0</a>':'\\0'))"
       ),
       $text
   );
}

/**
 * @desc return HTML code of messages
 *
 * @param boolean $onlyNewMessages : set to true to display only new messages from last display
 * @param boolean $invert: true to display last message in 1st position (mobile view).
 * @return string : HTML code
 */
function resWebcamChatMessages($onlyNewMessages){
	global $messages;
	global $users;
	global $colors;
	global $otherUserTyped;

	$output='';
	$lastDisplayedMessage=cfRGetVar('lastDisplayedMessage');
	if($onlyNewMessages && $lastDisplayedMessage==$messages['max']) return;
	if($lastDisplayedMessage>$messages['max']) $lastDisplayedMessage=$messages['max']-1; // should only occure after a cls

	$yesterday=mktime(null,null,null,date("m"),date("d"),date("Y"));

	foreach ($messages as $key=>$value) if($key!='min' && $key!='max' && (!$onlyNewMessages || $key>$lastDisplayedMessage) && (cfRGetVar('showPreviousMessages') || $value['time']>=cfRGetVar('connectionTime')-1)){

		$msg='<div style="width:95%';

		// System messages (connection/disconnection: use chatSystemMessage class
		if(!$users[$value['user']]['name']) $msg.='" class="chatSystemMessage">';

		// Color
		else $msg.=';color:'.$colors[$users[$value['user']]['color']].'">';

		// Date-time
		if(cfRGetVar('displayDate')) $msg.=date(cfCaption((($value['time']>$yesterday)?'_HMS_DATE_FORMAT':'_FULL_DATE_FORMAT')),$value['time']).' ';
		// Author
		if($users[$value['user']]['name']) $msg.='('.cfUTF8Encode($users[$value['user']]['name']).') ';

		// Message
		$msgBody=cfUTF8Encode($value['message'],false,false);
		$msgBody=str_replace('&lt;big&gt;','<span style="font-size:200%">',str_replace('&lt;/big&gt;','</span>',$msgBody));
		$msgBody=str_replace('&lt;small&gt;','<span style="font-size:70%">',str_replace('&lt;/small&gt;','</span>',$msgBody));
		$msgBody=resWebcamInsertSmileys(resWebcamMakeLinksBlank($msgBody));
		$msg.= $msgBody."</div>";

		// Add to message list
		if(cfRGetVar('invertOrder')) $output=$msg.$output; else $output.=$msg;

		if($value['user']!=cfRGetVar('chatUserId')) $otherUserTyped=1;
		if($key>$lastDisplayedMessage) $lastDisplayedMessage=$key;
	}
	cfRSetVar('lastDisplayedMessage',$lastDisplayedMessage);
	return $output;
}





// Init resource
resWebcamInitResource();

// Force open chat on client's browser(s)
if(cfIsInApp()){
	if(isset($_GET['userPseudo'])) {cfUSetVar('pseudo',$_GET['userPseudo']);cfRSetVar('pseudo',$_GET['userPseudo']);}
	// $_GET['forceOpen'] contains session id of client on which chat must be opened.
	// weezoChatForceOpen.sessionId is check by clients on ping
	if(isset($_GET['forceOpen'])) cfMSetVar('weezoChatForceOpen'.$_GET['forceOpen'],cfRGetVar('resourceName').'.res');
}


/*
 ***************************************************************************************************************************
 * Process POST commands
 ***************************************************************************************************************************
 */

// Chat window closed
if(@$_POST['command']==='close'){
	resWebcamReadAndUpdateUsers(false,true);
	exit;
}
elseif(isset($_POST['message'])){
	// watchdog message: discard message and reset timer
	if($_POST['message']=='*wdTO*'){
		cfAsyncHeader();
		echo cfAsyncXMLJSaction('resetTimeout()');
		die(cfAsyncFooter());
	}
	if(isset($_POST['cnxSound']) && $_POST['cnxSound']) cfRSetVar('browserCnxSound','true'); else cfRSetVar('browserCnxSound','false');
	if(isset($_POST['msgSound']) && $_POST['msgSound']) cfRSetVar('browserMsgSound','true'); else cfRSetVar('browserMsgSound','false');
	// Process commands
	if(cfUGetVar('administrator')){
		// Clear current messages
		if($_POST['message']=='cls'){$_POST['message']='';cfMUnsetVar('weezoChatRes'.cfRGetVar('id').'Messages');}
	}
	$users=resWebcamReadAndUpdateUsers(((isset($_POST['typing']) && $_POST['typing']=='1')?1:0));
	$messages=resWebcamReadAndUpdateMessages(cfUTF8Decode($_POST['message']));

	// Check if any of the new messages comes from another user
	$otherUserTyped=0;

	$output='';

	// XML header
	cfAsyncHeader();
	// Update displayed messages
	$msgs=str_replace('"','\\"',resWebcamChatMessages(true));
	if($msgs) $output.=cfAsyncXMLJSaction('updateMessages("'.$msgs.'",'.(0+$otherUserTyped).')');

	// Check users modifications
	$memUsers=cfRGetVar('users');
	foreach ($users as $userId => $value){
		// New user has connected
		if(!isset($memUsers[$userId])) $output.=cfAsyncXMLJSaction('addUser("'.$userId.'","'.addslashes(str_replace(array("\n","\r"),'',resWebcamDisplayUser($userId))).'")');
		// User has reconnected / disconnected
		elseif ($memUsers[$userId]['connected']!==$value['connected']){
			if($value['connected']===true) $output.=cfAsyncXMLJSaction('showUser("'.$userId.'")');
			else $output.=cfAsyncXMLJSaction('hideUser("'.$userId.'")');
		}
		// User typing status has changed
		elseif ($memUsers[$userId]['typing']!=$value['typing'] && $userId!=cfRGetVar('chatUserId')){
			$output.=cfAsyncXMLJSaction('setUserTyping("'.$userId.'","'.(($value['typing'])?'1':'0').'")');
		}
		// User webcam online
		elseif ($memUsers[$userId]['webcam']==0 && $value['webcam']>0 && $userId!=cfRGetVar('chatUserId')){
			$output.=cfAsyncXMLJSaction('setUserWebcamOnline("'.$userId.'")');
		}
		// User webcam onffine
		elseif ($memUsers[$userId]['webcam']>0 && $value['webcam']==0 && $userId!=cfRGetVar('chatUserId')){
			$output.=cfAsyncXMLJSaction('setUserWebcamOffline("'.$userId.'")');
		}
	}
	// Re set async request timeout
	$output.=cfAsyncXMLJSaction('resetTimeout()');
	echo $output;
	echo cfAsyncFooter(false);
	cfRSetVar('users',$users);
	exit;
}

/**
 * WEBCAM
 */
// client disconnected his webcam
elseif (isset($_POST['webcamShutdown'])){
	resWebcamReadAndUpdateUsers(-1,false,0);
	@unlink(resWebcamTmpImage());
	cfAsyncXMLJSaction();
	die(cfAsyncFooter());
}
// Check for new webcam image
// DISCARDED
elseif (isset($_GET['webcamCheck'])){
	$users=resWebcamReadAndUpdateUsers();
	// User disconnected or switched webcam off
	if(!isset($users[$_POST['webcamCheck']])||$users[$_POST['webcamCheck']]['webcam']==0){
		echo '<imageIndex>disconnect</imageIndex>';
	}
	// Return latest image index
	else{
		echo "<imageIndex>".$users[$_POST['webcamCheck']]['webcam'].'</imageIndex>';
	}
	die(cfAsyncFooter());
}
// Download latest webcam image
elseif (isset($_GET['webcamGetImage'])){
	if(!is_file(resWebcamTmpImage($_GET['webcamGetImage']))) exit;
	header('Content-Type: image/jpeg');
	echo @file_get_contents(resWebcamTmpImage($_GET['webcamGetImage']));
	exit;
}
// Set webcam size
elseif (isset($_POST['webcamSetSize'])){

	$s=(int)$_POST['webcamSetSize'];
	cfAsyncHeader();
	if($s===0||$s===1||$s===2){
		cfRSetVar('webcamSize',$s);
		cfRSetVar('webcamWidth',$webcamSizes[$s][0]);
		cfRSetVar('webcamHeight',$webcamSizes[$s][1]);
	}
	die(cfAsyncFooter());
}

// Client uploaded webcam image
elseif(isset($GLOBALS["HTTP_RAW_POST_DATA"])) resWebcamProcessUploadedImageWebcamProcessPOST();


$users=resWebcamReadAndUpdateUsers(); cfRSetVar('users',$users);
$messages=resWebcamReadAndUpdateMessages(false);





/**
 * Mobiles
 */
if(cfIsMobile()) {require(dirname($_SERVER['SCRIPT_FILENAME']).'/mobileIndex.php');exit;}

//if(!cfIsAsync()) cfDbg($_SERVER['REQUEST_URI']);

/*
 ***************************************************************************************************************************
 * Insert HEAD, JS functions and form
 ***************************************************************************************************************************
 */
cfInsertHEAD(false,array('title'=>cfCaption('genChat')));
?>
<script type="text/javascript">
var webcamWidth=<?php echo cfRGetVar('webcamWidth');?>;
var webcamHeight=<?php echo cfRGetVar('webcamHeight');?>;
var userId=<?php echo cfRGetVar('chatUserId');?>;
var playSoundOnConnection=<?php echo cfRGetVar('browserCnxSound'); ?>;
var playSoundOnMessage=<?php echo cfRGetVar('browserMsgSound'); ?>;
var userList=new Array(<?php
$fDisp=false;
foreach ($users as $userId=>$value) if($userId!='system'){
	if($fDisp) echo ','; else $fDisp=true;
	echo '"'.$userId.'"';
}
?>);
var tO,wdTO;
var blinkState="visible";
function init(){
	winResized();
	dgi("chatDiv").scrollTop = dgi("chatDiv").scrollHeight;
	resetTimeout();
	setInterval("blink()",500);
	setTimeout('dgi("textInput").focus()',500); // FF2 WA: pb if done before animation ended
}
function winResized(){
	var margins=<?php if(cfGetBrowser()=='ie') echo 10; else echo 20;?>;

	dgi("chatDiv").style.width=Math.max(10,(dgi("usersDiv").offsetLeft - dgi("chatDiv").offsetLeft - margins))+"px";
	dgi("chatDiv").style.height=Math.max(10,(dgi("inputDiv").offsetTop - dgi("chatDiv").offsetTop - margins))+"px";
	dgi("inputDiv").style.width=Math.max(10,(dgi("usersDiv").offsetLeft + dgi("usersDiv").offsetWidth - dgi("inputDiv").offsetLeft - (margins - 10)))+"px";
	dgi("usersDiv").style.height=Math.max(10,(dgi("inputDiv").offsetTop - dgi("usersDiv").offsetTop - margins))+"px";
}
function winOnClose(){
	dgn('command').value='close';
	asyncSubmitForm('updateForm',true);
}
function blink(){
	blinkState=(blinkState=="visible")?"hidden":"visible";
	for(var i=0;i<userList.length;i++) dgi("userT"+userList[i]).style.visibility=blinkState;
}
function sendMessage(message){
	if(tO) clearTimeout(tO); tO=0;
	if(wdTO) clearTimeout(wdTO); wdTO=0;
	if(message){dgn('message').value=message;if(message!="*wdTO*") dgi('textInput').value=""} else dgn('message').value="";
	if(playSoundOnConnection) dgn('cnxSound').value='1'; else dgn('cnxSound').value='0';
	if(playSoundOnMessage) dgn('msgSound').value='1'; else dgn('msgSound').value='0';
	if(dgi('textInput').value=="") dgn('typing').value='0'; else dgn('typing').value='1';
	wdTO=setTimeout('sendMessage("*wdTO*")',<?php echo max(3*cfRGetVar('refreshMs'),3000); // Watchdog timer ?>)
	asyncSubmitForm('updateForm',message);
}
function keyPress(e){
	if(e) kc=e.which; else {if(W.event) kc=W.event.keyCode; else return;}
	if((kc==13) && dgi('textInput').value!="") sendMessage(dgi('textInput').value);
}
function updateMessages(nm, otherUser){
	dgi("chatDiv").innerHTML+=nm;
	dgi("chatDiv").scrollTop = dgi("chatDiv").scrollHeight;
	if(playSoundOnMessage && otherUser) playsound("chatMsgSound");
	if(otherUser) bringToFront();
	<?php if(cfIsInApp()) echo 'if(otherUser) wl.UICommand("show");';?>
	dgi("textInput").focus();
}
function bringToFront(){
	if(dgi("textInput").value=="") self.focus();
	if(W.winMe) winMe.bringToFront();
}
function resetTimeout(){
	if(wdTO) clearTimeout(wdTO); wdTO=0;
	if(tO) clearTimeout(tO);
	tO=setTimeout("sendMessage()",<?php echo cfRGetVar('refreshMs');?>);
}
function addUser(userId, HTMLCode){
	dgi("usersDiv").innerHTML+=HTMLCode;
	userList[userList.length]=userId;
	if(playSoundOnConnection) playsound("chatCnxSound");
}
function showUser(userId){
	dgi("user"+userId).style.display="block";
	if(playSoundOnConnection) playsound("chatCnxSound");
}
function hideUser(userId){if(dgi("user"+userId)) dgi("user"+userId).style.display="none";}
function setUserTyping(userId,isTyping){
	if(isTyping==1) dgi("userT"+userId).style.display="block";
	else dgi("userT"+userId).style.display="none";
}

function format(tag,closeTag){ // FF & Opera
	if((dgi("textInput").selectionStart || dgi("textInput").selectionEnd || dgi("textInput").selectionEnd==0) && ((!closeTag && dgi("textInput").selectionStart == dgi("textInput").selectionEnd) || (closeTag && dgi("textInput").selectionStart != dgi("textInput").selectionEnd))){
		selStart=dgi("textInput").selectionStart;
		selEnd=dgi("textInput").selectionEnd;
		var ti=dgi("textInput");
		if(closeTag) ti.value=ti.value.substring(0,selStart) + '<'+tag+'>' + ti.value.substring(selStart,selEnd) + '</'+tag+'>' + ti.value.substr(selEnd);
		else ti.value=ti.value.substring(0,selStart) + tag + ti.value.substring(selStart);
		dgi("textInput").focus();
		return;
	}
	if (document.selection && document.selection.createRange){ //IE
		dgi("textInput").focus();
		var tr=document.selection.createRange();
		if(tr.text!="" && closeTag) tr.text='<'+tag+'>'+tr.text+'</'+tag+'>';
		if(!closeTag) tr.text=tag+tr.text;
		dgi("textInput").focus();
	}
}
function inputBlur(){}
function setBold(){format('b',true);}
function setItalic(){format('i',true);}
function setUnderline(){format('u',true);}
function setSmall(){format('small',true);}
function setBig(){format('big',true);}
function insertSmiley(s){format(' ' + s + ' ',false);}
function playsound(soundId){dgi(soundId).width=3-dgi(soundId).width;}
D.onkeypress=keyPress;
if(W.winMe) winMe.setCloseFunction("winOnClose");

var webcams={};
function setUserWebcamOnline(id,initialization){
	if(initialization && webcams[id]==1) return;
	bt='toggleWebcamBt_'+id;
	wl.button.enable(bt);
	wl.button.setIcon(bt,"<?php echo outIcon('webcam');?>");
	toggleWebcam(id,'on');
}

function setUserWebcamOffline(id){
	bt='toggleWebcamBt_'+id;
	wl.button.disable(bt);
	wl.button.setIcon(bt,"<?php echo outIcon('webcamNo');?>");
	toggleWebcam(id,'off');
}
function webcamSetSize(size){
	var e,webcams=D.getElementsByTagName('object');
	wl.asr.send({'webcamSetSize':size});
	switch(size){
		case 0:
			webcamWidth=160;webcamHeight=100;
			break;
		case 1:
			webcamWidth=320;webcamHeight=200;
			break;
		case 2:
			webcamWidth=480;webcamHeight=300;
	}
	dgi('usersDiv').style.width=(webcamWidth+10)+'px';
	winResized();
	for(e in webcams) {
		w=webcams[e];
		if(w.id && w.id.substr(0,12)=='webcamFlash_') {
			if(w.offsetWidth>1){
				w.width=webcamWidth;
				w.height=webcamHeight;

				if(w=dgi(w.id+'Embed')){
					w.width=webcamWidth;
					w.height=webcamHeight;
				}
			}
		}
	}
}

function toggleWebcam(id,force){
	var wfp=wl.getFlashPlayer('webcamFlash_'+id);

	if(force=='on' || (wfp.height==1 && force!='off')){
		wfp.style.display='';
		wfp.width=webcamWidth;
		wfp.height=webcamHeight;
		wl.button.setIcon('toggleWebcamBt_'+id,"<?php echo outIcon('webcamNo')?>");
		//wfp.setDebug();

		if(id==userId) {
			var resop=wfp.serverInit(id,"<?php echo $_SERVER['PHP_SELF']?>",<?php echo cfRGetVar('webcamFPS')?>,webcamWidth,webcamHeight);
			if(resop===false) {
				wfp.debug("<?php echo cfCaption('webcamError')?>");
			}
			else if(resop==undefined) setTimeout('toggleWebcam("'+id+'","on")');
			else {webcams[id]=1;}
		}
		else {
			webcams[id]=1;
			res=wfp.clientInit(id,"<?php echo $_SERVER['PHP_SELF']?>",<?php echo cfRGetVar('webcamFPS')?>);
		}

	}
	else{
		try{wfp.shutdown();} catch(err){}
		wfp.width=1;
		wfp.height=1;
		wfp.style.display='none';
		wl.button.setIcon('toggleWebcamBt_'+id,"<?php echo outIcon('webcam')?>");
		if(id==userId){
			dgn('command').value='webcamShutdown';
			asyncSubmitForm('updateForm',true);
			dgn('command').value='';
		}
	}
}
</script>
</head>

<body title="<?php echo cfCaption('genChat');?>" onorientationchange="updateOrientation()">
<form id="updateForm" enctype="multipart/form-data" method="POST" style="display:none">
<input name="message" type="text" />
<input name="command" type="text" />
<input name="typing" type="text" />
<input name="cnxSound" type="text" />
<input name="msgSound" type="text" />
<input name="noSt" type="text" value="true">
</form>
<div class="topDiv" style="height:97%;top:0">
<?php

/**
 * Messages Frame
 */
echo outDivFrame('frame1', false, 'height:100%'); ?>
<div id="chatDiv" class="frame2" style="position:absolute;left:1em;width:75%;height:80%;overflow:auto;">
<?php echo resWebcamChatMessages(false); ?>
</div>

<?php

/**
 * Users Frame
 */
echo '<div id="usersDiv" class="frame2 vBanner" style="position:absolute;right:1em;width:'.((cfRGetVar('webcam')?(cfRGetVar('webcamWidth')+10):'20%')).';height:80%;overflow:hidden">'."\n";
echo '<div class="frame2Header">'.cfCaption('adminConnectedUsers')."</div>\n";

// Webcams size selection controls
if(cfRGetVar('webcam')){
	$webcamSizeControls='';
	for ($i=0;$i<3;$i++) $webcamSizeControls.=outBlockItem('webcamSize',outImage(outIcon('efTNSize'.$i)),'title="'.cfCaptionJS('genWebcam').cfCaptionJS('genSeparator').$webcamSizes[$i][0].'x'.$webcamSizes[$i][1].'" id="sizeControl'.$i.'"',false,array('onclick'=>'webcamSetSize('.$i.')'),$i==(cfRGetVar('webcamSize'))?'sel':'unsel');
	echo '<div style="position:absolute;bottom:0;right:0">'.outImageIcon('webcam').$webcamSizeControls.'</div>';
}


// Display current user
echo resWebcamDisplayUser(cfRGetVar('chatUserId'))."\n";
echo '<br/>';
// Display other users
foreach ($users as $key=>$value) if($key!=cfRGetVar('chatUserId')) echo resWebcamDisplayUser($key)."\n";
echo "</div>\n\n";


/**
 * Input Frame
 */
echo '<div id="inputDiv" class="frame2" style="position:absolute;left:1em;bottom:1em;height:8em;overflow:hidden;">';
echo '<div style="white-space:nowrap;overflow:hidden;'.((cfGetBrowser()=='ie')?'width:100%':'').'">';
if(cfBGetVar('drag')){
	echo outButton('','javascript:setBold()',outIcon('bold'));
	echo outButton('','javascript:setUnderline()',outIcon('underline'));
	echo outButton('','javascript:setItalic()',outIcon('italic'));
	echo outButton('','javascript:setSmall()',outIcon('smallFont'));
	echo outButton('','javascript:setBig()',outIcon('bigFont'));
	echo '&nbsp;&nbsp;';
}
echo outButton('','javascript:insertSmiley(\':)\')',outIcon('smileys/smile'));
echo outButton('','javascript:insertSmiley(\':))\')',outIcon('smileys/smileBig'));
echo outButton('','javascript:insertSmiley(\';)\')',outIcon('smileys/blink'));
echo outButton('','javascript:insertSmiley(\':D\')',outIcon('smileys/laugh'));
echo outButton('','javascript:insertSmiley(\':(\')',outIcon('smileys/sad'));
echo outButton('','javascript:insertSmiley(\':\\\'(\')',outIcon('smileys/cry'));
echo outButton('','javascript:insertSmiley(\'8)\')',outIcon('smileys/glasses'));
echo "</div>\n";
// New user and new message flash sound player
echo '<object style="position:absolute;top:0px;left:0px;z-index:-999" id="chatMsgSound" type="application/x-shockwave-flash" data="/js/inlinePlayer.swf?file=/snd/'.cfUGetVar('audioProfile').'/chatMsg.mp3&amp;autoplay=0" width="1" height="1"><param name="movie" value="/js/inlinePlayer.swf?file=/snd/'.cfUGetVar('audioProfile').'/chatMsg.mp3&amp;autoplay=0&amp;isIE=1"></object>';
echo '<object style="position:absolute;top:0px;left:2px;z-index:-999" id="chatCnxSound" type="application/x-shockwave-flash" data="/js/inlinePlayer.swf?file=/snd/'.cfUGetVar('audioProfile').'/chatCnx.mp3&amp;autoplay=0" width="1" height="1"><param name="movie" value="/js/inlinePlayer.swf?file=/snd/'.cfUGetVar('audioProfile').'/chatCnx.mp3&amp;autoplay=0&amp;isIE=1"></object>';

/**
 * INPUT and send button
 */
if(cfRGetVar('textInputValidation')=='onchange'){// iPhone only so far...
	echo '<colgroup><col width="*"><col width="50px"></colgroup><tr><td>';
	echo '<input id="textInput" class="textInput" type="text" style="width:100%; color:'.$colors[cfRGetVar('color')].'" onchange="sendMessage(this.value)" autocorrect="off" autocapitalize="off" />';
	echo '</td><td>'.outButton(false,'javascript:sendMessage(dgi(\'textInput\').value)',outIcon('ok16'),cfCaption('genOK')).'</td></tr></table>';
}
else{
	echo outTableTransparent('frame2');
	echo '<colgroup><col width="*"><col width="50px"></colgroup><tr><td>';
	echo '<input id="textInput" class="textInput" type="text" style="width:100%; color:'.$colors[cfRGetVar('color')].'" />';
	echo '</td><td>'.outButton(false,'javascript:sendMessage(dgi(\'textInput\').value)',outIcon('ok16'),cfCaption('genOK')).'</td></tr></table>';
}

if(cfBGetVar('flash')){
	if(cfGetBrowser()!='ie') echo '<br/>';

	// New connection sound control
	echo '<div nowrap="nowrap" style="overflow:hidden;'.((cfGetBrowser()=='ie')?'width:100%':'').'">'.outImage(outIcon('user'),false,false,'margin-left:0em;margin-right:1em;vertical-align:middle');
	echo '<input onclick="playSoundOnConnection=dgn(\'cnxSoundInput\').checked" type="radio" name="cnxSoundInput" value="true" '.((cfRGetVar('browserCnxSound')=='true')?'checked="checked"':'').'>'.outImage(outIcon('soundOn'),false,false,'margin-right:1em;vertical-align:middle');
	echo '<input onclick="playSoundOnConnection=dgn(\'cnxSoundInput\').checked" type="radio" name="cnxSoundInput" value="false" '.((cfRGetVar('browserCnxSound')!='true')?'checked="checked"':'').'>'.outImage(outIcon('soundOff'),false,false,'margin-right:3em;vertical-align:middle');

	// New message sound control
	echo outImage(outIcon('message'),false,false,'margin-left:0em;margin-right:1em;vertical-align:middle');
	echo '<input onclick="playSoundOnMessage=dgn(\'msgSoundInput\').checked" type="radio" name="msgSoundInput" value="true" '.((cfRGetVar('browserMsgSound')=='true')?'checked="checked"':'').'>'.outImage(outIcon('soundOn'),false,false,'margin-right:1em;vertical-align:middle');
	echo '<input onclick="playSoundOnMessage=dgn(\'msgSoundInput\').checked" type="radio" name="msgSoundInput" value="false" '.((cfRGetVar('browserMsgSound')!='true')?'checked="checked"':'').'>'.outImage(outIcon('soundOff'),false,false,'vertical-align:middle');
	echo '</div>';
}
echo "</div>\n\n";
echo '</div>';
echo '</div>';

outKeysManager('dgi("chatDiv")');
?>
<script type="text/javascript">init()</script>
</body></html>